@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,29 +22,28 @@
  */
 
 #include "artistview.h"
-#include "localize.h"
-#include "covers.h"
-#include "utils.h"
-#include "musiclibrarymodel.h"
-#include "networkaccessmanager.h"
+#include "support/localize.h"
+#include "gui/covers.h"
+#include "support/utils.h"
+#include "models/musiclibrarymodel.h"
+#include "network/networkaccessmanager.h"
 #include "qtiocompressor/qtiocompressor.h"
-#include "textbrowser.h"
+#include "widgets/textbrowser.h"
 #include "contextengine.h"
-#include "actioncollection.h"
-#include "musiclibrarymodel.h"
-#include <QNetworkReply>
+#include "support/actioncollection.h"
+#include "models/musiclibrarymodel.h"
 #include <QApplication>
 #include <QTextStream>
 #include <QLayout>
 #if QT_VERSION >= 0x050000
 #include <QUrlQuery>
 #endif
-#include <QProcess>
 #include <QXmlStreamReader>
 #include <QPixmap>
 #include <QFile>
 #include <QMenu>
 #include <QTimer>
+#include <QDesktopServices>
 
 static const char *constNameKey="name";
 
@@ -87,10 +86,11 @@ ArtistView::ArtistView(QWidget *parent)
     connect(refreshAction, SIGNAL(triggered()), SLOT(refresh()));
     connect(engine, SIGNAL(searchResult(QString,QString)), this, SLOT(searchResponse(QString,QString)));
     connect(Covers::self(), SIGNAL(artistImage(Song,QImage,QString)), SLOT(artistImage(Song,QImage,QString)));
+    connect(Covers::self(), SIGNAL(coverUpdated(Song,QImage,QString)), SLOT(artistImageUpdated(Song,QImage,QString)));
     connect(text, SIGNAL(anchorClicked(QUrl)), SLOT(show(QUrl)));
     text->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(text, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showContextMenu(QPoint)));
-    setStandardHeader(i18n("Artist Information"));
+    setStandardHeader(i18n("Artist"));
    
     int imageHeight=fontMetrics().height()*14;
     int imageWidth=imageHeight*1.5;
@@ -106,11 +106,15 @@ ArtistView::ArtistView(QWidget *parent)
 
 void ArtistView::showContextMenu(const QPoint &pos)
 {
-   QMenu *menu = text->createStandardContextMenu();
-   menu->addSeparator();
-   menu->addAction(refreshAction);
-   menu->exec(text->mapToGlobal(pos));
-   delete menu;
+    QMenu *menu = text->createStandardContextMenu();
+    menu->addSeparator();
+    if (cancelJobAction->isEnabled()) {
+        menu->addAction(cancelJobAction);
+    } else {
+        menu->addAction(refreshAction);
+    }
+    menu->exec(text->mapToGlobal(pos));
+    delete menu;
 }
 
 void ArtistView::refresh()
@@ -131,6 +135,7 @@ void ArtistView::update(const Song &s, bool force)
         currentSong=s;
         engine->cancel();
         clear();
+        abort();
         return;
     }
 
@@ -140,6 +145,7 @@ void ArtistView::update(const Song &s, bool force)
 
     if (artistChanged) {
         artistAlbumsFirstTracks.clear();
+        abort();
     }
     if (!isVisible()) {
         if (artistChanged) {
@@ -160,11 +166,12 @@ void ArtistView::update(const Song &s, bool force)
             setHeader(currentSong.artist);
 
             Song s;
+            s.setArtistImageRequest();
             s.albumartist=currentSong.artist;
             if (!currentSong.isVariousArtists()) {
                 s.file=currentSong.file;
             }
-            Covers::Image img=Covers::self()->requestImage(s);
+            Covers::Image img=Covers::self()->requestImage(s, true);
             if (!img.img.isNull()) {
                 pic=createPicTag(img.img, img.fileName);
             }
@@ -183,7 +190,15 @@ const QList<Song> & ArtistView::getArtistAlbumsFirstTracks()
 
 void ArtistView::artistImage(const Song &song, const QImage &i, const QString &f)
 {
-    if (song.albumartist==currentSong.artist && pic.isEmpty()) {
+    if (song.isArtistImageRequest() && song.albumartist==currentSong.artist && pic.isEmpty()) {
+        pic=createPicTag(i, f);
+        setBio();
+    }
+}
+
+void ArtistView::artistImageUpdated(const Song &song, const QImage &i, const QString &f)
+{
+    if (song.isArtistImageRequest() && song.albumartist==currentSong.artist) {
         pic=createPicTag(i, f);
         setBio();
     }
@@ -242,12 +257,12 @@ void ArtistView::loadSimilar()
 
 void ArtistView::handleSimilarReply()
 {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    NetworkJob *reply = qobject_cast<NetworkJob*>(sender());
     if (!reply) {
         return;
     }
     if (reply==currentSimilarJob) {
-        if (QNetworkReply::NoError==reply->error()) {
+        if (reply->ok()) {
             QByteArray data=reply->readAll();
             QStringList artists=parseSimilarResponse(data);
             if (!artists.isEmpty()) {
@@ -292,7 +307,7 @@ void ArtistView::setBio()
             QStringList artistAlbums=c.value();
             qSort(artistAlbums);
             foreach (const QString &album, artistAlbums) {
-                albums+=QLatin1String("<li><a href=\"")+buildUrl(currentSong.albumArtist(), album)+"\">"+album+"</a></li>";
+                albums+=QLatin1String("<li><a href=\"")+buildUrl(currentSong.artist, album)+"\">"+album+"</a></li>";
             }
         }
         for (; it!=end; ++it) {
@@ -312,7 +327,7 @@ void ArtistView::setBio()
     }
 
     if (webLinks.isEmpty()) {
-        QFile file(":weblinks.xml");
+        QFile file(CANTATA_SYS_CONFIG_DIR+"weblinks.xml");
         if (file.open(QIODevice::ReadOnly)) {
             QXmlStreamReader reader(&file);
             while (!reader.atEnd()) {
@@ -331,7 +346,10 @@ void ArtistView::setBio()
     }
 
     if (!webLinks.isEmpty()) {
-        html+=View::subHeader(i18n("Web Links"))+QLatin1String("<ul>")+QString(webLinks).replace("${artist}", currentSong.artist)+QLatin1String("</ul>");
+        QString artist=currentSong.artist;
+        artist.replace(QLatin1Char('&'), QLatin1String("%26"));
+        artist.replace(QLatin1Char('?'), QLatin1String("%3f"));
+        html+=View::subHeader(i18n("Web Links"))+QLatin1String("<ul>")+QString(webLinks).replace("${artist}", artist)+QLatin1String("</ul>");
     }
 
     setHtml(html);
@@ -364,10 +382,10 @@ void ArtistView::abort()
 {
     engine->cancel();
     if (currentSimilarJob) {
-        disconnect(currentSimilarJob, SIGNAL(finished()), this, SLOT(handleSimilarReply()));
-        currentSimilarJob->abort();
+        currentSimilarJob->cancelAndDelete();
         currentSimilarJob=0;
     }
+    hideSpinner();
 }
 
 void ArtistView::searchResponse(const QString &resp, const QString &lang)
@@ -459,11 +477,7 @@ void ArtistView::show(const QUrl &url)
             }
         }
     } else {
-        #ifdef Q_OS_WIN
-        QProcess::startDetached(QLatin1String("cmd"), QStringList() << QLatin1String("/c") << QLatin1String("start") << url.toString());
-        #else
-        QProcess::startDetached(QLatin1String("xdg-open"), QStringList() << url.toString());
-        #endif
+        QDesktopServices::openUrl(url);
     }
 }
 
