@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /* This file is part of Clementine.
@@ -22,20 +22,28 @@
 */
 
 #include "cachesettings.h"
-#include "localize.h"
-#include "artistview.h"
-#include "albumview.h"
-#include "songview.h"
-#include "contextwidget.h"
+#include "support/localize.h"
+#include "context/artistview.h"
+#include "context/albumview.h"
+#include "context/songview.h"
+#include "context/contextwidget.h"
+#include "context/wikipediasettings.h"
 #include "covers.h"
-#include "musiclibrarymodel.h"
-#include "utils.h"
-#include "messagebox.h"
+#include "models/musiclibrarymodel.h"
+#include "support/utils.h"
+#include "support/messagebox.h"
 #include "config.h"
-#include "thread.h"
+#include "support/thread.h"
 #include "settings.h"
-#include "basicitemdelegate.h"
-#include "streamsmodel.h"
+#include "widgets/basicitemdelegate.h"
+#ifdef ENABLE_STREAMS
+#include "models/streamsmodel.h"
+#endif
+#ifdef ENABLE_ONLINE_SERVICES
+#include "online//podcastsearchdialog.h"
+#endif
+#include "support/squeezedtextlabel.h"
+#include "scrobbling/scrobbler.h"
 #include <QLabel>
 #include <QPushButton>
 #include <QStyle>
@@ -47,7 +55,7 @@
 
 static const int constMaxRecurseLevel=4;
 
-static void calculate(const QString &d, const QStringList &types, int &items, int &space, int level=0)
+static void calculate(const QString &d, const QStringList &types, int &items, quint64 &space, int level=0)
 {
     if (!d.isEmpty() && level<constMaxRecurseLevel) {
         QDir dir(d);
@@ -110,7 +118,7 @@ CacheItemCounter::~CacheItemCounter()
 void CacheItemCounter::getCount()
 {
     int items=0;
-    int space=0;
+    quint64 space=0;
     calculate(dir, types, items, space);
     emit count(items, space);
 }
@@ -125,10 +133,11 @@ CacheItem::CacheItem(const QString &title, const QString &d, const QStringList &
     : QTreeWidgetItem(p, QStringList() << title)
     , counter(new CacheItemCounter(title, d, t))
     , empty(true)
+    , usedSpace(0)
 {
     connect(this, SIGNAL(getCount()), counter, SLOT(getCount()), Qt::QueuedConnection);
     connect(this, SIGNAL(deleteAll()), counter, SLOT(deleteAll()), Qt::QueuedConnection);
-    connect(counter, SIGNAL(count(int, int)), this, SLOT(update(int, int)), Qt::QueuedConnection);
+    connect(counter, SIGNAL(count(int, quint64)), this, SLOT(update(int, quint64)), Qt::QueuedConnection);
     connect(this, SIGNAL(updated()), p, SIGNAL(itemSelectionChanged()));
 }
 
@@ -137,11 +146,12 @@ CacheItem::~CacheItem()
     delete counter;
 }
 
-void CacheItem::update(int itemCount, int space)
+void CacheItem::update(int itemCount, quint64 space)
 {
     setText(1, QString::number(itemCount));
     setText(2, Utils::formatByteSize(space));
     empty=0==itemCount;
+    usedSpace=space;
     setStatus();
     emit updated();
 }
@@ -171,17 +181,14 @@ void CacheItem::calculate()
     emit getCount();
 }
 
-#if QT_VERSION < 0x050000
 static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
 {
+    #if QT_VERSION < 0x050000
     hdr->setResizeMode(idx, mode);
-}
-#else
-static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
-{
+    #else
     hdr->setSectionResizeMode(idx, mode);
+    #endif
 }
-#endif
 
 CacheTree::CacheTree(QWidget *parent)
     : QTreeWidget(parent)
@@ -210,13 +217,29 @@ void CacheTree::showEvent(QShowEvent *e)
 {
     if (!calculated) {
         for (int i=0; i<topLevelItemCount(); ++i) {
-            ((CacheItem *)topLevelItem(i))->calculate();
+            static_cast<CacheItem *>(topLevelItem(i))->calculate();
         }
         calculated=true;
     }
     QTreeWidget::showEvent(e);
 }
 
+class SpaceLabel : public SqueezedTextLabel
+{
+public:
+    SpaceLabel(QWidget *p)
+        : SqueezedTextLabel(p)
+    {
+        QFont f=font();
+        f.setItalic(true);
+        setFont(f);
+        update(i18n("Calculating..."));
+    }
+
+    void update(int space) { update(Utils::formatByteSize(space)); }
+    void update(const QString &text) { setText(i18n("Total space used: %1", text)); }
+};
+
 CacheSettings::CacheSettings(QWidget *parent)
     : QWidget(parent)
 {
@@ -225,9 +248,10 @@ CacheSettings::CacheSettings(QWidget *parent)
     layout->setMargin(0);
     int row=0;
     int col=0;
-    QLabel *label=new QLabel(i18n("To speed up loading of the music library, Cantata caches a local copy of the MPD listing. Cantata might also have cached "
-                                  "covers, or lyrics, if these have been downloaded and could not be saved into the MPD folder (because Cantata cannot access it, "
-                                  "or you have configured Cantata to not save these items there). Below is a summary of Cantata's cache usage."), this);
+    QLabel *label=new QLabel(i18n("Cantata caches a local copy of the music listing to speed up loading of the library. Cantata might also have cached "
+                                  "covers, lyrics, or artist images, if these have been downloaded and could not be saved into the music folder (because "
+                                  "Cantata cannot access it, or you have configured Cantata to not save these items there). Below is a summary of Cantata's "
+                                  "current cache usage."), this);
     label->setWordWrap(true);
     layout->addWidget(label, row++, col, 1, 2);
     layout->addItem(new QSpacerItem(spacing, spacing, QSizePolicy::Fixed, QSizePolicy::Fixed), row++, 0);
@@ -238,18 +262,32 @@ CacheSettings::CacheSettings(QWidget *parent)
     new CacheItem(i18n("Music Library"), Utils::cacheDir(MusicLibraryModel::constLibraryCache, false),
                   QStringList() << "*"+MusicLibraryModel::constLibraryExt << "*"+MusicLibraryModel::constLibraryCompressedExt, tree);
     new CacheItem(i18n("Covers"), Utils::cacheDir(Covers::constCoverDir, false), QStringList() << "*.jpg" << "*.png", tree);
+    new CacheItem(i18n("Scaled Covers"), Utils::cacheDir(Covers::constScaledCoverDir, false), QStringList() << "*.jpg" << "*.png", tree);
     new CacheItem(i18n("Backdrops"), Utils::cacheDir(ContextWidget::constCacheDir, false), QStringList() << "*.jpg" << "*.png", tree);
     new CacheItem(i18n("Lyrics"), Utils::cacheDir(SongView::constLyricsDir, false), QStringList() << "*"+SongView::constExtension, tree);
     new CacheItem(i18n("Artist Information"), Utils::cacheDir(ArtistView::constCacheDir, false), QStringList() << "*"+ArtistView::constInfoExt
                   << "*"+ArtistView::constSimilarInfoExt << "*.json.gz" << "*.jpg" << "*.png", tree);
     new CacheItem(i18n("Album Information"), Utils::cacheDir(AlbumView::constCacheDir, false), QStringList() << "*"+AlbumView::constInfoExt << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Streams"), Utils::cacheDir(StreamsModel::constCacheDir, false), QStringList() << "*"+StreamsModel::constCacheExt, tree);
+    new CacheItem(i18n("Track Information"), Utils::cacheDir(SongView::constCacheDir, false), QStringList() << "*"+AlbumView::constInfoExt, tree);
+    #ifdef ENABLE_STREAMS
+    new CacheItem(i18n("Stream Listings"), Utils::cacheDir(StreamsModel::constSubDir, false), QStringList() << "*"+StreamsModel::constCacheExt, tree);
+    #endif
+    #ifdef ENABLE_ONLINE_SERVICES
     new CacheItem(i18n("Jamendo"), Utils::cacheDir("jamendo", false), QStringList() << "*"+MusicLibraryModel::constLibraryCompressedExt << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Magnatune"), Utils::cacheDir("magnatune", false), QStringList() << "*"+MusicLibraryModel::constLibraryCompressedExt<< "*.jpg" << "*.png", tree);
+    new CacheItem(i18n("Magnatune"), Utils::cacheDir("magnatune", false), QStringList() << "*"+MusicLibraryModel::constLibraryCompressedExt << "*.jpg" << "*.png", tree);
+    new CacheItem(i18n("Podcast Directories"), Utils::cacheDir(PodcastSearchDialog::constCacheDir, false), QStringList() << "*"+PodcastSearchDialog::constExt, tree);
+    #endif
+    new CacheItem(i18n("Wikipedia Languages"), Utils::cacheDir(WikipediaSettings::constSubDir, false), QStringList() << "*.xml.gz", tree);
+    new CacheItem(i18n("Scrobble Tracks"), Utils::cacheDir(Scrobbler::constCacheDir, false), QStringList() << "*.xml.gz", tree);
 
+    for (int i=0; i<tree->topLevelItemCount(); ++i) {
+        connect(static_cast<CacheItem *>(tree->topLevelItem(i)), SIGNAL(updated()), this, SLOT(updateSpace()));
+    }
+
+    spaceLabel=new SpaceLabel(this);
     button=new QPushButton(i18n("Delete All"), this);
-    layout->addItem(new QSpacerItem(0, 0, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed), row++, 0);
-    layout->addWidget(button, row, 1, 1, 1);
+    layout->addWidget(spaceLabel, row, 0, 1, 1);
+    layout->addWidget(button, row++, 1, 1, 1);
     button->setEnabled(false);
 
     connect(tree, SIGNAL(itemSelectionChanged()), this, SLOT(controlButton()));
@@ -264,7 +302,7 @@ void CacheSettings::controlButton()
 {
     button->setEnabled(false);
     for (int i=0; i<tree->topLevelItemCount(); ++i) {
-        CacheItem *item=(CacheItem *)tree->topLevelItem(i);
+        CacheItem *item=static_cast<CacheItem *>(tree->topLevelItem(i));
 
         if (item->isSelected() && !item->isEmpty()) {
             button->setEnabled(true);
@@ -277,7 +315,7 @@ void CacheSettings::deleteAll()
 {
     QList<CacheItem *> toDelete;
     for (int i=0; i<tree->topLevelItemCount(); ++i) {
-        CacheItem *item=(CacheItem *)tree->topLevelItem(i);
+        CacheItem *item=static_cast<CacheItem *>(tree->topLevelItem(i));
 
         if (item->isSelected() && !item->isEmpty()) {
             toDelete.append(item);
@@ -307,3 +345,12 @@ void CacheSettings::deleteAll()
         }
     }
 }
+
+void CacheSettings::updateSpace()
+{
+    quint64 space=0;
+    for (int i=0; i<tree->topLevelItemCount(); ++i) {
+        space+=static_cast<CacheItem *>(tree->topLevelItem(i))->spaceUsed();
+    }
+    spaceLabel->update(space);
+}
