@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,16 +28,16 @@
 #include <QHash>
 #include <QSet>
 #include <QMap>
-#include <QCache>
 #include <QImage>
 #include <QPixmap>
 #include <QMutex>
-#include "song.h"
+#include <QCache>
+#include "mpd/song.h"
 #include "config.h"
 
 class QString;
 class Thread;
-class QNetworkReply;
+class NetworkJob;
 class QMutex;
 class QTimer;
 #ifdef ENABLE_KDE_SUPPORT
@@ -53,16 +53,15 @@ public:
     enum JobType {
         JobHttpJpg,
         JobHttpPng,
-        JobLastFm,
-
-        JobOnline
+        JobLastFm
     };
 
     struct Job
     {
         Job(const Song &s, const QString &d, bool a=false)
-            : song(s), dir(d), isArtist(a), type(JobLastFm), level(0) { }
+            : song(s), filePath(s.filePath()), dir(d), isArtist(a), type(JobLastFm), level(0) { }
         Song song;
+        QString filePath;
         QString dir;
         bool isArtist;
         JobType type;
@@ -82,7 +81,6 @@ Q_SIGNALS:
     void artistImage(const Song &song, const QImage &img, const QString &file);
 
 private:
-    void downloadOnlineImage(Job &job);
     bool downloadViaHttp(Job &job, JobType type);
     void downloadViaLastFm(Job &job);
 
@@ -91,11 +89,13 @@ private Q_SLOTS:
     void jobFinished();
 
 private:
+    void failed(const Job &job);
     QString saveImg(const Job &job, const QImage &img, const QByteArray &raw);
-    QHash<QNetworkReply *, Job>::Iterator findJob(const Job &job);
+    QHash<NetworkJob *, Job>::Iterator findJob(const Job &job);
+    NetworkAccessManager * network();
 
 private:
-    QHash<QNetworkReply *, Job> jobs;
+    QHash<NetworkJob *, Job> jobs;
 
 private:
     Thread *thread;
@@ -136,6 +136,39 @@ private:
     QList<Song> queue;
 };
 
+struct LoadedCover
+{
+    LoadedCover(const Song &sng=Song(), const QImage &i=QImage())
+        : song(sng), img(i) { }
+    Song song;
+    QImage img;
+};
+
+class CoverLoader : public QObject
+{
+    Q_OBJECT
+public:
+    CoverLoader();
+    ~CoverLoader() { }
+
+    void stop();
+
+Q_SIGNALS:
+    void loaded(const QList<LoadedCover> &covers);
+
+public Q_SLOTS:
+    void load(const Song &song);
+    void load();
+
+private:
+    void startTimer(int interval);
+
+private:
+    Thread *thread;
+    QTimer *timer;
+    QList<LoadedCover> queue;
+};
+
 class Covers : public QObject
 {
     Q_OBJECT
@@ -153,16 +186,18 @@ public:
     };
 
     static void enableDebug();
+    static bool debugEnabled();
 
     static const QSize constMaxSize;
     static const QLatin1String constLastFmApiKey;
     static const QLatin1String constCoverDir;
+    static const QLatin1String constScaledCoverDir;
     static const QLatin1String constFileName;
     static const QLatin1String constCddaCoverDir;
     static const QLatin1String constArtistImage;
 
     static Covers * self();
-    static bool isCoverFile(const QString &file);
+//    static bool isCoverFile(const QString &file);
     static bool copyImage(const QString &sourceDir, const QString &destDir, const QString &coverFile, const QString &destName, unsigned short maxSize=0);
     static bool copyCover(const Song &song, const QString &sourceDir, const QString &destDir, const QString &name=QString(), unsigned short maxSize=0);
     static const QStringList &standardNames();
@@ -170,10 +205,20 @@ public:
     static QString albumFileName(const Song &song);
     static QString artistFileName(const Song &song);
     static QString fixArtist(const QString &artist);
+    static bool isJpg(const QByteArray &data);
+    static bool isPng(const QByteArray &data);
+    static const char * imageFormat(const QByteArray &data);
 
     Covers();
+    #ifdef ENABLE_UBUNTU
+    void setFetchCovers(bool f);
+    #else
+    void readConfig();
+    #endif
     void stop();
 
+    QPixmap * getScaledCover(const Song &song, int size);
+    QPixmap * saveScaledCover(const QImage &img, const Song &song, int size);
     // Get cover image of specified size. If this is not found 0 will be returned, and the cover
     // will be downloaded.
     QPixmap * get(const Song &song, int size);
@@ -184,8 +229,7 @@ public:
     // Get QImage and filename associated with Song request. If this is not found, then the cover
     // will be downloaded. If more than 5 covers have been requested in an event-loop iteration, then
     // the cover requests are placed on a queue.
-    Image requestImage(const Song &song);
-    void setSaveInMpdDir(bool s);
+    Image requestImage(const Song &song, bool urgent=false);
     void emitCoverUpdated(const Song &song, const QImage &img, const QString &file);
 
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
@@ -197,34 +241,39 @@ public:
 Q_SIGNALS:
     void download(const Song &s);
     void locate(const Song &s);
+    void load(const Song &song);
+    void loaded(const Song &song, int s);
     void cover(const Song &song, const QImage &img, const QString &file);
     void coverUpdated(const Song &song, const QImage &img, const QString &file);
     void artistImage(const Song &song, const QImage &img, const QString &file);
     void coverRetrieved(const Song &song);
 
 private Q_SLOTS:
-    void clearCount();
     void located(const QList<LocatedCover> &covers);
+    void loaded(const QList<LoadedCover> &covers);
     void coverDownloaded(const Song &song, const QImage &img, const QString &file);
     void artistImageDownloaded(const Song &song, const QImage &img, const QString &file);
 
 private:
+    void tryToLocate(const Song &song);
+    void tryToDownload(const Song &song);
+    void tryToLoad(const Song &song);
     Image findImage(const Song &song, bool emitResult);
-    void clearCache(const Song &song, const QImage &img, bool dummyEntriesOnly);
+    void updateCache(const Song &song, const QImage &img, bool dummyEntriesOnly);
     void gotAlbumCover(const Song &song, const QImage &img, const QString &fileName, bool emitResult=true);
     void gotArtistImage(const Song &song, const QImage &img, const QString &fileName, bool emitResult=true);
     QString getFilename(const Song &s, bool isArtist);
 
 private:
-    int retrieved;
+    QSet<QString> currentImageRequests;
     QList<Song> queue;
     QSet<int> cacheSizes;
-    QCache<quint32, QPixmap> cache;
+    QCache<QString, QPixmap> cache;
     QMap<QString, QString> filenames;
     CoverDownloader *downloader;
     CoverLocator *locator;
+    CoverLoader *loader;
     QMutex mutex;
-    QTimer *countResetTimer;
 };
 
 #endif
