@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,7 +25,7 @@
 #define INITIALSETTINGSWIZARD_H
 
 #include "ui_initialsettingswizard.h"
-#include "mpdconnection.h"
+#include "mpd/mpdconnection.h"
 #include <QWizard>
 
 class InitialSettingsWizard : public QWizard, public Ui::InitialSettingsWizard
