@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -30,55 +30,55 @@
 #include <qglobal.h>
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KXmlGuiWindow>
+#define MAIN_WINDOW_BASE_CLASS KXmlGuiWindow
+class KToggleAction;
 #else
 #include <QMainWindow>
+#define MAIN_WINDOW_BASE_CLASS QMainWindow
 #endif
-#include <QMenu>
-#include <QProxyStyle>
-#include <QPixmap>
-#include <QMoveEvent>
 #include <QToolButton>
 #include <QStringList>
 #include "ui_mainwindow.h"
-#include "playqueuemodel.h"
-#include "playqueueproxymodel.h"
-#include "mpdstatus.h"
-#include "mpdconnection.h"
-#include "song.h"
+#include "models/playqueuemodel.h"
+#include "models/playqueueproxymodel.h"
+#include "mpd/song.h"
 #include "config.h"
-#ifdef ENABLE_HTTP_STREAM_PLAYBACK
-#if QT_VERSION < 0x050000
-#include <phonon/mediaobject.h>
-#else
-#include <QtMultimedia/QMediaPlayer>
-#endif
-#endif
 
 class Action;
 class ActionCollection;
 class MainWindow;
+class Page;
 class LibraryPage;
 class AlbumsPage;
 class FolderPage;
 class PlaylistsPage;
+#ifdef ENABLE_DYNAMIC
 class DynamicPage;
+#endif
+#ifdef ENABLE_STREAMS
 class StreamsPage;
+#endif
+#ifdef ENABLE_ONLINE_SERVICES
 class OnlineServicesPage;
+#endif
 #ifdef ENABLE_DEVICES_SUPPORT
 class DevicesPage;
 #endif
+class SearchPage;
 class QAbstractItemView;
-#ifndef Q_OS_WIN
+#ifdef QT_QTDBUS_FOUND
 class Mpris;
-class GnomeMediaKeys;
-#endif // !defined Q_OS_WIN
+#endif
 class QTimer;
 class QPropertyAnimation;
 class QActionGroup;
 class QDateTime;
-class VolumeControl;
+class QMenu;
 class TrayItem;
-class GtkProxyStyle;
+class HttpStream;
+class MPDStatus;
+struct MPDConnectionDetails;
+struct Output;
 
 // Dummy classes so that when class name is saved to the config file, we get a more meaningful name than QWidget!!!
 class PlayQueuePage : public QWidget
@@ -94,90 +94,55 @@ public:
     ContextPage(QWidget *p) : QWidget(p) { }
 };
 
-class DeleteKeyEventHandler : public QObject
-{
-public:
-    DeleteKeyEventHandler(QAbstractItemView *v, QAction *a) : QObject(v), view(v), act(a) { }
-protected:
-    bool eventFilter(QObject *obj, QEvent *event);
-private:
-    QAbstractItemView *view;
-    QAction *act;
-};
-
-class VolumeSliderEventHandler : public QObject
-{
-public:
-    VolumeSliderEventHandler(MainWindow *w) : QObject((QObject *)w), window(w) { }
-protected:
-    bool eventFilter(QObject *obj, QEvent *event);
-protected:
-    MainWindow * const window;
-};
-
-class VolumeButtonEventHandler : public VolumeSliderEventHandler
-{
-public:
-    VolumeButtonEventHandler(MainWindow *w) : VolumeSliderEventHandler(w), down(false) { }
-protected:
-    bool eventFilter(QObject *obj, QEvent *event);
-private:
-    bool down;
-};
-
-#ifdef ENABLE_KDE_SUPPORT
-class MainWindow : public KXmlGuiWindow, private Ui::MainWindow
-#else
-class MainWindow : public QMainWindow, private Ui::MainWindow
-#endif
+class MainWindow : public MAIN_WINDOW_BASE_CLASS, private Ui::MainWindow
 {
     Q_OBJECT
 
 public:
-    enum Page
+    enum Pages
     {
         PAGE_PLAYQUEUE,
         PAGE_LIBRARY,
         PAGE_ALBUMS,
         PAGE_FOLDERS,
         PAGE_PLAYLISTS,
+        #ifdef ENABLE_DYNAMIC
         PAGE_DYNAMIC,
+        #endif
+        #ifdef ENABLE_STREAMS
         PAGE_STREAMS,
+        #endif
+        #ifdef ENABLE_ONLINE_SERVICES
         PAGE_ONLINE,
+        #endif
         #ifdef ENABLE_DEVICES_SUPPORT
         PAGE_DEVICES,
         #endif
+        PAGE_SEARCH,
         PAGE_CONTEXT
     };
 
-    Q_PROPERTY(int volume READ mpdVolume WRITE setMpdVolume)
-
     MainWindow(QWidget *parent = 0);
     ~MainWindow();
 
-    int mpdVolume() const { return volume; }
-    int currentTrackPosition() const;
-    QString coverFile() const;
+    int mpdVolume() const { return volumeSlider->value(); }
 
 protected:
     void keyPressEvent(QKeyEvent *event);
     void closeEvent(QCloseEvent *event);
 
 private:
+    void addMenuAction(QMenu *menu, QAction *action);
     void initSizes();
     void setupTrayIcon();
 
 Q_SIGNALS:
     // These are for communicating with MPD object (which is in its own thread, so need to talk via signal/slots)
     void setDetails(const MPDConnectionDetails &det);
-    void removeSongs(const QList<qint32> &);
     void pause(bool p);
     void play();
     void stop(bool afterCurrent=false);
     void getStatus();
-    void getStats(bool andUpdate);
-    void updateMpd();
-    void clear();
     void playListInfo();
     void currentSong();
     void setSeekId(qint32, quint32);
@@ -187,66 +152,64 @@ Q_SIGNALS:
     void enableOutput(int id, bool);
     void setPriority(const QList<qint32> &ids, quint8 priority);
     void addSongsToPlaylist(const QString &name, const QStringList &files);
-    void addAndPlay(const QString &file);
+    void showPreferencesPage(const QString &page);
 
 public Q_SLOTS:
     void showError(const QString &message, bool showActions=false);
     void showInformation(const QString &message);
-    void showPage(const QString &page, bool showSearch);
     void dynamicStatus(const QString &message);
     void hideWindow();
     void restoreWindow();
-    void load(const QStringList &urls);
+    void load(const QStringList &urls) { playQueueModel.load(urls); }
     #ifdef ENABLE_KDE_SUPPORT
     void configureShortcuts();
     void saveShortcuts();
+    #else
+    void showAboutDialog();
     #endif
-    void setMpdVolume(int );
-    void songLoaded();
-    void messageWidgetVisibility(bool v);
     void mpdConnectionStateChanged(bool connected);
     void playQueueItemsSelected(bool s);
-    void showPreferencesDialog();
+    void showSidebarPreferencesPage() { showPreferencesDialog("interface:sidebar"); }
+    void showPreferencesDialog(const QString &page=QString());
     void quit();
     void updateSettings();
     void toggleOutput();
     void changeConnection();
     void connectToMpd();
     void connectToMpd(const MPDConnectionDetails &details);
-    void refresh();
-    #ifndef ENABLE_KDE_SUPPORT
-    void showAboutDialog();
-    #endif
+    void streamUrl(const QString &u);
+    void refreshDbPromp();
     void showServerInfo();
-    void toggleStream(bool s);
     void stopPlayback();
     void stopAfterCurrentTrack();
     void stopAfterTrack();
     void playPauseTrack();
-    void nextTrack();
-    void prevTrack();
     void setPosition();
     void searchPlayQueue();
     void realSearchPlayQueue();
+    void playQueueSearchActivated(bool a);
     void updatePlayQueue(const QList<Song> &songs);
-    void updateCurrentSong(const Song &song);
-    void scrollPlayQueue();
+    void updateCurrentSong(const Song &song, bool wasEmpty=false);
+    void scrollPlayQueue(bool wasEmpty=false);
     void updateStats();
     void updateStatus();
     void playQueueItemActivated(const QModelIndex &);
-    void removeFromPlayQueue();
-    void replacePlayQueue();
-    void addToPlayQueue();
+    void promptClearPlayQueue();
+    void clearPlayQueue();
+    void centerPlayQueue();
+    void removeFromPlayQueue() { playQueueModel.remove(playQueueProxyModel.mapToSourceRows(playQueue->selectedIndexes())); }
+    void replacePlayQueue() { addToPlayQueue(true); }
+    void addToPlayQueue() { addToPlayQueue(false); }
+    void addRandomToPlayQueue() { addToPlayQueue(false, 0, true); }
     void addWithPriority();
     void addToNewStoredPlaylist();
-    void addToExistingStoredPlaylist(const QString &name);
+    void addToExistingStoredPlaylist(const QString &name) { addToExistingStoredPlaylist(name, playQueue->hasFocus()); }
     void addToExistingStoredPlaylist(const QString &name, bool pq);
     void addStreamToPlayQueue();
     void removeItems();
     void checkMpdAccessibility();
-    void cropPlayQueue();
+    void cropPlayQueue() { playQueueModel.crop(playQueueProxyModel.mapToSourceRows(playQueue->selectedIndexes())); }
     void updatePlayQueueStats(int songs, quint32 time);
-    void copyTrackInfo();
     void expandOrCollapse(bool saveCurrentSize=true);
     void showSongInfo();
     void fullScreen();
@@ -258,21 +221,34 @@ public Q_SLOTS:
     void showAlbumsTab() { showTab(PAGE_ALBUMS); }
     void showFoldersTab() { showTab(PAGE_FOLDERS); }
     void showPlaylistsTab() { showTab(PAGE_PLAYLISTS); }
-    void showDynamicTab() { showTab(PAGE_DYNAMIC); }
-    void showStreamsTab() { showTab(PAGE_STREAMS); }
-    void showOnlineTab() { showTab(PAGE_ONLINE); }
+    void showDynamicTab() {
+        #ifdef ENABLE_DYNAMIC
+        showTab(PAGE_DYNAMIC);
+        #endif
+    }
+    void showStreamsTab() {
+        #ifdef ENABLE_STREAMS
+        showTab(PAGE_STREAMS);
+        #endif
+    }
+    void showOnlineTab() {
+        #ifdef ENABLE_ONLINE_SERVICES
+        showTab(PAGE_ONLINE); 
+        #endif
+    }
     void showContextTab() { showTab(PAGE_CONTEXT); }
     void showDevicesTab() {
         #ifdef ENABLE_DEVICES_SUPPORT
         showTab(PAGE_DEVICES);
         #endif
     }
+    void showSearchTab() { showTab(PAGE_SEARCH); }
     void toggleSplitterAutoHide();
     void toggleMonoIcons();
+    void locateTracks(const QList<Song> &songs);
     void locateTrack();
     void locateArtist(const QString &artist);
     void locateAlbum(const QString &artist, const QString &album);
-    void playSong(const QString &song);
     void editTags();
     void editPlayQueueTags();
     void organiseFiles();
@@ -282,7 +258,6 @@ public Q_SLOTS:
     void deleteSongs(const QString &from, const QList<Song> &songs);
     void replayGain();
     void setCover();
-    void goBack();
     void showSearch();
     void expandAll();
     void collapseAll();
@@ -293,56 +268,57 @@ public Q_SLOTS:
     void controlDynamicButton();
 
 private:
+    void expand();
     bool canShowDialog();
     void enableStopActions(bool enable);
     void updateStatus(MPDStatus * const status);
     void readSettings();
-    int calcMinHeight();
-    int calcCompactHeight();
-    void addToPlayQueue(bool replace, quint8 priority=0);
+    void addToPlayQueue(bool replace, quint8 priority=0, bool randomAlbums=false);
     #ifdef TAGLIB_FOUND
     void editTags(const QList<Song> &songs, bool isPlayQueue);
     #endif
-    bool currentIsStream() const;
+    bool currentIsStream() const { return playQueueModel.rowCount() && -1!=current.id && current.isStream(); }
     void updateWindowTitle();
-    void startVolumeFade(/*bool stop*/);
-    void stopVolumeFade();
-    void showTab(int page);
-    bool fadeWhenStop() const;
+    void showTab(int page) { tabWidget->setCurrentIndex(page); }
     void updateNextTrack(int nextTrackId);
+    void updateActionToolTips();
+    void setPlaylistsEnabled(bool e);
+    void controlPlaylistActions();
+    void startContextTimer();
+    int calcMinHeight();
+    int calcCollapsedSize();
+    void setCollapsedSize();
+
+private Q_SLOTS:
+    void toggleContext();
+    void toggleMenubar();
 
 private:
-    int loaded;
+    int prevPage;
     MPDState lastState;
     qint32 lastSongId;
     PlayQueueModel playQueueModel;
     PlayQueueProxyModel playQueueProxyModel;
     bool autoScrollPlayQueue;
-    VolumeSliderEventHandler *volumeSliderEventHandler;
-    VolumeControl *volumeControl;
-    Action *prefAction;
     #ifdef ENABLE_KDE_SUPPORT
+    KToggleAction *showMenuAction;
     Action *shortcutsAction;
+    #else
+    Action *showMenuAction;
     #endif
+    Action *prefAction;
+    Action *refreshDbAction;
+    Action *doDbRefreshAction;
     Action *connectAction;
     Action *connectionsAction;
     Action *outputsAction;
     QActionGroup *connectionsGroup;
-    Action *prevTrackAction;
-    Action *nextTrackAction;
-    Action *playPauseTrackAction;
-    Action *stopPlaybackAction;
-    Action *stopAfterCurrentTrackAction;
     Action *stopAfterTrackAction;
-    Action *increaseVolumeAction;
-    Action *decreaseVolumeAction;
-    Action *muteAction;
-    Action *removeFromPlayQueueAction;
     Action *addPlayQueueToStoredPlaylistAction;
-    Action *clearPlayQueueAction;
-    Action *copyTrackInfoAction;
+    Action *promptClearPlayQueueAction;
+    Action *centerPlayQueueAction;
+    Action *expandInterfaceAction;
     Action *cropPlayQueueAction;
-    Action *shufflePlayQueueAction;
     Action *addStreamToPlayQueueAction;
     Action *randomPlayQueueAction;
     Action *repeatPlayQueueAction;
@@ -351,94 +327,70 @@ private:
     Action *searchPlayQueueAction;
     Action *setPriorityAction;
     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+    HttpStream *httpStream;
     Action *streamPlayAction;
     #endif
-    Action *expandInterfaceAction;
     Action *songInfoAction;
     Action *fullScreenAction;
     Action *quitAction;
     Action *restoreAction;
     Action *locateTrackAction;
-    Action *showPlayQueueAction;
-    Action *libraryTabAction;
-    Action *albumsTabAction;
-    Action *foldersTabAction;
-    Action *playlistsTabAction;
-    Action *dynamicTabAction;
-    Action *streamsTabAction;
-    Action *onlineTabAction;
     #ifdef TAGLIB_FOUND
     Action *editPlayQueueTagsAction;
     #endif
-    #ifdef ENABLE_DEVICES_SUPPORT
-    Action *devicesTabAction;
-    #endif
+    Action *searchTabAction;
     Action *expandAllAction;
     Action *collapseAllAction;
     Action *serverInfoAction;
-    QAction *autoHideSplitterAction;
-    QAction *monoIconsAction;
+    Action *clearPlayQueueAction;
+    Action *cancelAction;
+    Action *clearNewStateAction;
     TrayItem *trayItem;
     QPoint lastPos;
     QSize expandedSize;
     QSize collapsedSize;
     Song current;
+    Page *currentPage;
+    Action *showPlayQueueAction;
     QWidget *playQueuePage;
+    Action *libraryTabAction;
     LibraryPage *libraryPage;
+    Action *albumsTabAction;
     AlbumsPage *albumsPage;
+    Action *foldersTabAction;
     FolderPage *folderPage;
+    Action *playlistsTabAction;
     PlaylistsPage *playlistsPage;
+    #ifdef ENABLE_DYNAMIC
+    Action *dynamicTabAction;
     DynamicPage *dynamicPage;
+    #endif
+    #ifdef ENABLE_STREAMS
+    Action *streamsTabAction;
     StreamsPage *streamsPage;
+    #endif
+    #ifdef ENABLE_ONLINE_SERVICES
+    Action *onlineTabAction;
     OnlineServicesPage *onlinePage;
+    #endif
     QWidget *contextPage;
     #ifdef ENABLE_DEVICES_SUPPORT
+    Action *devicesTabAction;
     DevicesPage *devicesPage;
     #endif
-    #ifndef Q_OS_WIN
+    SearchPage *searchPage;
+    #ifdef QT_QTDBUS_FOUND
     Mpris *mpris;
-    GnomeMediaKeys *gnomeMediaKeys;
-    GtkProxyStyle *gtkStyle;
-    #endif // Q_OS_WIN
+    #endif
     QTimer *statusTimer;
     QTimer *playQueueSearchTimer;
-    #ifdef Q_OS_LINUX
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC
     QTimer *mpdAccessibilityTimer;
     #endif
-
-    enum ConnState {
-        CS_Init,
-        CS_Connected,
-        CS_Disconnected
-    };
-
-    ConnState connectedState;
-
-    enum StopState {
-        StopState_None     = 0,
-        StopState_Stopping = 1
-//         StopState_Pausing  = 2
-    };
-
-    bool fadeStop;
+    QTimer *contextTimer;
+    int contextSwitchTime;
+    enum { CS_Init, CS_Connected, CS_Disconnected } connectedState;
     bool stopAfterCurrent;
-    QPropertyAnimation *volumeFade;
-    int volume;
-    int origVolume;
-    int lastVolume;
-    StopState stopState;
-    #ifdef ENABLE_HTTP_STREAM_PLAYBACK
-    bool httpStreamEnabled;
-    #if QT_VERSION < 0x050000
-    Phonon::MediaObject *httpStream;
-    #else
-    QMediaPlayer *httpStream;
-    #endif
-    #endif
-
-    friend class VolumeSliderEventHandler;
-    friend class VolumeButtonEventHandler;
-    friend class CoverEventHandler;
     friend class TrayItem;
 };
 
