@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,27 +28,27 @@
 #include "musiclibrarymodel.h"
 #include "devicesmodel.h"
 #include "playqueuemodel.h"
-#include "settings.h"
-#include "itemview.h"
-#include "mpdparseutils.h"
-#include "mpdconnection.h"
-#include "umsdevice.h"
-#include "httpserver.h"
-#include "localize.h"
-#include "icons.h"
-#include "mountpoints.h"
-#include "stdactions.h"
-#include "actioncollection.h"
+#include "gui/settings.h"
+#include "roles.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdconnection.h"
+#include "devices/umsdevice.h"
+#include "http/httpserver.h"
+#include "support/localize.h"
+#include "widgets/icons.h"
+#include "devices/mountpoints.h"
+#include "gui/stdactions.h"
+#include "support/actioncollection.h"
+#include "config.h"
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
-#include "audiocddevice.h"
+#include "devices/audiocddevice.h"
 #endif
+#include "support/globalstatic.h"
 #include <QMenu>
 #include <QStringList>
 #include <QMimeData>
 #include <QTimer>
 #ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KIcon>
-#include <KDE/KGlobal>
 #include <solid/device.h>
 #include <solid/deviceinterface.h>
 #include <solid/devicenotifier.h>
@@ -57,7 +57,6 @@
 #include <solid/storagedrive.h>
 #include <solid/storagevolume.h>
 #include <solid/opticaldisc.h>
-K_GLOBAL_STATIC(DevicesModel, instance)
 #else
 #include "solid-lite/device.h"
 #include "solid-lite/deviceinterface.h"
@@ -68,6 +67,9 @@ K_GLOBAL_STATIC(DevicesModel, instance)
 #include "solid-lite/storagevolume.h"
 #include "solid-lite/opticaldisc.h"
 #endif
+#if defined ENABLE_MODEL_TEST
+#include "modeltest.h"
+#endif
 
 #include <QDebug>
 #define DBUG qDebug()
@@ -86,18 +88,7 @@ QString DevicesModel::fixDevicePath(const QString &path)
     return path;
 }
 
-DevicesModel * DevicesModel::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static DevicesModel *instance=0;
-    if(!instance) {
-        instance=new DevicesModel;
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(DevicesModel, instance)
 
 DevicesModel::DevicesModel(QObject *parent)
     : MultiMusicModel(parent)
@@ -114,6 +105,9 @@ DevicesModel::DevicesModel(QObject *parent)
     #endif
     updateItemMenu();
     connect(this, SIGNAL(add(const QStringList &, bool, quint8)), MPDConnection::self(), SLOT(add(const QStringList &, bool, quint8)));
+    #if defined ENABLE_MODEL_TEST
+    new ModelTest(this, this);
+    #endif
 }
 
 DevicesModel::~DevicesModel()
@@ -129,7 +123,15 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
     MusicLibraryItem *item = static_cast<MusicLibraryItem *>(index.internalPointer());
 
     switch (role) {
-    case ItemView::Role_SubText:
+    case Qt::DisplayRole:
+        if (MusicLibraryItem::Type_Song==item->itemType()) {
+            MusicLibraryItemSong *song = static_cast<MusicLibraryItemSong *>(item);
+            if (MusicLibraryItem::Type_Root==song->parentItem()->itemType()) {
+                return song->song().trackAndTitleStr();
+            }
+        }
+        break;
+    case Cantata::Role_SubText:
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             Device *dev=static_cast<Device *>(item);
             if (!dev->statusMessage().isEmpty()) {
@@ -144,19 +146,19 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
             }
         }
         break;
-    case ItemView::Role_Capacity:
+    case Cantata::Role_Capacity:
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             return static_cast<Device *>(item)->usedCapacity();
         }
         return QVariant();
-    case ItemView::Role_CapacityText:
+    case Cantata::Role_CapacityText:
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             return static_cast<Device *>(item)->capacityString();
         }
         return QVariant();
-    case ItemView::Role_Actions:
+    case Cantata::Role_Actions: {
+        QVariant v;
         if (MusicLibraryItem::Type_Root==item->itemType()) {
-            QVariant v;
             QList<Action *> actions;
             if (Device::AudioCd!=static_cast<Device *>(item)->devType()) {
                 actions << configureAction;
@@ -173,19 +175,11 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
             }
             #endif
             v.setValue<QList<Action *> >(actions);
-            return v;
-        } /*else if (HttpServer::self()->isAlive()) {
-            MusicLibraryItem *root=static_cast<MusicLibraryItem *>(item);
-            while (root && MusicLibraryItem::Type_Root!=root->itemType()) {
-                root=root->parentItem();
-            }
-            if (root && static_cast<Device *>(root)->canPlaySongs()) {
-                QVariant v;
-                v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->addToPlayQueueAction);
-                return v;
-            }
-        } */
-        break;
+        } else if (root(item)->canPlaySongs() && HttpServer::self()->isAlive()) {
+            v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->addToPlayQueueAction);
+        }
+        return v;
+    }
     default:
         break;
     }
@@ -233,8 +227,10 @@ void DevicesModel::setEnabled(bool e)
     if (enabled) {
         connect(Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(const QString &)), this, SLOT(deviceAdded(const QString &)));
         connect(Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(const QString &)), this, SLOT(deviceRemoved(const QString &)));
-//        connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
-//                this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+        #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+        connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
+                this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+        #endif
         // Call loadLocal via a timer, so that upon Cantata start-up model is loaded into view before we try and expand items!
         QTimer::singleShot(0, this, SIGNAL(loadLocal()));
         connect(MountPoints::self(), SIGNAL(updated()), this, SLOT(mountsChanged()));
@@ -257,9 +253,14 @@ void DevicesModel::stop()
 
     disconnect(Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(const QString &)), this, SLOT(deviceAdded(const QString &)));
     disconnect(Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(const QString &)), this, SLOT(deviceRemoved(const QString &)));
-//        disconnect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
-//                   this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    disconnect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
+               this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+    #endif
     disconnect(MountPoints::self(), SIGNAL(updated()), this, SLOT(mountsChanged()));
+    #if defined ENABLE_REMOTE_DEVICES
+    unmountRemote();
+    #endif
 }
 
 Device * DevicesModel::device(const QString &udi)
@@ -268,6 +269,25 @@ Device * DevicesModel::device(const QString &udi)
     return idx<0 ? 0 : static_cast<Device *>(collections.at(idx));
 }
 
+void DevicesModel::setCover(const Song &song, const QImage &img, const QString &file)
+{
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    if (song.isCdda()) {
+        int idx=indexOf(song.title);
+        if (idx>=0) {
+            Device *dev=static_cast<Device *>(collections.at(idx));
+            if (Device::AudioCd==dev->devType()) {
+                static_cast<AudioCdDevice *>(dev)->setCover(song, img, file);
+            }
+        }
+    }
+    #else
+    Q_UNUSED(song)
+    Q_UNUSED(img)
+    Q_UNUSED(file)
+    #endif
+}
+
 void DevicesModel::setCover(const Song &song, const QImage &img)
 {
     if (MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize()) {
@@ -289,7 +309,7 @@ void DevicesModel::setCover(const Song &song, const QImage &img)
     MusicLibraryItemArtist *artistItem = dev->artist(song, false);
     if (artistItem) {
         MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
-        if (albumItem && albumItem->setCover(img)) {
+        if (albumItem && albumItem->saveToCache(img)) {
             QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, index(i, 0, QModelIndex())));
             emit dataChanged(idx, idx);
         }
@@ -322,7 +342,7 @@ Qt::ItemFlags DevicesModel::flags(const QModelIndex &index) const
     if (index.isValid()) {
         return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled;
     }
-    return Qt::ItemIsEnabled;
+    return Qt::NoItemFlags;
 }
 
 QStringList DevicesModel::playableUrls(const QModelIndexList &indexes) const
@@ -398,11 +418,12 @@ void DevicesModel::addLocalDevice(const QString &udi)
         connect(dev, SIGNAL(invalid(QList<Song>)), SIGNAL(invalid(QList<Song>)));
         connect(dev, SIGNAL(updatedDetails(QList<Song>)), SIGNAL(updatedDetails(QList<Song>)));
         connect(dev, SIGNAL(play(QList<Song>)), SLOT(play(QList<Song>)));
+        connect(dev, SIGNAL(renamed()), this, SLOT(updateItemMenu()));
         #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
         if (Device::AudioCd==dev->devType()) {
             connect(static_cast<AudioCdDevice *>(dev), SIGNAL(matches(const QString &, const QList<CdAlbum> &)),
                     SIGNAL(matches(const QString &, const QList<CdAlbum> &)));
-            if (!autoplayCd.isEmpty() && static_cast<AudioCdDevice *>(dev)->isDevice(autoplayCd)) {
+            if (!autoplayCd.isEmpty() && static_cast<AudioCdDevice *>(dev)->isAudioDevice(autoplayCd)) {
                 autoplayCd=QString();
                 static_cast<AudioCdDevice *>(dev)->autoplay();
             }
@@ -656,7 +677,7 @@ void DevicesModel::playCd(const QString &dev)
 {
     foreach (MusicLibraryItemRoot *col, collections) {
         Device *d=static_cast<Device *>(col);
-        if (Device::AudioCd==d->devType() && static_cast<AudioCdDevice *>(d)->isDevice(dev)) {
+        if (Device::AudioCd==d->devType() && static_cast<AudioCdDevice *>(d)->isAudioDevice(dev)) {
             static_cast<AudioCdDevice *>(d)->autoplay();
             return;
         }
