@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -28,24 +28,29 @@
 #include "musiclibraryitemartist.h"
 #include "musiclibraryitemalbum.h"
 #include "musiclibraryitemsong.h"
-#include "song.h"
-#include "covers.h"
-#include "config.h"
-#include "icons.h"
+#include "gui/covers.h"
+#include "widgets/icons.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "device.h"
-#include "utils.h"
+#include "devices/device.h"
+#include "support/utils.h"
+#endif
+#ifdef ENABLE_ONLINE_SERVICES
+#include "online/onlineservice.h"
+#include "onlineservicesmodel.h"
 #endif
-#include "onlineservice.h"
 #include <QPixmap>
 #include <QApplication>
 #include <QFontMetrics>
-#include <QFile>
 
 static MusicLibraryItemAlbum::CoverSize coverSize=MusicLibraryItemAlbum::CoverNone;
+#ifdef ENABLE_UBUNTU
+static const QString constDefaultCover=QLatin1String("qrc:/album.svg");
+#else
 static QPixmap *theDefaultIcon=0;
-static QPixmap *theDefaultLargeIcon=0;
-static bool useDate=false;
+#endif
+static bool dateSort=false;
+
+#ifndef ENABLE_UBUNTU
 static QSize iconItemSize;
 
 static inline int adjust(int v)
@@ -60,13 +65,6 @@ static inline int adjust(int v)
 
 static int fontHeight=16;
 
-#ifdef CACHE_SCALED_COVERS
-static QString cacheCoverName(const QString &artist, const QString &album, int size, bool createDir=false)
-{
-    return Utils::cacheDir(Covers::constCoverDir+QString::number(size)+"/"+Covers::encodeName(artist), createDir)+Covers::encodeName(album)+".png";
-}
-#endif
-
 void MusicLibraryItemAlbum::setup()
 {
     fontHeight=QApplication::fontMetrics().height();
@@ -124,23 +122,20 @@ void MusicLibraryItemAlbum::setCoverSize(MusicLibraryItemAlbum::CoverSize size)
             delete theDefaultIcon;
             theDefaultIcon=0;
         }
-        if (theDefaultLargeIcon) {
-            delete theDefaultLargeIcon;
-            theDefaultLargeIcon=0;
-        }
         MusicLibraryItemArtist::clearDefaultCover();
         coverSize=size;
     }
 }
+#endif
 
-void MusicLibraryItemAlbum::setShowDate(bool sd)
+void MusicLibraryItemAlbum::setSortByDate(bool sd)
 {
-    useDate=sd;
+    dateSort=sd;
 }
 
-bool MusicLibraryItemAlbum::showDate()
+bool MusicLibraryItemAlbum::sortByDate()
 {
-    return useDate;
+    return dateSort;
 }
 
 bool MusicLibraryItemAlbum::lessThan(const MusicLibraryItem *a, const MusicLibraryItem *b)
@@ -152,140 +147,129 @@ bool MusicLibraryItemAlbum::lessThan(const MusicLibraryItem *a, const MusicLibra
         return aa->isSingleTracks() > ab->isSingleTracks();
     }
 
-    if (!MusicLibraryItemAlbum::showDate() || aa->year()==ab->year()) {
-        return aa->data().localeAwareCompare(ab->data())<0;
+    if (!MusicLibraryItemAlbum::sortByDate() || aa->year()==ab->year()) {
+        int compare=aa->data().localeAwareCompare(ab->data());
+        return compare==0 ? aa->id().compare(ab->id())<0 : compare<0;
     }
     return aa->year()<ab->year();
 }
 
-MusicLibraryItemAlbum::MusicLibraryItemAlbum(const QString &data, quint32 year, MusicLibraryItemContainer *parent)
+MusicLibraryItemAlbum::MusicLibraryItemAlbum(const QString &data, const QString &original, const QString &mbId, quint32 year, MusicLibraryItemContainer *parent)
     : MusicLibraryItemContainer(data, parent)
     , m_year(year)
     , m_yearOfTrack(0xFFFF)
     , m_yearOfDisc(0xFFFF)
     , m_totalTime(0)
     , m_numTracks(0)
-    , m_coverIsDefault(false)
-    , m_cover(0)
+    , m_originalName(original!=data ? original : QString())
+    , m_id(mbId)
+    , m_coverRequested(false)
     , m_type(Song::Standard)
 {
 }
 
 MusicLibraryItemAlbum::~MusicLibraryItemAlbum()
 {
-    delete m_cover;
 }
 
-void MusicLibraryItemAlbum::setCoverImage(const QImage &img) const
+QString MusicLibraryItemAlbum::displayData(bool full) const
 {
-    int size=iconSize(largeImages());
-    QImage scaled=img.scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-    m_cover = new QPixmap(QPixmap::fromImage(scaled));
-    m_coverIsDefault=false;
-    #ifdef CACHE_SCALED_COVERS
-    scaled.save(cacheCoverName(parentItem()->data(), data(), size, true));
-    #endif
+    return dateSort || full ? Song::displayAlbum(m_itemData, m_year) : m_itemData;
 }
 
-bool MusicLibraryItemAlbum::setCover(const QImage &img, bool update) const
+#ifdef ENABLE_UBUNTU
+const QString & MusicLibraryItemAlbum::cover() const
 {
-    if ((update || m_coverIsDefault) && !img.isNull()) {
-        setCoverImage(img);
-        return true;
-    }
-
-    return false;
-}
-
-const QPixmap & MusicLibraryItemAlbum::cover()
-{
-    if (m_coverIsDefault) {
-        if (largeImages()) {
-            if (theDefaultLargeIcon) {
-                return *theDefaultLargeIcon;
-            }
-        } else if (theDefaultIcon) {
-            return *theDefaultIcon;
+    if (Song::SingleTracks!=m_type && m_coverName.isEmpty() && !m_coverRequested && childCount()) {
+        MusicLibraryItemSong *firstSong=static_cast<MusicLibraryItemSong*>(childItem(0));
+        Song song;
+        song.artist=firstSong->song().artist;
+        song.albumartist=Song::useComposer() && !firstSong->song().composer().isEmpty() ? firstSong->song().albumArtist() : parentItem()->data();
+        song.album=Song::useComposer() ? firstSong->song().album : m_itemData;
+        song.year=m_year;
+        song.file=firstSong->file();
+        song.type=m_type;
+        song.setComposer(firstSong->song().composer());
+        song.setMbAlbumId(firstSong->song().mbAlbumId());
+        m_coverName=Covers::self()->requestImage(song).fileName;
+        if (!m_coverName.isEmpty()) {
+            m_coverRequested=false;
         }
     }
 
-    if (!m_cover) {
-        bool useLarge=largeImages();
-        int iSize=iconSize(useLarge);
+    return m_coverName.isEmpty() ? constDefaultCover : m_coverName;
+}
+#else
+QPixmap *MusicLibraryItemAlbum::saveToCache(const QImage &img) const
+{
+    int size=MusicLibraryItemAlbum::iconSize(largeImages());
+    return Covers::self()->saveScaledCover(img.scaled(size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation), coverSong(), size);
+}
 
-        if ((useLarge && !theDefaultLargeIcon) || (!useLarge && !theDefaultIcon)) {
-            int cSize=iSize;
-            if (0==cSize) {
-                cSize=22;
-            }
-            if (useLarge) {
-                theDefaultLargeIcon = new QPixmap(Icons::self()->albumIcon.pixmap(cSize, cSize)
-                                                 .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
-            } else {
-                theDefaultIcon = new QPixmap(Icons::self()->albumIcon.pixmap(cSize, cSize)
-                                            .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
-            }
+const QPixmap & MusicLibraryItemAlbum::cover() const
+{
+    int iSize=iconSize(largeImages());
+    if (Song::SingleTracks!=m_type && parentItem() && iSize && childCount()) {
+        Song song=coverSong();
+
+        QPixmap *pix=Covers::self()->getScaledCover(song, iSize);
+        if (pix) {
+            return *pix;
         }
-        m_coverIsDefault = true;
-        if (Song::SingleTracks!=m_type && parentItem() && iSize && childCount()) {
-            #ifdef CACHE_SCALED_COVERS
-            QString cache=cacheCoverName(parentItem()->data(), data(), iSize);
-
-            if (QFile::exists(cache)) {
-                QImage img(cache);
-                if (!img.isNull()) {
-                    m_cover=new QPixmap(QPixmap::fromImage(img));
-                    m_coverIsDefault=false;
-                    return *m_cover;
-                }
-            }
-            #endif
 
+        if (!m_coverRequested) {
             MusicLibraryItemSong *firstSong=static_cast<MusicLibraryItemSong*>(childItem(0));
-            Song song;
-            if (Song::MultipleArtists==m_type) { // Then Cantata has placed this album under 'Various Artists' but the actual album as a different AlbumArtist tag
-                song.artist=firstSong->song().albumArtist();
-            } else {
-                song.artist=firstSong->song().artist;
-                song.albumartist=parentItem()->data();
-            }
-            song.album=m_itemData;
             song.year=m_year;
             song.file=firstSong->file();
             song.type=m_type;
+            song.setComposer(firstSong->song().composer());
+            song.setMbAlbumId(firstSong->song().mbAlbumId());
             Covers::Image img;
+            MusicLibraryItemRoot *root=parentItem() && parentItem()->parentItem() && MusicLibraryItem::Type_Root==parentItem()->parentItem()->itemType()
+                    ? static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem()) : 0;
+            m_coverRequested=true;
+            if (root && !root->useAlbumImages()) {
+                // Not showing album images in this model, so dont request any!
+            }
             #ifdef ENABLE_DEVICES_SUPPORT
-            if (parentItem() && parentItem()->parentItem() && dynamic_cast<Device *>(parentItem()->parentItem()) &&
-                static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem())->useAlbumImages()) {
-                // This item is in the devices model, so get cover from device...
+            else if (root && root->isDevice()) {
                 song.id=firstSong->song().id;
                 static_cast<Device *>(parentItem()->parentItem())->requestCover(song);
-            } else
+            }
             #endif
-            if (parentItem() && parentItem()->parentItem() && dynamic_cast<OnlineService *>(parentItem()->parentItem()) &&
-                static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem())->useAlbumImages()) {
-                // ONLINE: Image URL is encoded in song.name...
-                if (!m_imageUrl.isEmpty()) {
-                    song.name=m_imageUrl;
-                    song.title=parentItem()->parentItem()->data().toLower();
-                    img=Covers::self()->requestImage(song);
-                }
-            } else if (parentItem() && parentItem()->parentItem() && !static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem())->useAlbumImages()) {
-                // Not showing album images in this model, so dont request any!
-            } else {
+            #ifdef ENABLE_ONLINE_SERVICES
+            else if (root && root->isOnlineService()) {
+                img.img=OnlineServicesModel::self()->requestImage(static_cast<OnlineService *>(root)->id(), parentItem()->data(), data(), m_imageUrl);
+            }
+            #endif
+            else {
                 img=Covers::self()->requestImage(song);
             }
 
             if (!img.img.isNull()) {
-                setCoverImage(img.img);
-                return *m_cover;
+                pix=saveToCache(img.img);
+                if (pix) {
+                    return *pix;
+                }
             }
         }
-        return useLarge ? *theDefaultLargeIcon : *theDefaultIcon;
     }
 
-    return *m_cover;
+    int cSize=iSize;
+    if (0==cSize) {
+        cSize=22;
+    }
+
+    if (!theDefaultIcon || theDefaultIcon->width()!=cSize) {
+        if (theDefaultIcon) {
+            delete theDefaultIcon;
+        }
+        theDefaultIcon = new QPixmap(Icons::self()->albumIcon.pixmap(cSize, cSize)
+                                     .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
+    }
+    return *theDefaultIcon;
 }
+#endif
 
 quint32 MusicLibraryItemAlbum::totalTime()
 {
@@ -332,14 +316,6 @@ void MusicLibraryItemAlbum::setIsSingleTracks()
     m_type=Song::SingleTracks;
 }
 
-void MusicLibraryItemAlbum::setIsMultipleArtists()
-{
-    foreach (MusicLibraryItem *track, m_childItems) {
-        static_cast<MusicLibraryItemSong*>(track)->song().type=Song::MultipleArtists;
-    }
-    m_type=Song::MultipleArtists;
-}
-
 void MusicLibraryItemAlbum::append(MusicLibraryItem *i)
 {
     MusicLibraryItemSong *song=static_cast<MusicLibraryItemSong *>(i);
@@ -348,8 +324,6 @@ void MusicLibraryItemAlbum::append(MusicLibraryItem *i)
     if (Song::SingleTracks==m_type) {
         song->song().type=Song::SingleTracks;
         m_singleTrackFiles.insert(song->song().file);
-    } else if (Song::MultipleArtists==m_type) {
-        song->song().type=Song::MultipleArtists;
     }
     m_totalTime=0;
     m_artists.clear();
@@ -413,31 +387,10 @@ QMap<QString, Song> MusicLibraryItemAlbum::getSongs(const QSet<QString> &fileNam
     return map;
 }
 
-bool MusicLibraryItemAlbum::detectIfIsMultipleArtists()
-{
-    if (m_childItems.count()<2) {
-        return false;
-    }
-
-    if (Song::Standard==m_type) {
-        QString a;
-        foreach (MusicLibraryItem *track, m_childItems) {
-            if (a.isEmpty()) {
-                a=static_cast<MusicLibraryItemSong*>(track)->song().artist;
-            } else if (static_cast<MusicLibraryItemSong*>(track)->song().artist!=a) {
-                m_type=Song::MultipleArtists;
-                break;
-            }
-        }
-    }
-    return Song::MultipleArtists==m_type;
-}
-
 const MusicLibraryItemSong * MusicLibraryItemAlbum::getCueFile() const
 {
     foreach (const MusicLibraryItem *s, m_childItems) {
-        if (Song::Playlist==static_cast<const MusicLibraryItemSong *>(s)->song().type &&
-            static_cast<const MusicLibraryItemSong *>(s)->song().file.endsWith(".cue", Qt::CaseInsensitive)) {
+        if (static_cast<const MusicLibraryItemSong *>(s)->song().isCueFile()) {
             return static_cast<const MusicLibraryItemSong *>(s);
         }
     }
@@ -449,9 +402,15 @@ bool MusicLibraryItemAlbum::updateYear()
 {
     quint32 currentYear=m_year;
     foreach (MusicLibraryItem *track, m_childItems) {
-        m_year=static_cast<MusicLibraryItemSong*>(track)->song().year;
-        if (m_year==currentYear) {
-            return false;
+        MusicLibraryItemSong *song=static_cast<MusicLibraryItemSong*>(track);
+        if (Song::Playlist!=song->song().type) {
+            m_year=song->song().year;
+            // Store which track/disc we obtained the year from!
+            m_yearOfTrack=song->track();
+            m_yearOfDisc=song->disc();
+            if (m_year==currentYear) {
+                return false;
+            }
         }
     }
     return true;
@@ -468,21 +427,21 @@ bool MusicLibraryItemAlbum::containsArtist(const QString &a)
     return m_artists.contains(a);
 }
 
-void MusicLibraryItemAlbum::clearImage()
+Song MusicLibraryItemAlbum::coverSong() const
 {
-    if (!m_coverIsDefault) {
-        m_coverIsDefault=true;
-        delete m_cover;
-        m_cover=0;
-        if (theDefaultIcon) {
-            m_cover=theDefaultIcon;
-        }
-    }
+    MusicLibraryItemSong *firstSong=static_cast<MusicLibraryItemSong*>(childItem(0));
+    Song song;
+    song.artist=firstSong->song().artist;
+    song.albumartist=Song::useComposer() && !firstSong->song().composer().isEmpty() ? firstSong->song().albumArtist() : parentItem()->data();
+    song.album=Song::useComposer() ? firstSong->song().album : m_itemData;
+    song.setMbAlbumId(firstSong->song().mbAlbumId());
+    return song;
 }
 
 void MusicLibraryItemAlbum::setYear(const MusicLibraryItemSong *song)
 {
-    if (m_childItems.isEmpty() || (m_yearOfDisc>song->disc() || (m_yearOfDisc==song->disc() && m_yearOfTrack>song->track()))) {
+    if (Song::Playlist!=song->song().type &&
+        (m_childItems.isEmpty() || (m_yearOfDisc>song->disc() || (m_yearOfDisc==song->disc() && m_yearOfTrack>song->track())))) {
         m_year=song->song().year;
         // Store which track/disc we obtained the year from!
         m_yearOfTrack=song->track();
