@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -28,15 +28,15 @@
 #include "musiclibraryitemartist.h"
 #include "musiclibraryitemalbum.h"
 #include "musiclibraryitemsong.h"
-#include "mpdparseutils.h"
-#include "mpdconnection.h"
-#include "song.h"
-#include "localize.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdconnection.h"
+#include "support/localize.h"
 #include "qtiocompressor/qtiocompressor.h"
 #include "musicmodel.h"
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 #include <QFile>
+#include <QElapsedTimer>
 #ifdef TIME_XML_FILE_LOADING
 #include <QDebug>
 #include <QElapsedTimer>
@@ -56,7 +56,7 @@ MusicLibraryItemArtist * MusicLibraryItemRoot::artist(const Song &s, bool create
 MusicLibraryItemArtist * MusicLibraryItemRoot::createArtist(const Song &s)
 {
     QString aa=songArtist(s);
-    MusicLibraryItemArtist *item=new MusicLibraryItemArtist(aa, this);
+    MusicLibraryItemArtist *item=new MusicLibraryItemArtist(aa, Song::Standard==s.type ? s.albumArtist() : QString(), this);
     m_indexes.insert(aa, m_childItems.count());
     m_childItems.append(item);
     return item;
@@ -75,10 +75,9 @@ void MusicLibraryItemRoot::groupSingleTracks()
     for (; it!=m_childItems.end(); ) {
         if (various!=(*it) && static_cast<MusicLibraryItemArtist *>(*it)->allSingleTrack()) {
             if (!various) {
-                QString artist=i18n("Various Artists");
-                QHash<QString, int>::ConstIterator it=m_indexes.find(artist);
+                QHash<QString, int>::ConstIterator it=m_indexes.find(Song::variousArtists());
                 if (m_indexes.end()==it) {
-                    various=new MusicLibraryItemArtist(artist, this);
+                    various=new MusicLibraryItemArtist(Song::variousArtists(), QString(), this);
                     created=true;
                 } else {
                     various=static_cast<MusicLibraryItemArtist *>(m_childItems.at(*it));
@@ -105,70 +104,10 @@ void MusicLibraryItemRoot::groupSingleTracks()
     }
 }
 
-void MusicLibraryItemRoot::groupMultipleArtists()
-{
-    if (!supportsAlbumArtist || isFlat) {
-        return;
-    }
-
-    QList<MusicLibraryItem *>::iterator it=m_childItems.begin();
-    MusicLibraryItemArtist *various=0;
-    bool created=false;
-    QString va=i18n("Various Artists");
-    bool checkDiffVaString=va!=QLatin1String("Various Artists");
-
-    // When grouping multiple artists - if 'Various Artists' is spelt different in curernt language, then we also need to place
-    // items by 'Various Artists' into i18n('Various Artists')
-    for (; it!=m_childItems.end(); ) {
-        if (various!=(*it) && (!static_cast<MusicLibraryItemArtist *>(*it)->isVarious() ||
-                               (checkDiffVaString && static_cast<MusicLibraryItemArtist *>(*it)->isVarious() && va!=(*it)->data())) ) {
-            QList<MusicLibraryItem *> mutipleAlbums=static_cast<MusicLibraryItemArtist *>(*it)->mutipleArtistAlbums();
-            if (mutipleAlbums.count()) {
-                if (!various) {
-                    QHash<QString, int>::ConstIterator it=m_indexes.find(va);
-                    if (m_indexes.end()==it) {
-                        various=new MusicLibraryItemArtist(va, this);
-                        created=true;
-                    } else {
-                        various=static_cast<MusicLibraryItemArtist *>(m_childItems.at(*it));
-                    }
-                }
-
-                foreach (MusicLibraryItem *i, mutipleAlbums) {
-                    i->setParent(various);
-                    static_cast<MusicLibraryItemAlbum *>(i)->setIsMultipleArtists();
-                }
-
-                if (0==(*it)->childCount()) {
-                    delete (*it);
-                    it=m_childItems.erase(it);
-                    continue;
-                } else {
-                    static_cast<MusicLibraryItemArtist *>(*it)->updateIndexes();
-                }
-            }
-        }
-        ++it;
-    }
-
-    if (various) {
-        various->updateIndexes();
-        m_indexes.clear();
-        if (created) {
-            m_childItems.append(various);
-        }
-        it=m_childItems.begin();
-        QList<MusicLibraryItem *>::iterator end=m_childItems.end();
-        for (int i=0; it!=end; ++it, ++i) {
-            m_indexes.insert((*it)->data(), i);
-        }
-    }
-}
-
 bool MusicLibraryItemRoot::isFromSingleTracks(const Song &s) const
 {
     if (!isFlat && (supportsAlbumArtist && !s.file.isEmpty())) {
-        QHash<QString, int>::ConstIterator it=m_indexes.find(i18n("Various Artists"));
+        QHash<QString, int>::ConstIterator it=m_indexes.find(Song::variousArtists());
 
         if (m_indexes.end()!=it) {
             return static_cast<MusicLibraryItemArtist *>(m_childItems.at(*it))->isFromSingleTracks(s);
@@ -244,23 +183,25 @@ QSet<Song> MusicLibraryItemRoot::allSongs(bool revertVa) const
     return songs;
 }
 
-void MusicLibraryItemRoot::getDetails(QSet<QString> &artists, QSet<QString> &albumArtists, QSet<QString> &albums, QSet<QString> &genres)
+void MusicLibraryItemRoot::getDetails(QSet<QString> &artists, QSet<QString> &albumArtists, QSet<QString> &composers, QSet<QString> &albums, QSet<QString> &genres)
 {
     foreach (const MusicLibraryItem *child, m_childItems) {
         if (MusicLibraryItem::Type_Song==child->itemType()) {
             const Song &s=static_cast<const MusicLibraryItemSong *>(child)->song();
             artists.insert(s.artist);
             albumArtists.insert(s.albumArtist());
+            composers.insert(s.composer());
             albums.insert(s.album);
             if (!s.genre.isEmpty()) {
                 genres.insert(s.genre);
             }
-        } else {
+        } else if (MusicLibraryItem::Type_Artist==child->itemType()) {
             foreach (const MusicLibraryItem *album, static_cast<const MusicLibraryItemContainer *>(child)->childItems()) {
                 foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
                     const Song &s=static_cast<const MusicLibraryItemSong *>(song)->song();
                     artists.insert(s.artist);
                     albumArtists.insert(s.albumArtist());
+                    composers.insert(s.composer());
                     albums.insert(s.album);
                     if (!s.genre.isEmpty()) {
                         genres.insert(s.genre);
@@ -291,17 +232,17 @@ void MusicLibraryItemRoot::updateSongFile(const Song &from, const Song &to)
     }
 }
 
-static quint32 constVersion=8;
+static quint32 constVersion=9;
 static QLatin1String constTopTag("CantataLibrary");
 
-void MusicLibraryItemRoot::toXML(const QString &filename, const QDateTime &date, MusicLibraryProgressMonitor *prog) const
+void MusicLibraryItemRoot::toXML(const QString &filename, const QDateTime &date, bool dateUnreliable, MusicLibraryProgressMonitor *prog) const
 {
     if (isFlat) {
         return;
     }
 
     // If saving device cache, and we have NO items, then remove cache file...
-    if (0==childCount() && date==QDateTime()) {
+    if (0==childCount() && date.date().year()<2000) {
         if (QFile::exists(filename)) {
             QFile::remove(filename);
         }
@@ -316,7 +257,7 @@ void MusicLibraryItemRoot::toXML(const QString &filename, const QDateTime &date,
     }
 
     QXmlStreamWriter writer(&compressor);
-    toXML(writer, date, prog);
+    toXML(writer, date, dateUnreliable, prog);
     compressor.close();
 }
 
@@ -326,26 +267,30 @@ static const QString constTrackElement=QLatin1String("Track");
 static const QString constNameAttribute=QLatin1String("name");
 static const QString constArtistAttribute=QLatin1String("artist");
 static const QString constAlbumArtistAttribute=QLatin1String("albumartist");
+static const QString constComposerAttribute=QLatin1String("composer");
 static const QString constAlbumAttribute=QLatin1String("album");
+static const QString constActualAttribute=QLatin1String("actual");
 static const QString constTrackAttribute=QLatin1String("track");
 static const QString constGenreAttribute=QLatin1String("genre");
 static const QString constYearAttribute=QLatin1String("year");
 static const QString constTimeAttribute=QLatin1String("time");
 static const QString constDiscAttribute=QLatin1String("disc");
+static const QString constMbIdAttribute=QLatin1String("mbid");
 static const QString constFileAttribute=QLatin1String("file");
 static const QString constPlaylistAttribute=QLatin1String("playlist");
 static const QString constGuessedAttribute=QLatin1String("guessed");
 static const QString constDateAttribute=QLatin1String("date");
+static const QString constDateUnreliableAttribute=QLatin1String("dateUnreliable");
 static const QString constVersionAttribute=QLatin1String("version");
 static const QString constGroupSingleAttribute=QLatin1String("groupSingle");
-static const QString constGroupMultipleAttribute=QLatin1String("groupMultiple");
+static const QString constUseComposerAttribute=QLatin1String("useComposer");
 static const QString constSingleTracksAttribute=QLatin1String("singleTracks");
 static const QString constMultipleArtistsAttribute=QLatin1String("multipleArtists");
 static const QString constImageAttribute=QLatin1String("img");
 static const QString constnumTracksAttribute=QLatin1String("numTracks");
 static const QString constTrueValue=QLatin1String("true");
 
-void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date, MusicLibraryProgressMonitor *prog) const
+void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date, bool dateUnreliable, MusicLibraryProgressMonitor *prog) const
 {
     if (isFlat) {
         return;
@@ -353,20 +298,27 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
 
     quint64 total=0;
     quint64 count=0;
+    int percent=0;
+    QElapsedTimer timer;
+    if (prog) {
+        prog->writeProgress(0.0);
+        timer.start();
+    }
     writer.writeStartDocument();
-    QString unknown=i18n("Unknown");
 
     //Start with the document
     writer.writeStartElement(constTopTag);
     writer.writeAttribute(constVersionAttribute, QString::number(constVersion));
     writer.writeAttribute(constDateAttribute, QString::number(date.toTime_t()));
+    if (dateUnreliable) {
+        writer.writeAttribute(constDateUnreliableAttribute, constTrueValue);
+    }
     if (MPDParseUtils::groupSingle()) {
         writer.writeAttribute(constGroupSingleAttribute, constTrueValue);
     }
-    if (MPDParseUtils::groupMultiple()) {
-        writer.writeAttribute(constGroupMultipleAttribute, constTrueValue);
+    if (Song::useComposer()) {
+        writer.writeAttribute(constUseComposerAttribute, constTrueValue);
     }
-
     foreach (const MusicLibraryItem *a, childItems()) {
         foreach (const MusicLibraryItem *al, static_cast<const MusicLibraryItemArtist *>(a)->childItems()) {
             total+=al->childCount();
@@ -377,40 +329,40 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
     }
 
     writer.writeAttribute(constnumTracksAttribute, QString::number(total));
-    if (prog) {
-        prog->writeProgress(0.0);
-    }
+
     //Loop over all artist, albums and tracks.
     foreach (const MusicLibraryItem *a, childItems()) {
         const MusicLibraryItemArtist *artist = static_cast<const MusicLibraryItemArtist *>(a);
         writer.writeStartElement(constArtistElement);
         writer.writeAttribute(constNameAttribute, artist->data());
-        if (!artist->imageUrl().isEmpty()) {
-            writer.writeAttribute(constImageAttribute, artist->imageUrl());
+        if (!artist->actualArtist().isEmpty()) {
+            writer.writeAttribute(constActualAttribute, artist->actualArtist());
         }
         foreach (const MusicLibraryItem *al, artist->childItems()) {
             if (prog && prog->wasStopped()) {
                 return;
             }
             const MusicLibraryItemAlbum *album = static_cast<const MusicLibraryItemAlbum *>(al);
-            QString albumGenre=!album->childItems().isEmpty() ? static_cast<const MusicLibraryItemSong *>(album->childItems().at(0))->song().genre : QString();
+            QString albumGenre=Song::combineGenres(album->genres());
             writer.writeStartElement(constAlbumElement);
-            writer.writeAttribute(constNameAttribute, album->data());
+            writer.writeAttribute(constNameAttribute, album->originalName().isEmpty() ? album->data() : album->originalName());
             writer.writeAttribute(constYearAttribute, QString::number(album->year()));
-            if (!albumGenre.isEmpty() && albumGenre!=unknown) {
+            if (!albumGenre.isEmpty() && albumGenre!=Song::unknown()) {
                 writer.writeAttribute(constGenreAttribute, albumGenre);
             }
             if (album->isSingleTracks()) {
                 writer.writeAttribute(constSingleTracksAttribute, constTrueValue);
-            } else if (album->isMultipleArtists()) {
-                writer.writeAttribute(constMultipleArtistsAttribute, constTrueValue);
             }
             if (!album->imageUrl().isEmpty()) {
                 writer.writeAttribute(constImageAttribute, album->imageUrl());
             }
+            if (!album->id().isEmpty()) {
+                writer.writeAttribute(constMbIdAttribute, album->id());
+            }
+            QString artistName=artist->actualArtist().isEmpty() ? artist->data() : artist->actualArtist();
+
             foreach (const MusicLibraryItem *t, album->childItems()) {
                 const MusicLibraryItemSong *track = static_cast<const MusicLibraryItemSong *>(t);
-                bool wroteArtist=false;
                 writer.writeEmptyElement(constTrackElement);
                 if (!track->song().title.isEmpty()) {
                     writer.writeAttribute(constNameAttribute, track->song().title);
@@ -426,21 +378,21 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
                 if (track->disc() != 0) {
                     writer.writeAttribute(constDiscAttribute, QString::number(track->disc()));
                 }
-                if (!track->song().artist.isEmpty() && track->song().artist!=artist->data()) {
+                if (!track->song().artist.isEmpty() && track->song().artist!=artistName) {
                     writer.writeAttribute(constArtistAttribute, track->song().artist);
-                    wroteArtist=true;
                 }
-                if (track->song().albumartist!=artist->data()) {
+                if (supportsAlbumArtist && track->song().albumartist!=artistName) {
                     writer.writeAttribute(constAlbumArtistAttribute, track->song().albumartist);
                 }
-//                 writer.writeAttribute("id", QString::number(track->song().id));
-                if (!track->song().genre.isEmpty() && track->song().genre!=albumGenre && track->song().genre!=unknown) {
+                if (!track->song().composer().isEmpty()) {
+                    writer.writeAttribute(constComposerAttribute, track->song().composer());
+                }
+                QString trackGenre=track->multipleGenres() ? Song::combineGenres(track->allGenres()) : track->genre();
+                if (!trackGenre.isEmpty() && trackGenre!=albumGenre && trackGenre!=Song::unknown()) {
                     writer.writeAttribute(constGenreAttribute, track->song().genre);
                 }
                 if (album->isSingleTracks()) {
                     writer.writeAttribute(constAlbumAttribute, track->song().album);
-                } else if (!wroteArtist && album->isMultipleArtists() && !track->song().artist.isEmpty() && track->song().artist!=artist->data()) {
-                    writer.writeAttribute(constArtistAttribute, track->song().artist);
                 }
                 if (Song::Playlist==track->song().type) {
                     writer.writeAttribute(constPlaylistAttribute, constTrueValue);
@@ -453,7 +405,12 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
                 }
                 if (prog && !prog->wasStopped() && total>0) {
                     count++;
-                    prog->writeProgress((count*100.0)/(total*1.0));
+                    int pc=((count*100.0)/(total*1.0))+0.5;
+                    if (pc!=percent && timer.elapsed()>=250) {
+                        prog->writeProgress(pc);
+                        timer.restart();
+                        percent=pc;
+                    }
                 }
             }
             writer.writeEndElement();
@@ -465,7 +422,7 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
     writer.writeEndDocument();
 }
 
-quint32 MusicLibraryItemRoot::fromXML(const QString &filename, const QDateTime &date, const QString &baseFolder, MusicLibraryProgressMonitor *prog)
+quint32 MusicLibraryItemRoot::fromXML(const QString &filename, const QDateTime &date, bool *dateUnreliable, const QString &baseFolder, MusicLibraryProgressMonitor *prog)
 {
     if (isFlat) {
         return 0;
@@ -482,17 +439,16 @@ quint32 MusicLibraryItemRoot::fromXML(const QString &filename, const QDateTime &
     if (!compressor.open(QIODevice::ReadOnly)) {
         return 0;
     }
-
     QXmlStreamReader reader(&compressor);
-    quint32 rv=fromXML(reader, date, baseFolder, prog);
+    quint32 rv=fromXML(reader, date, dateUnreliable, baseFolder, prog);
     compressor.close();
     #ifdef TIME_XML_FILE_LOADING
     qWarning() << filename << timer.elapsed();
     #endif
     return rv;
 }
-
-quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime &date, const QString &baseFolder, MusicLibraryProgressMonitor *prog)
+#include <QDebug>
+quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime &date, bool *dateUnreliable, const QString &baseFolder, MusicLibraryProgressMonitor *prog)
 {
     if (isFlat) {
         return 0;
@@ -502,15 +458,17 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
     MusicLibraryItemAlbum *albumItem = 0;
     Song song;
     quint32 xmlDate=0;
-    QString unknown=i18n("Unknown");
     quint64 total=0;
     quint64 count=0;
     bool gs=MPDParseUtils::groupSingle();
-    bool gm=MPDParseUtils::groupMultiple();
+    bool gm=false;
     int percent=0;
+    bool online=isOnlineService();
+    QElapsedTimer timer;
 
     if (prog) {
         prog->readProgress(0.0);
+        timer.start();
     }
 
     while (!reader.atEnd() && (!prog || !prog->wasStopped())) {
@@ -524,26 +482,32 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                 quint32 version = attributes.value(constVersionAttribute).toString().toUInt();
                 xmlDate = attributes.value(constDateAttribute).toString().toUInt();
                 gs = constTrueValue==attributes.value(constGroupSingleAttribute).toString();
-                gm = constTrueValue==attributes.value(constGroupMultipleAttribute).toString();
-
-                if ( version < constVersion || (date.isValid() && xmlDate < date.toTime_t())) {
+                gm = constTrueValue==attributes.value(constGroupSingleAttribute).toString();
+                bool uc = constTrueValue==attributes.value(constUseComposerAttribute).toString();
+                if ( version < constVersion || uc!=Song::useComposer() || (date.isValid() && xmlDate < date.toTime_t())) {
                     return 0;
                 }
                 if (prog) {
                     total=attributes.value(constnumTracksAttribute).toString().toUInt();
                 }
-            } else if (constArtistElement==element) {
-                song.type=Song::Standard;
-                song.artist=song.albumartist=attributes.value(constNameAttribute).toString();
-                artistItem = createArtist(song);
-                QString img = attributes.value(constImageAttribute).toString();
-                if (!img.isEmpty()) {
-                    artistItem->setImageUrl(img);
+                if (dateUnreliable) {
+                    *dateUnreliable=constTrueValue==attributes.value(constDateUnreliableAttribute).toString();
                 }
+            } else if (constArtistElement==element) {
+                QString actual=attributes.value(constActualAttribute).toString();
+                song.type=Song::Standard;
+                if (actual.isEmpty()) {
+                    song.artist=song.albumartist=attributes.value(constNameAttribute).toString();
+                } else {
+                    song.artist=song.albumartist=actual;
+                    song.setComposer(attributes.value(constNameAttribute).toString());
+                }
+                artistItem = createArtist(song);
             } else if (constAlbumElement==element) {
                 song.album=attributes.value(constNameAttribute).toString();
                 song.year=attributes.value(constYearAttribute).toString().toUInt();
                 song.genre=attributes.value(constGenreAttribute).toString();
+                song.setMbAlbumId(attributes.value(constMbIdAttribute).toString());
                 if (!song.file.isEmpty()) {
                     song.file.append("dummy.mp3");
                 }
@@ -555,9 +519,6 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                 if (constTrueValue==attributes.value(constSingleTracksAttribute).toString()) {
                     albumItem->setIsSingleTracks();
                     song.type=Song::SingleTracks;
-                } else if (constTrueValue==attributes.value(constMultipleArtistsAttribute).toString()) {
-                    albumItem->setIsMultipleArtists();
-                    song.type=Song::MultipleArtists;
                 } else {
                     song.type=Song::Standard;
                 }
@@ -578,26 +539,30 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                     QString genre=attributes.value(constGenreAttribute).toString();
                     if (genre.isEmpty() ) {
                         if (song.genre.isEmpty()) {
-                            song.genre=unknown;
+                            song.genre=Song::unknown();
                         }
                     } else {
                         song.genre=genre;
                     }
                     if (attributes.hasAttribute(constArtistAttribute)) {
                         song.artist=attributes.value(constArtistAttribute).toString();
-                    } else {
+                    } else if (artistItem->actualArtist().isEmpty()) {
                         song.artist=artistItem->data();
-                    }
-                    if (attributes.hasAttribute(constAlbumArtistAttribute)) {
-                        song.albumartist=attributes.value(constAlbumArtistAttribute).toString();
                     } else {
-                        song.albumartist=artistItem->data();
+                        song.artist=artistItem->actualArtist();
                     }
-
-                    // Fix cache error - where MusicLibraryItemSong::data() was saved as name instead of song.name!!!!
-                    if (!song.albumartist.isEmpty() && !song.artist.isEmpty() && song.albumartist!=song.artist &&
-                        song.title.startsWith(song.artist+QLatin1String(" - "))) {
-                        song.title=song.title.mid(song.artist.length()+3);
+                    if (supportsAlbumArtist) {
+                        if (attributes.hasAttribute(constAlbumArtistAttribute)) {
+                            song.albumartist=attributes.value(constAlbumArtistAttribute).toString();
+                        } else if (artistItem->actualArtist().isEmpty()) {
+                            song.albumartist=artistItem->data();
+                        } else {
+                            song.albumartist=artistItem->actualArtist();
+                        }
+                    }
+                    QString composer=attributes.value(constComposerAttribute).toString();
+                    if (!composer.isEmpty()) {
+                        song.setComposer(composer);
                     }
 
                     QString str=attributes.value(constTrackAttribute).toString();
@@ -610,54 +575,48 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                     if (!str.isEmpty()) {
                         song.year=str.toUInt();
                     }
-    //                 str=attributes.value("id").toString();
-    //                 song.id=str.isEmpty() ? 0 : str.toUInt();
 
                     if (albumItem->isSingleTracks()) {
                         str=attributes.value(constAlbumAttribute).toString();
                         if (!str.isEmpty()) {
                             song.album=str;
                         }
-                    } else if (albumItem->isMultipleArtists()) {
-                        str=attributes.value(constArtistAttribute).toString();
-                        if (!str.isEmpty()) {
-                            song.artist=str;
-                        }
                     }
 
                     song.fillEmptyFields();
                     song.guessed=constTrueValue==attributes.value(constGuessedAttribute).toString();
-                    albumItem->append(new MusicLibraryItemSong(song, albumItem));
-                    albumItem->addGenre(song.genre);
-                    artistItem->addGenre(song.genre);
-                    addGenre(song.genre);
+                    if (online) {
+                        song.type=Song::OnlineSvrTrack;
+                    }
+
+                    MusicLibraryItemSong *songItem=new MusicLibraryItemSong(song, albumItem);
+                    QSet<QString> songGenres=songItem->allGenres();
+                    albumItem->append(songItem);
+                    albumItem->addGenres(songGenres);
+                    artistItem->addGenres(songGenres);
+                    addGenres(songGenres);
 
                     if (prog && !prog->wasStopped() && total>0) {
                         count++;
                         int pc=((count*100.0)/(total*1.0))+0.5;
-                        if (pc!=percent) {
+                        if (pc!=percent && timer.elapsed()>=250) {
                             prog->readProgress(pc);
+                            timer.restart();
                             percent=pc;
                         }
                     }
                 }
                 song.time=song.track=0;
+                song.setComposer(QString());
             }
         }
     }
 
     // Grouping has changed!
-    if (gs!=MPDParseUtils::groupSingle() || gm!=MPDParseUtils::groupMultiple()) {
-        if (gs && gm) {
-            // Now using both groupings, where previously we were using none.
-            // ...so, just apply grouping!
-            groupSingleTracks();
-            groupMultipleArtists();
-            updateGenres();
-        } else {
-            // Mixed grouping, so need to redo from scratch...
-            toggleGrouping();
-        }
+    // As of 1.4.0, Cantata no-longer groups multiple-artist albums under 'Various Artists' - so if this setting
+    // has been used, we need to undo it!
+    if (gs!=MPDParseUtils::groupSingle() || gm) {
+        toggleGrouping();
     }
 
     return xmlDate;
@@ -684,10 +643,12 @@ void MusicLibraryItemRoot::add(const QSet<Song> &songs)
             albumItem = artistItem->album(s);
         }
 
-        albumItem->append(new MusicLibraryItemSong(s, albumItem));
-        albumItem->addGenre(s.genre);
-        artistItem->addGenre(s.genre);
-        addGenre(s.genre);
+        MusicLibraryItemSong *songItem=new MusicLibraryItemSong(s, albumItem);
+        QSet<QString> songGenres=songItem->allGenres();
+        albumItem->append(songItem);
+        albumItem->addGenres(songGenres);
+        artistItem->addGenres(songGenres);
+        addGenres(songGenres);
     }
 }
 
@@ -708,17 +669,19 @@ void MusicLibraryItemRoot::toggleGrouping()
             currentSong.type=MPDConnection::isPlaylist(currentSong.file) ? Song::Playlist : Song::Standard;
         }
 
-        if (!artistItem || currentSong.albumArtist()!=artistItem->data()) {
+        if (!artistItem || currentSong.artistOrComposer()!=artistItem->data()) {
             artistItem = artist(currentSong);
         }
-        if (!albumItem || currentSong.year!=albumItem->year() || albumItem->parentItem()!=artistItem || currentSong.album!=albumItem->data()) {
+        if (!albumItem || currentSong.year!=albumItem->year() || albumItem->parentItem()!=artistItem || currentSong.albumName()!=albumItem->data()) {
             albumItem = artistItem->album(currentSong);
         }
 
-        albumItem->append(new MusicLibraryItemSong(currentSong, albumItem));
-        albumItem->addGenre(currentSong.genre);
-        artistItem->addGenre(currentSong.genre);
-        addGenre(currentSong.genre);
+        MusicLibraryItemSong *song=new MusicLibraryItemSong(currentSong, albumItem);
+        QSet<QString> songGenres=song->allGenres();
+        albumItem->append(song);
+        albumItem->addGenres(songGenres);
+        artistItem->addGenres(songGenres);
+        addGenres(songGenres);
     }
 
     // Library rebuilt, now apply any grouping...
@@ -729,11 +692,8 @@ void MusicLibraryItemRoot::applyGrouping()
 {
     if (MPDParseUtils::groupSingle()) {
         groupSingleTracks();
+        updateGenres();
     }
-    if (MPDParseUtils::groupMultiple()) {
-        groupMultipleArtists();
-    }
-    updateGenres();
 }
 
 void MusicLibraryItemRoot::clearItems()
@@ -752,6 +712,7 @@ bool MusicLibraryItemRoot::update(const QSet<Song> &songs)
     QSet<Song> added=updateSongs-currentSongs;
 
     bool updatedSongs=added.count()||removed.count();
+
     foreach (const Song &s, removed) {
         removeSongFromList(s);
     }
@@ -773,7 +734,7 @@ const MusicLibraryItem * MusicLibraryItemRoot::findSong(const Song &s) const
             }
         }
     } else {
-        MusicLibraryItemArtist *artistItem = ((MusicLibraryItemRoot *)this)->artist(s, false);
+        MusicLibraryItemArtist *artistItem = const_cast<MusicLibraryItemRoot *>(this)->artist(s, false);
         if (artistItem) {
             MusicLibraryItemAlbum *albumItem = artistItem->album(s, false);
             if (albumItem) {
@@ -798,16 +759,7 @@ bool MusicLibraryItemRoot::songExists(const Song &s) const
 
     if (!s.isVariousArtists()) {
         Song mod(s);
-        mod.albumartist=i18n("Various Artists");
-        if (MPDParseUtils::groupMultiple()) {
-            song=findSong(mod);
-            if (song) {
-                Song sng=static_cast<const MusicLibraryItemSong *>(song)->song();
-                if (sng.albumArtist()==s.albumArtist()) {
-                    return true;
-                }
-            }
-        }
+        mod.albumartist=Song::variousArtists();
         if (MPDParseUtils::groupSingle()) {
             mod.album=i18n("Single Tracks");
             song=findSong(mod);
@@ -832,7 +784,7 @@ bool MusicLibraryItemRoot::updateSong(const Song &orig, const Song &edit)
     if (isFlat) {
         int songRow=0;
         foreach (MusicLibraryItem *song, childItems()) {
-            if (static_cast<MusicLibraryItemSong *>(song)->song()==orig) {
+            if (static_cast<MusicLibraryItemSong *>(song)->song().file==orig.file) {
                 static_cast<MusicLibraryItemSong *>(song)->setSong(edit);
                 if (orig.genre!=edit.genre) {
                     updateGenres();
@@ -854,8 +806,9 @@ bool MusicLibraryItemRoot::updateSong(const Song &orig, const Song &edit)
         }
         int songRow=0;
         foreach (MusicLibraryItem *song, albumItem->childItems()) {
-            if (static_cast<MusicLibraryItemSong *>(song)->song()==orig) {
+            if (static_cast<MusicLibraryItemSong *>(song)->song().file==orig.file) {
                 static_cast<MusicLibraryItemSong *>(song)->setSong(edit);
+                bool yearUpdated=orig.year!=edit.year && albumItem->updateYear();
                 if (orig.genre!=edit.genre) {
                     albumItem->updateGenres();
                     artistItem->updateGenres();
@@ -863,6 +816,10 @@ bool MusicLibraryItemRoot::updateSong(const Song &orig, const Song &edit)
                 }
                 QModelIndex idx=m_model->createIndex(songRow, 0, song);
                 emit m_model->dataChanged(idx, idx);
+                if (yearUpdated) {
+                    idx=m_model->createIndex(albumItem->row(), 0, albumItem);
+                    emit m_model->dataChanged(idx, idx);
+                }
                 return true;
             }
             songRow++;
@@ -881,28 +838,44 @@ void MusicLibraryItemRoot::addSongToList(const Song &s)
     if (!artistItem) {
         m_model->beginInsertRows(index(), childCount(), childCount());
         artistItem = createArtist(s);
+        artistItem->setIsNew(true);
         m_model->endInsertRows();
     }
     MusicLibraryItemAlbum *albumItem = artistItem->album(s, false);
     if (!albumItem) {
-        m_model->beginInsertRows(m_model->createIndex(childItems().indexOf(artistItem), 0, artistItem), artistItem->childCount(), artistItem->childCount());
+        m_model->beginInsertRows(m_model->createIndex(artistItem->row(), 0, artistItem), artistItem->childCount(), artistItem->childCount());
         albumItem = artistItem->createAlbum(s);
+        albumItem->setIsNew(true);
         m_model->endInsertRows();
+        if (!artistItem->isNew()) {
+            artistItem->setIsNew(true);
+            QModelIndex idx=m_model->createIndex(artistItem->row(), 0, artistItem);
+            emit m_model->dataChanged(idx, idx);
+        }
     }
     quint32 year=albumItem->year();
     foreach (const MusicLibraryItem *songItem, albumItem->childItems()) {
-        const MusicLibraryItemSong *song=static_cast<const MusicLibraryItemSong *>(songItem);
-        if (song->track()==s.track && song->disc()==s.disc && song->data()==s.displayTitle()) {
+        if (static_cast<const MusicLibraryItemSong *>(songItem)->song().file==s.file) {
             return;
         }
     }
 
-    m_model->beginInsertRows(m_model->createIndex(artistItem->childItems().indexOf(albumItem), 0, albumItem), albumItem->childCount(), albumItem->childCount());
+    m_model->beginInsertRows(m_model->createIndex(albumItem->row(), 0, albumItem), albumItem->childCount(), albumItem->childCount());
     MusicLibraryItemSong *songItem = new MusicLibraryItemSong(s, albumItem);
     albumItem->append(songItem);
     m_model->endInsertRows();
-    if (year!=albumItem->year()) {
-        QModelIndex idx=m_model->createIndex(artistItem->childItems().indexOf(albumItem), 0, albumItem);
+
+    if (!artistItem->isNew()) {
+        artistItem->setIsNew(true);
+        QModelIndex idx=m_model->createIndex(artistItem->row(), 0, artistItem);
+        emit m_model->dataChanged(idx, idx);
+    }
+    if (!albumItem->isNew()) {
+        albumItem->setIsNew(true);
+        QModelIndex idx=m_model->createIndex(albumItem->row(), 0, albumItem);
+        emit m_model->dataChanged(idx, idx);
+    } else if (year!=albumItem->year()) {
+        QModelIndex idx=m_model->createIndex(albumItem->row(), 0, albumItem);
         emit m_model->dataChanged(idx, idx);
     }
 }
@@ -924,7 +897,7 @@ void MusicLibraryItemRoot::removeSongFromList(const Song &s)
     MusicLibraryItem *songItem=0;
     int songRow=0;
     foreach (MusicLibraryItem *song, albumItem->childItems()) {
-        if (static_cast<MusicLibraryItemSong *>(song)->song().title==s.title) {
+        if (static_cast<MusicLibraryItemSong *>(song)->song().file==s.file) {
             songItem=song;
             break;
         }
@@ -936,7 +909,7 @@ void MusicLibraryItemRoot::removeSongFromList(const Song &s)
 
     if (1==artistItem->childCount() && 1==albumItem->childCount()) {
         // 1 album with 1 song - so remove whole artist
-        int row=m_childItems.indexOf(artistItem);
+        int row=artistItem->row();
         m_model->beginRemoveRows(index(), row, row);
         remove(artistItem);
         m_model->endRemoveRows();
@@ -945,39 +918,30 @@ void MusicLibraryItemRoot::removeSongFromList(const Song &s)
 
     if (1==albumItem->childCount()) {
         // multiple albums, but this album only has 1 song - remove album
-        int row=artistItem->childItems().indexOf(albumItem);
-        m_model->beginRemoveRows(m_model->createIndex(childItems().indexOf(artistItem), 0, artistItem), row, row);
+        int row=albumItem->row();
+        m_model->beginRemoveRows(m_model->createIndex(artistItem->row(), 0, artistItem), row, row);
         artistItem->remove(albumItem);
         m_model->endRemoveRows();
         return;
     }
 
     // Just remove particular song
-    m_model->beginRemoveRows(m_model->createIndex(artistItem->childItems().indexOf(albumItem), 0, albumItem), songRow, songRow);
+    m_model->beginRemoveRows(m_model->createIndex(albumItem->row(), 0, albumItem), songRow, songRow);
     quint32 year=albumItem->year();
     albumItem->remove(songRow);
     m_model->endRemoveRows();
     if (year!=albumItem->year()) {
-        QModelIndex idx=m_model->createIndex(artistItem->childItems().indexOf(albumItem), 0, albumItem);
+        QModelIndex idx=m_model->createIndex(albumItem->row(), 0, albumItem);
         emit m_model->dataChanged(idx, idx);
     }
 }
 
-void MusicLibraryItemRoot::clearImages()
-{
-    foreach (MusicLibraryItem *i, m_childItems) {
-        if (MusicLibraryItem::Type_Artist==i->itemType()) {
-            static_cast<MusicLibraryItemArtist *>(i)->clearImages();
-        }
-    }
-}
-
 QString MusicLibraryItemRoot::artistName(const Song &s)
 {
-    if (Song::Standard==s.type || (Song::Playlist==s.type && !s.albumArtist().isEmpty())) {
-        return s.albumArtist();
+    if (Song::Standard==s.type || Song::Cdda==s.type || Song::OnlineSvrTrack==s.type || (Song::Playlist==s.type && !s.albumArtist().isEmpty())) {
+        return s.artistOrComposer();
     }
-    return i18n("Various Artists");
+    return Song::variousArtists();
 }
 
 QString MusicLibraryItemRoot::songArtist(const Song &s) const
