@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -27,42 +27,60 @@
 #include <QModelIndex>
 #include <QMimeData>
 #include <QMenu>
+#include <QFont>
+#include "config.h"
 #include "playlistsmodel.h"
 #include "playlistsproxymodel.h"
-#include "itemview.h"
-#include "qtplural.h"
-#include "groupedview.h"
-#include "localize.h"
-#include "utils.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobal>
-K_GLOBAL_STATIC(PlaylistsModel, instance)
-#endif
-#include "mpdparseutils.h"
-#include "mpdstats.h"
-#include "mpdconnection.h"
 #include "playqueuemodel.h"
-#include "icons.h"
-#include "gtkstyle.h"
+#include "widgets/groupedview.h"
+#include "roles.h"
+#include "gui/plurals.h"
+#include "support/localize.h"
+#include "support/utils.h"
+#include "support/globalstatic.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdstats.h"
+#include "mpd/mpdconnection.h"
+#include "playqueuemodel.h"
+#include "streamsmodel.h"
+#include "widgets/icons.h"
+#ifdef ENABLE_HTTP_SERVER
+#include "http/httpserver.h"
+#endif
 
-PlaylistsModel * PlaylistsModel::self()
+#if defined ENABLE_MODEL_TEST
+#include "modeltest.h"
+#endif
+
+#ifndef ENABLE_UBUNTU
+QString PlaylistsModel::headerText(int col)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static PlaylistsModel *instance=0;
-    if(!instance) {
-        instance=new PlaylistsModel;
+    switch (col) {
+    case COL_TITLE:     return PlayQueueModel::headerText(PlayQueueModel::COL_TITLE);
+    case COL_ARTIST:    return PlayQueueModel::headerText(PlayQueueModel::COL_ARTIST);
+    case COL_ALBUM:     return PlayQueueModel::headerText(PlayQueueModel::COL_ALBUM);
+    case COL_LENGTH:    return PlayQueueModel::headerText(PlayQueueModel::COL_LENGTH);
+    case COL_YEAR:      return PlayQueueModel::headerText(PlayQueueModel::COL_YEAR);
+    case COL_GENRE:     return PlayQueueModel::headerText(PlayQueueModel::COL_GENRE);
+    case COL_COMPOSER:  return PlayQueueModel::headerText(PlayQueueModel::COL_COMPOSER);
+    case COL_PERFORMER: return PlayQueueModel::headerText(PlayQueueModel::COL_PERFORMER);
+    default:            return QString();
     }
-    return instance;
-    #endif
 }
+#endif
+
+GLOBAL_STATIC(PlaylistsModel, instance)
 
 PlaylistsModel::PlaylistsModel(QObject *parent)
     : ActionModel(parent)
+    , enabled(true)
+    , multiCol(false)
+    #ifndef ENABLE_UBUNTU
     , itemMenu(0)
     , dropAdjust(0)
+    #endif
 {
+    connect(MPDConnection::self(), SIGNAL(stateChanged(bool)), SLOT(mpdConnectionStateChanged(bool)));
     connect(MPDConnection::self(), SIGNAL(playlistsRetrieved(const QList<Playlist> &)), this, SLOT(setPlaylists(const QList<Playlist> &)));
     connect(MPDConnection::self(), SIGNAL(playlistInfoRetrieved(const QString &, const QList<Song> &)), this, SLOT(playlistInfoRetrieved(const QString &, const QList<Song> &)));
     connect(MPDConnection::self(), SIGNAL(removedFromPlaylist(const QString &, const QList<quint32> &)),
@@ -74,26 +92,34 @@ PlaylistsModel::PlaylistsModel(QObject *parent)
     connect(this, SIGNAL(listPlaylists()), MPDConnection::self(), SLOT(listPlaylists()));
     connect(this, SIGNAL(playlistInfo(const QString &)), MPDConnection::self(), SLOT(playlistInfo(const QString &)));
     connect(this, SIGNAL(addToPlaylist(const QString &, const QStringList, quint32, quint32)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList, quint32, quint32)));
-    connect(this, SIGNAL(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)), MPDConnection::self(), SLOT(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)));
+    connect(this, SIGNAL(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)), MPDConnection::self(), SLOT(moveInPlaylist(const QString &, const
+    QList<quint32> &, quint32, quint32)));
+    #ifndef ENABLE_UBUNTU
     newAction=new QAction(Icon("document-new"), i18n("New Playlist..."), this);
     connect(newAction, SIGNAL(triggered(bool)), this, SIGNAL(addToNew()));
     Action::initIcon(newAction);
-    updateItemMenu();
+    #endif
+    #if defined ENABLE_MODEL_TEST
+    new ModelTest(this, this);
+    #endif
 }
 
 PlaylistsModel::~PlaylistsModel()
 {
-    itemMenu->deleteLater();
-    itemMenu=0;
-}
-
-QVariant PlaylistsModel::headerData(int /*section*/, Qt::Orientation /*orientation*/, int /*role*/) const
-{
-    return QVariant();
+    #ifndef ENABLE_UBUNTU
+    if (itemMenu) {
+        itemMenu->deleteLater();
+        itemMenu=0;
+    }
+    #endif
 }
 
 int PlaylistsModel::rowCount(const QModelIndex &index) const
 {
+    if (index.column()>0) {
+        return 0;
+    }
+
     if (!index.isValid()) {
         return items.size();
     }
@@ -101,14 +127,30 @@ int PlaylistsModel::rowCount(const QModelIndex &index) const
     Item *item=static_cast<Item *>(index.internalPointer());
     if (item->isPlaylist()) {
         PlaylistItem *pl=static_cast<PlaylistItem *>(index.internalPointer());
-        if (!pl->loaded) {
-            pl->loaded=true;
-            emit playlistInfo(pl->name);
-        }
         return pl->songs.count();
     }
     return 0;
-    //return item->isPlaylist() ? static_cast<PlaylistItem *>(item)->songs.count() : 0;
+}
+
+bool PlaylistsModel::canFetchMore(const QModelIndex &index) const
+{
+    if (!index.isValid()) {
+        return false;
+    }
+    Item *item=static_cast<Item *>(index.internalPointer());
+    return item && item->isPlaylist() && !static_cast<PlaylistItem *>(item)->loaded;
+}
+
+void PlaylistsModel::fetchMore(const QModelIndex &index)
+{
+    if (!index.isValid()) {
+        return;
+    }
+    Item *item=static_cast<Item *>(index.internalPointer());
+    if (item->isPlaylist() && !static_cast<PlaylistItem *>(item)->loaded) {
+        static_cast<PlaylistItem *>(item)->loaded=true;
+        emit playlistInfo(static_cast<PlaylistItem *>(item)->name);
+    }
 }
 
 bool PlaylistsModel::hasChildren(const QModelIndex &parent) const
@@ -155,59 +197,155 @@ QModelIndex PlaylistsModel::index(int row, int col, const QModelIndex &parent) c
     return row<items.count() ? createIndex(row, col, items.at(row)) : QModelIndex();
 }
 
+#ifndef ENABLE_UBUNTU
+QVariant PlaylistsModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if (Qt::Horizontal==orientation) {
+        switch (role) {
+        case Qt::DisplayRole:
+            return headerText(section);
+        case Qt::TextAlignmentRole:
+            switch (section) {
+            case COL_TITLE:
+            case COL_ARTIST:
+            case COL_ALBUM:
+            case COL_GENRE:
+            case COL_COMPOSER:
+            case COL_PERFORMER:
+            default:
+                return int(Qt::AlignVCenter|Qt::AlignLeft);
+            case COL_LENGTH:
+            case COL_YEAR:
+                return int(Qt::AlignVCenter|Qt::AlignRight);
+            }
+        case Cantata::Role_Hideable:
+            return COL_YEAR==section || COL_GENRE==section || COL_COMPOSER==section || COL_PERFORMER==section ? true : false;
+        case Cantata::Role_Width:
+            switch (section) {
+            case COL_TITLE:     return 0.4;
+            case COL_ARTIST:    return 0.15;
+            case COL_ALBUM:     return 0.15;
+            case COL_YEAR:      return 0.05;
+            case COL_GENRE:     return 0.125;
+            case COL_LENGTH:    return 0.125;
+            case COL_COMPOSER:  return 0.15;
+            case COL_PERFORMER: return 0.15;
+            }
+        default:
+            break;
+        }
+    }
+
+    return QVariant();
+}
+#endif
+
 QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
         return QVariant();
     }
 
+    if (Qt::TextAlignmentRole==role) {
+        switch (index.column()) {
+        case COL_TITLE:
+        case COL_ARTIST:
+        case COL_ALBUM:
+        case COL_GENRE:
+        case COL_COMPOSER:
+        case COL_PERFORMER:
+        default:
+            return int(Qt::AlignVCenter|Qt::AlignLeft);
+        case COL_LENGTH:
+        case COL_YEAR:
+            return int(Qt::AlignVCenter|Qt::AlignRight);
+        }
+    }
+
     Item *item=static_cast<Item *>(index.internalPointer());
 
     if (item->isPlaylist()) {
         PlaylistItem *pl=static_cast<PlaylistItem *>(item);
 
         switch(role) {
-        case GroupedView::Role_IsCollection:
+        #ifdef ENABLE_UBUNTU
+        case Cantata::Role_Image:
+            return QString();
+        #endif
+        case Cantata::Role_IsCollection:
             return true;
-        case GroupedView::Role_CollectionId:
+        case Cantata::Role_CollectionId:
             return pl->key;
-        case GroupedView::Role_Key:
+        case Cantata::Role_Key:
             return 0;
-        case GroupedView::Role_Song: {
+        case Cantata::Role_Song: {
             QVariant var;
             var.setValue<Song>(Song());
             return var;
         }
-        case GroupedView::Role_AlbumDuration:
+        case Cantata::Role_AlbumDuration:
             return pl->totalTime();
-        case GroupedView::Role_SongCount:
-            return pl->songs.count();
-        case GroupedView::Role_CurrentStatus:
-        case GroupedView::Role_Status:
-            return (int)GroupedView::State_Default;
-        case Qt::DisplayRole:
+        case Cantata::Role_SongCount:
             if (!pl->loaded) {
                 pl->loaded=true;
                 emit playlistInfo(pl->name);
             }
-            return pl->name;
+            return pl->songs.count();
+        case Cantata::Role_CurrentStatus:
+        case Cantata::Role_Status:
+            return (int)GroupedView::State_Default;
+        case Qt::FontRole:
+            if (multiCol) {
+                QFont font;
+                font.setBold(true);
+                return font;
+            }
+            return QVariant();
+        case Cantata::Role_TitleText:
+        case Cantata::Role_MainText:
+        case Qt::DisplayRole:
+            if (multiCol) {
+                switch (index.column()) {
+                case COL_TITLE:
+                    return pl->visibleName();
+                case COL_ARTIST:
+                case COL_ALBUM:
+                    return QVariant();
+                case COL_LENGTH:
+                    if (!pl->loaded) {
+                        pl->loaded=true;
+                        emit playlistInfo(pl->name);
+                    }
+                    return pl->loaded && !pl->isSmartPlaylist ? Utils::formatTime(pl->totalTime()) : QVariant();
+                case COL_YEAR:
+                case COL_GENRE:
+                    return QVariant();
+                default:
+                    break;
+                }
+            }
+            return pl->visibleName();
         case Qt::ToolTipRole:
+            if (!pl->loaded) {
+                pl->loaded=true;
+                emit playlistInfo(pl->name);
+            }
             return 0==pl->songs.count()
-                ? pl->name
-                : pl->name+"\n"+
-                    #ifdef ENABLE_KDE_SUPPORT
-                    i18np("1 Track (%2)", "%1 Tracks (%2)", pl->songs.count(), Song::formattedTime(pl->totalTime()));
-                    #else
-                    QTP_TRACKS_DURATION_STR(pl->songs.count(), Song::formattedTime(pl->totalTime()));
-                    #endif
+                ? pl->visibleName()
+                : pl->visibleName()+"\n"+Plurals::tracksWithDuration(pl->songs.count(), Utils::formatTime(pl->totalTime()));
+        #ifndef ENABLE_UBUNTU
         case Qt::DecorationRole:
-            return Icons::self()->playlistIcon;
-        case ItemView::Role_SubText:
-            #ifdef ENABLE_KDE_SUPPORT
-            return i18np("1 Track (%2)", "%1 Tracks (%2)", pl->songs.count(), Song::formattedTime(pl->totalTime()));
-            #else
-            return QTP_TRACKS_DURATION_STR(pl->songs.count(), Song::formattedTime(pl->totalTime()));
-            #endif
+            return multiCol ? QVariant() : (pl->isSmartPlaylist ? Icons::self()->dynamicRuleIcon : Icons::self()->playlistIcon);
+        #endif
+        case Cantata::Role_SubText:
+            if (!pl->loaded) {
+                pl->loaded=true;
+                emit playlistInfo(pl->name);
+            }
+            if (pl->isSmartPlaylist) {
+                return i18n("Smart Playlist");
+            }
+            return Plurals::tracksWithDuration(pl->songs.count(), Utils::formatTime(pl->totalTime()));
         default:
             return ActionModel::data(index, role);
         }
@@ -215,18 +353,22 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
         SongItem *s=static_cast<SongItem *>(item);
 
         switch (role) {
-        case GroupedView::Role_IsCollection:
+        #ifdef ENABLE_UBUNTU
+        case Cantata::Role_Image:
+            return QString();
+        #endif
+        case Cantata::Role_IsCollection:
             return false;
-        case GroupedView::Role_CollectionId:
+        case Cantata::Role_CollectionId:
             return s->parent->key;
-        case GroupedView::Role_Key:
+        case Cantata::Role_Key:
             return s->key;
-        case GroupedView::Role_Song: {
+        case Cantata::Role_Song: {
             QVariant var;
             var.setValue<Song>(*s);
             return var;
         }
-        case GroupedView::Role_AlbumDuration: {
+        case Cantata::Role_AlbumDuration: {
             quint32 d=s->time;
             for (int i=index.row()+1; i<s->parent->songs.count(); ++i) {
                 const SongItem *song = s->parent->songs.at(i);
@@ -246,7 +388,7 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             }
             return d;
         }
-        case GroupedView::Role_SongCount:{
+        case Cantata::Role_SongCount:{
             quint32 count=1;
             for (int i=index.row()+1; i<s->parent->songs.count(); ++i) {
                 const SongItem *song = s->parent->songs.at(i);
@@ -266,28 +408,52 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             }
             return count;
         }
-        case GroupedView::Role_CurrentStatus:
-        case GroupedView::Role_Status:
+        case Cantata::Role_CurrentStatus:
+        case Cantata::Role_Status:
             return (int)GroupedView::State_Default;
         case Qt::DisplayRole:
-        case Qt::ToolTipRole: {
-            QString text=s->entryName();
-
-            if (Qt::ToolTipRole==role) {
-                text=text.replace("\n", "<br/>");
-                if (!s->title.isEmpty()) {
-                    text+=QLatin1String("<br/>")+Song::formattedTime(s->time);
-                    text+=QLatin1String("<br/><small><i>")+s->file+QLatin1String("</i></small>");
+            if (multiCol) {
+                switch (index.column()) {
+                case COL_TITLE:
+                    return s->trackAndTitleStr(false);
+                case COL_ARTIST:
+                    return s->artist.isEmpty() ? Song::unknown() : s->artist;
+                case COL_ALBUM:
+                    if (s->isStream() && s->album.isEmpty()) {
+                        QString n=s->name();
+                        if (!n.isEmpty()) {
+                            return n;
+                        }
+                    }
+                    return s->album;
+                case COL_LENGTH:
+                    return Utils::formatTime(s->time);
+                case COL_YEAR:
+                    if (s->year <= 0) {
+                        return QVariant();
+                    }
+                    return s->year;
+                case COL_GENRE:
+                    return s->genre;
+                case COL_COMPOSER:
+                    return s->composer();
+                case COL_PERFORMER:
+                    return s->performer();
+                default:
+                    break;
                 }
             }
-            return text;
-        }
+            return s->entryName();
+        case Qt::ToolTipRole:
+            return s->toolTip();
+        #ifndef ENABLE_UBUNTU
         case Qt::DecorationRole:
-            return s->title.isEmpty() ? Icons::self()->streamIcon : Icon("audio-x-generic");
-        case ItemView::Role_MainText:
+            return multiCol ? QVariant() : (s->title.isEmpty() ? Icons::self()->streamIcon : Icons::self()->audioFileIcon);
+        #endif
+        case Cantata::Role_MainText:
             return s->title.isEmpty() ? s->file : s->title;
-        case ItemView::Role_SubText:
-            return s->artist+QLatin1String(" - ")+s->album;
+        case Cantata::Role_SubText:
+            return s->artist+QLatin1String(" - ")+s->displayAlbum();
         default:
             return ActionModel::data(index, role);
         }
@@ -296,29 +462,24 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
+#ifndef ENABLE_UBUNTU
 bool PlaylistsModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
-    if (GroupedView::Role_DropAdjust==role) {
+    if (Cantata::Role_DropAdjust==role) {
         dropAdjust=value.toUInt();
         return true;
     } else {
         return QAbstractItemModel::setData(index, value, role);
     }
 }
+#endif
 
 Qt::ItemFlags PlaylistsModel::flags(const QModelIndex &index) const
 {
-    Q_UNUSED(index)
-    return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsDropEnabled;
-//     if (index.isValid()) {
-//         if (static_cast<Item *>(index.internalPointer())->isPlaylist()) {
-//             return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsDropEnabled;
-//         } else {
-//             return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsDropEnabled;
-//         }
-//     } else {
-//         return 0;
-//     }
+    if (index.isValid()) {
+        return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsDropEnabled;
+    }
+    return Qt::NoItemFlags;
 }
 
 Qt::DropActions PlaylistsModel::supportedDropActions() const
@@ -377,6 +538,7 @@ QList<Song> PlaylistsModel::songs(const QModelIndexList &indexes) const
     return songs;
 }
 
+#ifndef ENABLE_UBUNTU
 static const QLatin1String constPlaylistNameMimeType("cantata/playlistnames");
 static const QLatin1String constPositionsMimeType("cantata/positions");
 static const char *constPlaylistProp="hasPlaylist";
@@ -386,9 +548,13 @@ QMimeData * PlaylistsModel::mimeData(const QModelIndexList &indexes) const
     QMimeData *mimeData = new QMimeData();
     QStringList filenames;
     QStringList playlists;
-    QStringList positions;
+    QList<quint32> positions;
     QSet<Item *> selectedPlaylists;
     foreach(QModelIndex index, indexes) {
+        if (!index.isValid() || 0!=index.column()) {
+            continue;
+        }
+
         Item *item=static_cast<Item *>(index.internalPointer());
 
         if (item->isPlaylist()) {
@@ -397,13 +563,13 @@ QMimeData * PlaylistsModel::mimeData(const QModelIndexList &indexes) const
             foreach (const SongItem *s, static_cast<PlaylistItem*>(item)->songs) {
                 filenames << s->file;
                 playlists << static_cast<PlaylistItem*>(item)->name;
-                positions << QString::number(pos++);
+                positions << pos++;
             }
             mimeData->setProperty(constPlaylistProp, true);
         } else if (!selectedPlaylists.contains(static_cast<SongItem*>(item)->parent)) {
             filenames << static_cast<SongItem*>(item)->file;
             playlists << static_cast<SongItem*>(item)->parent->name;
-            positions << QString::number(static_cast<SongItem*>(item)->parent->songs.indexOf(static_cast<SongItem*>(item)));
+            positions << static_cast<SongItem*>(item)->parent->songs.indexOf(static_cast<SongItem*>(item));
         }
     }
 
@@ -475,27 +641,8 @@ bool PlaylistsModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
                 }
             }
             if (fromThis) {
-//                 if (!item->isPlaylist()) {
-                    QStringList list=PlayQueueModel::decode(*data, constPositionsMimeType);
-                    QList<quint32> pos;
-
-                    foreach (const QString &s, list) {
-                        pos.append(s.toUInt());
-                    }
-//                     qSort(pos);
-//                     int offset=0;
-//                     foreach (int p, pos) {
-//                         int dest=parent.row();
-//                         int source=p;
-//
-//                         if (dest>source) {
-//                             source-=offset++;
-//                         }
-                        emit moveInPlaylist(pl->name, pos, row < 0 ? pl->songs.size() : row, pl->songs.size());
-//                         emit moveInPlaylist(pl->name, source, dest);
-//                     }
-                    return true;
-//                 }
+                emit moveInPlaylist(pl->name, PlayQueueModel::decodeInts(*data, constPositionsMimeType), row < 0 ? pl->songs.size() : row, pl->songs.size());
+                return true;
             } else {
                 emit addToPlaylist(pl->name, filenames,
                                    row < 0 || (origRow<0 && item->isPlaylist()) ? 0 : row,
@@ -514,10 +661,14 @@ QStringList PlaylistsModel::mimeTypes() const
     types << PlayQueueModel::constFileNameMimeType;
     return types;
 }
+#endif
 
 void PlaylistsModel::getPlaylists()
 {
-    emit listPlaylists();
+// ALWAYS need to send listPlaylists() -> as it is now used for favourite streams!
+//    if (enabled) {
+        emit listPlaylists();
+//    }
 }
 
 void PlaylistsModel::clear()
@@ -528,20 +679,56 @@ void PlaylistsModel::clear()
     endResetModel();
 }
 
+void PlaylistsModel::setEnabled(bool e)
+{
+    if (e==enabled) {
+        return;
+    }
+
+    enabled=e;
+    if (enabled) {
+        getPlaylists();
+    } else {
+        clear();
+    }
+}
+
+#ifndef ENABLE_UBUNTU
+QMenu * PlaylistsModel::menu()
+{
+    if (!itemMenu) {
+        updateItemMenu(true);
+    }
+    return itemMenu;
+}
+#endif
+
 void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
 {
+    if (!enabled) {
+        return;
+    }
+
     if (items.isEmpty()) {
         if (playlists.isEmpty()) {
             return;
         }
         beginResetModel();
         foreach (const Playlist &p, playlists) {
-            items.append(new PlaylistItem(p, allocateKey()));
+            if (p.name!=StreamsModel::constPlayListName) {
+                items.append(new PlaylistItem(p, allocateKey()));
+            }
         }
         endResetModel();
         updateItemMenu();
+        #ifdef ENABLE_UBUNTU
+        emit updated();
+        #endif
     } else if (playlists.isEmpty()) {
         clear();
+        #ifdef ENABLE_UBUNTU
+        emit updated();
+        #endif
     } else {
         QModelIndex parent=QModelIndex();
         QSet<QString> existing;
@@ -554,12 +741,15 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         }
 
         foreach (const Playlist &p, playlists) {
+            if (p.name==StreamsModel::constPlayListName) {
+                continue;
+            }
             retreived.insert(p.name);
             PlaylistItem *pl=getPlaylist(p.name);
 
             if (pl && pl->lastModified<p.lastModified) {
                 pl->lastModified=p.lastModified;
-                if (pl->loaded) {
+                if (pl->loaded && !pl->isSmartPlaylist) {
                     emit playlistInfo(pl->name);
                 }
             }
@@ -594,6 +784,9 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
 
         if (added.count() || removed.count()) {
             updateItemMenu();
+            #ifdef ENABLE_UBUNTU
+            emit updated();
+            #endif
         }
     }
 }
@@ -651,6 +844,7 @@ void PlaylistsModel::playlistInfoRetrieved(const QString &name, const QList<Song
         }
         pl->updateGenres();
         emit updated(idx);
+        emit dataChanged(idx, idx);
     } else {
         emit listPlaylists();
     }
@@ -743,9 +937,22 @@ void PlaylistsModel::playlistRenamed(const QString &from, const QString &to)
     }
 }
 
-void PlaylistsModel::updateItemMenu()
+void PlaylistsModel::mpdConnectionStateChanged(bool connected)
 {
+    if (!connected) {
+        clear();
+    } else {
+        getPlaylists();
+    }
+}
+
+void PlaylistsModel::updateItemMenu(bool create)
+{
+    #ifndef ENABLE_UBUNTU
     if (!itemMenu) {
+        if (!create) {
+            return;
+        }
         itemMenu = new QMenu(0);
     }
 
@@ -753,12 +960,15 @@ void PlaylistsModel::updateItemMenu()
     itemMenu->addAction(newAction);
     QStringList names;
     foreach (const PlaylistItem *p, items) {
-        names << p->name;
+        if (!p->isSmartPlaylist) {
+            names << p->name;
+        }
     }
     qSort(names.begin(), names.end(), PlaylistsProxyModel::compareNames);
     foreach (const QString &n, names) {
         itemMenu->addAction(n, this, SLOT(emitAddToExisting()));
     }
+    #endif
 }
 
 PlaylistsModel::PlaylistItem * PlaylistsModel::getPlaylist(const QString &name)
@@ -809,6 +1019,18 @@ quint32 PlaylistsModel::allocateKey()
     return 0xFFFFFFFF;
 }
 
+PlaylistsModel::PlaylistItem::PlaylistItem(const Playlist &pl, quint32 k)
+    : name(pl.name)
+    , time(0)
+    , key(k)
+    , lastModified(pl.lastModified)
+{
+    loaded=isSmartPlaylist=MPDConnection::self()->isMopdidy() && name.startsWith("Smart Playlist:");
+    if (isSmartPlaylist) {
+        shortName=name.mid(16);
+    }
+}
+
 PlaylistsModel::PlaylistItem::~PlaylistItem()
 {
     clearSongs();
