@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,8 +23,8 @@
 
 #include "magnatunesettingsdialog.h"
 #include "magnatuneservice.h"
-#include "localize.h"
-#include "buddylabel.h"
+#include "support/localize.h"
+#include "support/buddylabel.h"
 #include <QFormLayout>
 
 MagnatuneSettingsDialog::MagnatuneSettingsDialog(QWidget *parent)
