@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,12 +26,13 @@
 
 #include "ui_onlineservicespage.h"
 #include "onlineservice.h"
-#include "musiclibraryproxymodel.h"
+#include "models/musiclibraryproxymodel.h"
+#include "gui/page.h"
 
 class Action;
 class QAction;
 
-class OnlineServicesPage : public QWidget, public Ui::OnlineServicesPage
+class OnlineServicesPage : public QWidget, public Ui::OnlineServicesPage, public Page
 {
     Q_OBJECT
 
@@ -41,14 +42,13 @@ public:
 
     void setEnabled(bool e);
     void clear();
-    QString activeService() const;
     QStringList selectedFiles() const;
-    QList<Song> selectedSongs() const;
-    void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0);
+    QList<Song> selectedSongs(bool allowPlaylists=false) const;
+    void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0, bool randomAlbums=false);
     void setView(int v) { view->setMode((ItemView::Mode)v); }
     void focusSearch() { view->focusSearch(); }
-    void goBack() { view->backActivated(); }
     void refresh();
+    void showEvent(QShowEvent *e);
 
 public Q_SLOTS:
     void itemDoubleClicked(const QModelIndex &);
@@ -60,7 +60,15 @@ public Q_SLOTS:
     void updateGenres(const QModelIndex &idx);
     void setSearchable(const QModelIndex &idx);
     void updated(const QModelIndex &idx);
+//    void sortList();
     void download();
+    void subscribe();
+    void unSubscribe();
+    void refreshSubscription();
+    void downloadPodcast();
+    void deleteDownloadedPodcast();
+    void showPreferencesPage();
+    void providersChanged();
 
 Q_SIGNALS:
     // These are for communicating with MPD object (which is in its own thread, so need to talk via signal/slots)
@@ -68,17 +76,22 @@ Q_SIGNALS:
     void addSongsToPlaylist(const QString &name, const QStringList &files);
 
     void addToDevice(const QString &from, const QString &to, const QList<Song> &songs);
+    void showPreferencesPage(const QString &page);
 
 private:
     OnlineService * activeSrv() const;
+    void expandPodcasts();
 
 private:
     MusicLibraryProxyModel proxy;
     Action *downloadAction;
+    Action *downloadPodcastAction;
+    Action *deleteDownloadedPodcastAction;
     QSet<QString> genres;
     bool onlineSearchRequest;
     QString searchService;
     bool searchable;
+    bool expanded;
 };
 
 #endif
