@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,19 +23,19 @@
 
 #include "rgdialog.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "device.h"
+#include "devices/device.h"
+#include "models/devicesmodel.h"
 #endif
-#include "devicesmodel.h"
-#include "settings.h"
-#include "tags.h"
+#include "gui/settings.h"
+#include "tags/tags.h"
 #include "tagreader.h"
-#include "utils.h"
-#include "localize.h"
-#include "messagebox.h"
+#include "support/utils.h"
+#include "support/localize.h"
+#include "support/messagebox.h"
 #include "jobcontroller.h"
-#include "basicitemdelegate.h"
-#include "action.h"
-#include "cuefile.h"
+#include "widgets/basicitemdelegate.h"
+#include "support/action.h"
+#include "mpd/cuefile.h"
 #include <QComboBox>
 #include <QTreeWidget>
 #include <QLabel>
@@ -45,18 +45,6 @@
 #include <QCloseEvent>
 #include <QCoreApplication>
 #include <QEventLoop>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KLocale>
-#endif
-
-static inline QString formatNumber(double number, int precision)
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return KGlobal::locale()->formatNumber(number, precision);
-    #else
-    return QString::number(number, 'f', precision);
-    #endif
-}
 
 enum Columns
 {
@@ -71,40 +59,22 @@ enum Columns
 
 static int iCount=0;
 
-static inline QString createAlbumName(const Song &s)
-{
-    return s.albumArtist()+QLatin1String(" - ")+s.album;
-}
-
 int RgDialog::instanceCount()
 {
     return iCount;
 }
 
-#if QT_VERSION < 0x050000
 static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
 {
+    #if QT_VERSION < 0x050000
     hdr->setResizeMode(idx, mode);
-}
-
-static inline void setResizeMode(QHeaderView *hdr, QHeaderView::ResizeMode mode)
-{
-    hdr->setResizeMode(mode);
-}
-#else
-static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
-{
+    #else
     hdr->setSectionResizeMode(idx, mode);
+    #endif
 }
 
-static inline void setResizeMode(QHeaderView *hdr, QHeaderView::ResizeMode mode)
-{
-    hdr->setSectionResizeMode(mode);
-}
-#endif
-
 RgDialog::RgDialog(QWidget *parent)
-    : Dialog(parent, "RgDialog", QSize(800, 400))
+    : SongDialog(parent, "RgDialog", QSize(800, 400))
     , state(State_Idle)
     , totalToScan(0)
     , tagReader(0)
@@ -199,6 +169,7 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
 
     autoScanTags=autoScan;
     qSort(origSongs);
+
     #ifdef ENABLE_DEVICES_SUPPORT
     if (udi.isEmpty()) {
         base=MPDConnection::self()->getDetails().dir;
@@ -213,9 +184,13 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
         base=dev->path();
     }
     #else
-    Q_UNUSED(udi)
     base=MPDConnection::self()->getDetails().dir;
     #endif
+
+    if (!songsOk(origSongs, base, udi.isEmpty())) {
+        return;
+    }
+
     state=State_Idle;
     enableButton(User1, origSongs.count());
     view->clear();
@@ -236,9 +211,10 @@ void RgDialog::slotButtonClicked(int button)
     case Ok:
         if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Update ReplayGain tags in tracks?"), i18n("Update Tags"),
                                                        GuiItem(i18n("Update Tags")), StdGuiItem::cancel())) {
-            saveTags();
-            stopScanning();
-            accept();
+            if (saveTags()) {
+                stopScanning();
+                accept();
+            }
         }
         break;
     case User1:
@@ -332,7 +308,7 @@ void RgDialog::createScanner(const QList<int> &indexes)
 {
     QMap<int, QString> fileMap;
     foreach (int i, indexes) {
-        fileMap[i]=base+origSongs.at(i).file;
+        fileMap[i]=base+origSongs.at(i).filePath();
     }
 
     AlbumScanner *s=new AlbumScanner(fileMap);
@@ -388,7 +364,7 @@ void RgDialog::stopReadingTags()
     autoScanTags=false;
 }
 
-void RgDialog::saveTags()
+bool RgDialog::saveTags()
 {
     state=State_Saving;
     enableButton(Ok, false);
@@ -402,13 +378,23 @@ void RgDialog::saveTags()
     progress->setRange(0, tagsToSave.count());
 
     int count=0;
+    bool someTimedout=false;
     QMap<int, Tags::ReplayGain>::ConstIterator it=tagsToSave.constBegin();
     QMap<int, Tags::ReplayGain>::ConstIterator end=tagsToSave.constEnd();
 
     for (; it!=end; ++it) {
-        if (Tags::Update_Failed==Tags::updateReplaygain(base+origSongs.at(it.key()).file, it.value())) {
-            failed.append(origSongs.at(it.key()).file);
+        QString filePath=origSongs.at(it.key()).filePath();
+        switch (Tags::updateReplaygain(base+filePath, it.value())) {
+        case Tags::Update_Failed:
+            failed.append(filePath);
+            break;
+        case Tags::Update_BadFile:
+            failed.append(i18nc("filename (Corrupt tags?)", "%1 (Corrupt tags?)", filePath));
+            break;
+        default:
+            break;
         }
+
         progress->setValue(progress->value()+1);
         if (0==count++%10) {
             QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
@@ -418,6 +404,8 @@ void RgDialog::saveTags()
     if (failed.count()) {
         MessageBox::errorListEx(this, i18n("Failed to update the tags of the following tracks:"), failed);
     }
+
+    return !someTimedout;
 }
 
 void RgDialog::updateView()
@@ -495,15 +483,15 @@ void RgDialog::scannerDone()
             Tags::ReplayGain updatedTags(it.value().gain, s->albumValues().gain, it.value().peak, s->albumValues().peak);
             QTreeWidgetItem *item=view->topLevelItem(it.key());
             if (it.value().ok) {
-                item->setText(COL_TRACKGAIN, i18n("%1 dB", formatNumber(updatedTags.trackGain, 2)));
-                item->setText(COL_TRACKPEAK, formatNumber(updatedTags.trackPeak, 6));
+                item->setText(COL_TRACKGAIN, i18n("%1 dB", Utils::formatNumber(updatedTags.trackGain, 2)));
+                item->setText(COL_TRACKPEAK, Utils::formatNumber(updatedTags.trackPeak, 6));
             } else {
                 item->setText(COL_TRACKGAIN, i18n("Failed"));
                 item->setText(COL_TRACKPEAK, i18n("Failed"));
             }
             if (s->albumValues().ok) {
-                item->setText(COL_ALBUMGAIN, i18n("%1 dB", formatNumber(updatedTags.albumGain, 2)));
-                item->setText(COL_ALBUMPEAK, formatNumber(updatedTags.albumPeak, 6));
+                item->setText(COL_ALBUMGAIN, i18n("%1 dB", Utils::formatNumber(updatedTags.albumGain, 2)));
+                item->setText(COL_ALBUMPEAK, Utils::formatNumber(updatedTags.albumPeak, 6));
             } else {
                 item->setText(COL_ALBUMGAIN, i18n("Failed"));
                 item->setText(COL_ALBUMPEAK, i18n("Failed"));
@@ -515,18 +503,22 @@ void RgDialog::scannerDone()
                 bool diffAlbum=false;
                 if (!Utils::equal(t.trackGain, updatedTags.trackGain, 0.01)) {
                     item->setFont(COL_TRACKGAIN, italic);
+                    item->setToolTip(COL_TRACKGAIN, i18n("Original: %1 dB", Utils::formatNumber(t.trackGain, 2)));
                     diff=true;
                 }
                 if (!Utils::equal(t.trackPeak, updatedTags.trackPeak, 0.000001)) {
                     item->setFont(COL_TRACKPEAK, italic);
+                    item->setToolTip(COL_TRACKPEAK, i18n("Original: %1", Utils::formatNumber(t.trackPeak, 6)));
                     diff=true;
                 }
                 if (!Utils::equal(t.albumGain, updatedTags.albumGain, 0.01)) {
                     item->setFont(COL_ALBUMGAIN, italic);
+                    item->setToolTip(COL_ALBUMGAIN, i18n("Original: %1 dB", Utils::formatNumber(t.albumGain, 2)));
                     diffAlbum=true;
                 }
                 if (!Utils::equal(t.albumPeak, updatedTags.albumPeak, 0.000001)) {
                     item->setFont(COL_ALBUMPEAK, italic);
+                    item->setToolTip(COL_ALBUMPEAK, i18n("Original: %1", Utils::formatNumber(t.albumPeak, 6)));
                     diffAlbum=true;
                 }
                 if (diff || diffAlbum) {
@@ -581,10 +573,10 @@ void RgDialog::songTags(int index, Tags::ReplayGain tags)
             if (!item) {
                 return;
             }
-            item->setText(COL_TRACKGAIN, i18n("%1 dB", formatNumber(tags.trackGain, 2)));
-            item->setText(COL_TRACKPEAK, formatNumber(tags.trackPeak, 6));
-            item->setText(COL_ALBUMGAIN, i18n("%1 dB", formatNumber(tags.albumGain, 2)));
-            item->setText(COL_ALBUMPEAK, formatNumber(tags.albumPeak, 6));
+            item->setText(COL_TRACKGAIN, i18n("%1 dB", Utils::formatNumber(tags.trackGain, 2)));
+            item->setText(COL_TRACKPEAK, Utils::formatNumber(tags.trackPeak, 6));
+            item->setText(COL_ALBUMGAIN, i18n("%1 dB", Utils::formatNumber(tags.albumGain, 2)));
+            item->setText(COL_ALBUMPEAK, Utils::formatNumber(tags.albumPeak, 6));
         }
     }
 }
