@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,17 +23,16 @@
 
 #include "streamspage.h"
 #include "streamdialog.h"
-#include "mpdconnection.h"
-#include "messagebox.h"
-#include "localize.h"
-#include "icons.h"
-#include "stdactions.h"
-#include "actioncollection.h"
-#include "networkaccessmanager.h"
-#include "settings.h"
-#include "streamsmodel.h"
-#include "statuslabel.h"
-#include "digitallyimported.h"
+#include "mpd/mpdconnection.h"
+#include "support/messagebox.h"
+#include "support/localize.h"
+#include "widgets/icons.h"
+#include "gui/stdactions.h"
+#include "support/actioncollection.h"
+#include "network/networkaccessmanager.h"
+#include "gui/settings.h"
+#include "widgets/statuslabel.h"
+#include "models/digitallyimported.h"
 #include "digitallyimportedsettings.h"
 #include <QToolButton>
 #ifdef ENABLE_KDE_SUPPORT
@@ -42,7 +41,6 @@
 #include <QFileDialog>
 #include <QDir>
 #endif
-#include <QAction>
 #include <QMenu>
 #include <QFileInfo>
 #if QT_VERSION >= 0x050000
@@ -53,7 +51,6 @@ static const int constMsgDisplayTime=1500;
 
 StreamsPage::StreamsPage(QWidget *p)
     : QWidget(p)
-    , enabled(false)
     , searching(false)
     , proxy(&streamsProxy)
 {
@@ -66,6 +63,7 @@ StreamsPage::StreamsPage(QWidget *p)
 //     connect(view, SIGNAL(itemsSelected(bool)), addToPlaylist, SLOT(setEnabled(bool)));
     connect(view, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(itemDoubleClicked(const QModelIndex &)));
     connect(view, SIGNAL(searchIsActive(bool)), this, SLOT(controlSearch(bool)));
+    connect(view, SIGNAL(searchItems()), this, SLOT(searchItems()));
     connect(view, SIGNAL(itemsSelected(bool)), SLOT(controlActions()));
     connect(searchView, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(itemDoubleClicked(const QModelIndex &)));
     connect(searchView, SIGNAL(searchItems()), this, SLOT(searchItems()));
@@ -82,10 +80,13 @@ StreamsPage::StreamsPage(QWidget *p)
     connect(StreamsModel::self(), SIGNAL(error(const QString &)), this, SIGNAL(error(const QString &)));
     connect(StreamsModel::self(), SIGNAL(loading()), view, SLOT(showSpinner()));
     connect(StreamsModel::self(), SIGNAL(loaded()), view, SLOT(hideSpinner()));
+    connect(StreamsModel::self(), SIGNAL(categoriesChanged()), view, SLOT(closeSearch()));
+    connect(StreamsModel::self(), SIGNAL(favouritesLoaded()), SLOT(expandFavourites()));
+    connect(StreamsModel::self(), SIGNAL(addedToFavourites(QString)), SLOT(addedToFavourites(QString)));
     connect(&searchModel, SIGNAL(loading()), searchView, SLOT(showSpinner()));
     connect(&searchModel, SIGNAL(loaded()), searchView, SLOT(hideSpinner()));
-    connect(MPDConnection::self(), SIGNAL(dirChanged()), SLOT(mpdDirChanged()));
     connect(DigitallyImported::self(), SIGNAL(loginStatus(bool,QString)), SLOT(updateDiStatus()));
+    connect(DigitallyImported::self(), SIGNAL(updated()), SLOT(updateDiStatus()));
     QMenu *menu=new QMenu(this);
     menu->addAction(addAction);
     menu->addAction(StdActions::self()->removeAction);
@@ -95,6 +96,9 @@ StreamsPage::StreamsPage(QWidget *p)
     menu->addAction(importAction);
     menu->addAction(exportAction);
     menu->addSeparator();
+    QAction *configAction=new QAction(Icons::self()->configureIcon, i18n("Configure..."), this);
+    menu->addAction(configAction);
+    connect(configAction, SIGNAL(triggered(bool)), this, SLOT(showPreferencesPage()));
     menuButton->setMenu(menu);
     Icon::init(replacePlayQueue);
 
@@ -109,6 +113,7 @@ StreamsPage::StreamsPage(QWidget *p)
     streamsProxy.setSourceModel(StreamsModel::self());
     view->setModel(&streamsProxy);
     view->setDeleteAction(StdActions::self()->removeAction);
+    view->setSearchResetLevel(1);
 
     searchView->setUniformRowHeights(true);
     searchView->addAction(StdActions::self()->replacePlayQueueAction);
@@ -116,61 +121,26 @@ StreamsPage::StreamsPage(QWidget *p)
     searchView->addAction(StreamsModel::self()->addBookmarkAct());
     searchProxy.setSourceModel(&searchModel);
     searchView->setModel(&searchProxy);
-    searchView->setBackgroundImage(StreamsModel::self()->tuneInIcon());
 
-    diStatusLabel->setText("DI");
-    updateDiStatus();
-    searchView->setSearchLabelText(i18n("Search TuneIn:"));
+    diStatusLabel->setText("DI", i18nc("Service name", "Digitally Imported"));
+    connect(diStatusLabel, SIGNAL(clicked()), SLOT(diSettings()));
+    updateDiStatus();  
 }
 
 StreamsPage::~StreamsPage()
 {
 }
 
-void StreamsPage::setEnabled(bool e)
+void StreamsPage::showEvent(QShowEvent *e)
 {
-    if (e==enabled) {
-        return;
-    }
-    enabled=e;
+    view->focusView();
+    QWidget::showEvent(e);
 }
 
-void StreamsPage::mpdDirChanged()
-{
-    if (Settings::self()->storeStreamsInMpdDir()) {
-        refresh();
-    }
-}
-
-void StreamsPage::checkWritable()
-{
-    bool wasWriteable=StreamsModel::self()->isFavoritesWritable();
-    bool nowWriteable=StreamsModel::self()->checkFavouritesWritable();
-
-    if (wasWriteable!=nowWriteable) {
-        controlActions();
-    }
-}
-
-void StreamsPage::refresh()
-{
-    if (enabled) {
-        checkWritable();
-        view->setLevel(0);
-        searchView->setLevel(0);
-        StreamsModel::self()->reloadFavourites();
-        exportAction->setEnabled(StreamsModel::self()->rowCount()>0);
-        view->expand(proxy->mapFromSource(StreamsModel::self()->favouritesIndex()));
-    }
-}
-
-void StreamsPage::save()
-{
-    StreamsModel::self()->saveFavourites(true);
-}
-
-void StreamsPage::addSelectionToPlaylist(bool replace, quint8 priorty)
+void StreamsPage::addSelectionToPlaylist(const QString &name, bool replace, quint8 priorty, bool randomAlbums)
 {
+    Q_UNUSED(name)
+    Q_UNUSED(randomAlbums)
     addItemsToPlayQueue(itemView()->selectedIndexes(), replace, priorty);
 }
 
@@ -214,7 +184,7 @@ void StreamsPage::configureStreams()
 
     const StreamsModel::Item *item=static_cast<const StreamsModel::Item *>(proxy->mapToSource(selected.first()).internalPointer());
     if (item->isCategory() && static_cast<const StreamsModel::CategoryItem *>(item)->canConfigure()) {
-        // TODO: In future migh tneed to confirm category type, at the moment only digitially imported can be configured...
+        // TODO: In future might need to confirm category type, at the moment only digitially imported can be configured...
         diSettings();
     }
 }
@@ -222,12 +192,11 @@ void StreamsPage::configureStreams()
 void StreamsPage::diSettings()
 {
     DigitallyImportedSettings(this).show();
-    updateDiStatus();
 }
 
 void StreamsPage::importXml()
 {
-    if (searching || !StreamsModel::self()->isFavoritesWritable()) {
+    if (searching) {
         return;
     }
     #ifdef ENABLE_KDE_SUPPORT
@@ -240,10 +209,7 @@ void StreamsPage::importXml()
     if (fileName.isEmpty()) {
         return;
     }
-
-    if (!StreamsModel::self()->importIntoFavourites(fileName)) {
-        MessageBox::error(this, i18n("Failed to import <b>%1</b>!<br/>Please check this is of the correct type.", fileName));
-    }
+    StreamsModel::self()->importIntoFavourites(fileName);
 }
 
 void StreamsPage::exportXml()
@@ -252,11 +218,11 @@ void StreamsPage::exportXml()
         return;
     }
 
-    QLatin1String ext(".xml");
+    QLatin1String ext(".xml.gz");
     #ifdef ENABLE_KDE_SUPPORT
-    QString fileName=KFileDialog::getSaveFileName(QLatin1String("Cantata")+ext, i18n("*.xml|XML Streams"), this, i18n("Export Streams"));
+    QString fileName=KFileDialog::getSaveFileName(QLatin1String("streams")+ext, i18n("*.xml|XML Streams"), this, i18n("Export Streams"));
     #else
-    QString fileName=QFileDialog::getSaveFileName(this, i18n("Export Streams"), QDir::homePath()+QLatin1String("/Cantata")+ext, i18n("XML Streams (*.xml)"));
+    QString fileName=QFileDialog::getSaveFileName(this, i18n("Export Streams"), QDir::homePath()+QLatin1String("/streams")+ext, i18n("XML Streams (*.xml.gz)"));
     #endif
 
     if (fileName.isEmpty()) {
@@ -274,7 +240,7 @@ void StreamsPage::exportXml()
 
 void StreamsPage::add()
 {
-    if (searching || !StreamsModel::self()->isFavoritesWritable()) {
+    if (searching) {
         return;
     }
     StreamDialog dlg(this);
@@ -292,7 +258,6 @@ void StreamsPage::add()
             StreamsModel::self()->addToFavourites(url, name);
         }
     }
-    exportAction->setEnabled(StreamsModel::self()->haveFavourites());
 }
 
 void StreamsPage::addBookmark()
@@ -315,10 +280,6 @@ void StreamsPage::addBookmark()
 
 void StreamsPage::addToFavourites()
 {
-    if (!StreamsModel::self()->isFavoritesWritable()) {
-        return;
-    }
-
     QModelIndexList selected = itemView()->selectedIndexes();
 
     QList<const StreamsModel::Item *> items;
@@ -349,14 +310,12 @@ void StreamsPage::addToFavourites()
         if (urlStr.endsWith('&')) {
             urlStr=urlStr.left(urlStr.length()-1);
         }
-        if (StreamsModel::self()->addToFavourites(urlStr, item->name)) {
+        if (StreamsModel::self()->addToFavourites(urlStr, item->modifiedName())) {
             added++;
         }
     }
 
-    if (added) {
-        itemView()->showMessage(i18n("Added to favorites"), constMsgDisplayTime);
-    } else {
+    if (!added) {
         itemView()->showMessage(i18n("Already in favorites"), constMsgDisplayTime);
     }
 }
@@ -414,10 +373,6 @@ void StreamsPage::removeItems()
             }
         }
     }
-    
-    if (!StreamsModel::self()->isFavoritesWritable()) {
-        return;
-    }
         
     QModelIndexList useable;
 
@@ -444,16 +399,12 @@ void StreamsPage::removeItems()
         }
     }
 
-    foreach (const QModelIndex &i, useable) {
-        StreamsModel::self()->removeFromFavourites(i);
-    }
-
-    exportAction->setEnabled(StreamsModel::self()->haveFavourites());
+    StreamsModel::self()->removeFromFavourites(useable);
 }
 
 void StreamsPage::edit()
 {
-    if (searching || !StreamsModel::self()->isFavoritesWritable()) {
+    if (searching) {
         return;
     }
 
@@ -485,9 +436,7 @@ void StreamsPage::edit()
         } else if (newName!=name && StreamsModel::self()->nameExistsInFavourites(newName)) {
             MessageBox::error(this, i18n("A stream named <b>%1</b> already exists!", newName));
         } else {
-            item->name=newName;
-            item->url=newUrl;
-            StreamsModel::self()->updateFavouriteStream(item);
+            StreamsModel::self()->updateFavouriteStream(newUrl, newName, index);
         }
     }
 }
@@ -497,7 +446,40 @@ void StreamsPage::searchItems()
     if (!searching) {
         return;
     }
-    searchModel.search(searchView->searchText().trimmed(), false);
+    if (proxy==&searchProxy) {
+        searchModel.search(searchView->searchText().trimmed(), false);
+    } else {
+        QString text=view->searchText().trimmed();
+        if (!view->isSearchActive()) {
+            proxy->setFilterItem(0);
+        }
+        proxy->update(view->isSearchActive() ? text : QString(), QString());
+        if (proxy->enabled() && !text.isEmpty()) {
+            view->expandAll(proxy->filterItem()
+                            ? proxy->mapFromSource(StreamsModel::self()->categoryIndex(static_cast<const StreamsModel::CategoryItem *>(proxy->filterItem())))
+                            : QModelIndex());
+        }
+    }
+}
+
+StreamsModel::CategoryItem * StreamsPage::getSearchCategory()
+{
+    bool srch=searching;
+    searching=false;
+    QModelIndexList selected = itemView()->selectedIndexes(false); // Dont need sorted selection here...
+    searching=srch;
+
+    if (1!=selected.size()) {
+        return StreamsModel::self()->tuneInCat();
+    }
+
+    QModelIndex index=proxy->mapToSource(selected.first());
+    StreamsModel::Item *item=static_cast<StreamsModel::Item *>(index.internalPointer());
+    if (!item) {
+        return StreamsModel::self()->tuneInCat();
+    }
+    StreamsModel::CategoryItem *cat=item->getTopLevelCategory();
+    return 0==cat ? StreamsModel::self()->tuneInCat() : cat;
 }
 
 void StreamsPage::controlSearch(bool on)
@@ -505,14 +487,48 @@ void StreamsPage::controlSearch(bool on)
     if (on!=searching) {
         searching=on;
         if (searching) {
-            proxy=&searchProxy;
+            StreamsModel::CategoryItem *cat=getSearchCategory();
+            StreamSearchModel::Category searchCat=StreamsModel::self()->isTuneIn(cat)
+                                                    ? StreamSearchModel::TuneIn
+                                                    : StreamsModel::self()->isShoutCast(cat)
+                                                        ? StreamSearchModel::ShoutCast
+                                                        : StreamsModel::self()->isDirble(cat)
+                                                          ? StreamSearchModel::Dirble
+                                                          : StreamSearchModel::Filter;
+
+            if (StreamSearchModel::Filter==searchCat) {
+                proxy=&streamsProxy;
+                searchModel.clear();
+                viewStack->setCurrentIndex(0);
+                view->setSearchLabelText(i18n("Search %1:", cat->name));
+                view->setBackgroundImage(cat->icon);
+                proxy->setFilterItem(cat);
+                proxy->update(QString(), QString());
+                QModelIndex filterIndex=cat ? StreamsModel::self()->categoryIndex(cat) : QModelIndex();
+                if (filterIndex.isValid()) {
+                    view->expand(proxy->mapFromSource(filterIndex), true);
+                }
+                // We need to call focusSearch via 'invokeMethod' as we need this to occur at the next event-loop iteration.
+                // This is due to the fact that we have 2 views - search (for TuneIn/ShoutCast) and standard. Our focusSearch()
+                // calls focusSearch on both - which triggers this controlSearch().
+                QMetaObject::invokeMethod(view, "focusSearch", Qt::QueuedConnection);
+            } else {
+                proxy=&searchProxy;
+                searchModel.setCat(searchCat);
+                viewStack->setCurrentIndex(1);
+                searchView->setSearchLabelText(i18n("Search %1:", cat->name));
+                searchView->setBackgroundImage(cat->icon);
+            }
             view->clearSelection();
         } else {
+            proxy->setFilterItem(0);
+            proxy->update(QString(), QString());
             proxy=&streamsProxy;
             searchModel.clear();
             view->setSearchVisible(false);
+            viewStack->setCurrentIndex(0);
+            view->setBackgroundImage(QIcon());
         }
-        viewStack->setCurrentIndex(on ? 1 : 0);
         controlActions();
     }
 }
@@ -523,7 +539,6 @@ void StreamsPage::controlActions()
     bool haveSelection=!selected.isEmpty();
     bool enableAddToFav=true;
     bool onlyStreamsSelected=true;
-    bool favWriteable=StreamsModel::self()->isFavoritesWritable();
     StreamsModel::self()->addBookmarkAct()->setEnabled(false);
     if (searching) {
         foreach (const QModelIndex &idx, selected) {
@@ -534,12 +549,9 @@ void StreamsPage::controlActions()
                 break;
             }
         }
-        addAction->setEnabled(false);
-        exportAction->setEnabled(false);
-        importAction->setEnabled(false);
         StreamsModel::self()->reloadAct()->setEnabled(false);
         StdActions::self()->removeAction->setEnabled(false);
-        StreamsModel::self()->addToFavouritesAct()->setEnabled(favWriteable && haveSelection && enableAddToFav);
+        StreamsModel::self()->addToFavouritesAct()->setEnabled(haveSelection && enableAddToFav);
         StreamsModel::self()->configureAct()->setEnabled(false);
         if (1==selected.size()) {
             StreamsModel::self()->addBookmarkAct()->setEnabled(static_cast<const StreamsModel::Item *>(proxy->mapToSource(selected.first()).internalPointer())
@@ -566,12 +578,12 @@ void StreamsPage::controlActions()
             }
         }
 
-        StdActions::self()->removeAction->setEnabled(favWriteable && haveSelection && enableRemove);
-        StreamsModel::self()->addToFavouritesAct()->setEnabled(favWriteable && haveSelection && enableAddToFav);
+        StdActions::self()->removeAction->setEnabled(haveSelection && enableRemove);
+        StreamsModel::self()->addToFavouritesAct()->setEnabled(haveSelection && enableAddToFav);
 
         if (1==selected.size()) {
             const StreamsModel::Item *item=static_cast<const StreamsModel::Item *>(proxy->mapToSource(selected.first()).internalPointer());
-            if (StreamsModel::self()->isFavoritesWritable() && !item->isCategory() && item->parent && item->parent->isFavourites()) {
+            if (!item->isCategory() && item->parent && item->parent->isFavourites()) {
                 editAction->setEnabled(true);
             }
             StreamsModel::self()->reloadAct()->setEnabled(item->isCategory() && static_cast<const StreamsModel::CategoryItem *>(item)->canReload());
@@ -584,10 +596,6 @@ void StreamsPage::controlActions()
         } else {
             StreamsModel::self()->configureAct()->setEnabled(false);
         }
-
-        addAction->setEnabled(favWriteable);
-        exportAction->setEnabled(StreamsModel::self()->haveFavourites());
-        importAction->setEnabled(favWriteable);
     }
     StdActions::self()->replacePlayQueueAction->setEnabled(haveSelection && onlyStreamsSelected);
     StdActions::self()->addWithPriorityAction->setEnabled(haveSelection && onlyStreamsSelected);
@@ -599,18 +607,22 @@ void StreamsPage::updateDiStatus()
     if (DigitallyImported::self()->user().isEmpty() || DigitallyImported::self()->pass().isEmpty()) {
         diStatusLabel->setVisible(false);
     } else {
-        bool loggedIn=DigitallyImported::self()->loggedIn();
-        diStatusLabel->setVisible(true);
-        diStatusLabel->setToolTip(loggedIn ? i18n("Logged into Digitally Imported") : i18n("<b>NOT</b> logged into Digitally Imported"));
-        QString col=loggedIn
-                        ? palette().highlight().color().name()
-                        : palette().color(QPalette::Disabled, QPalette::WindowText).name();
-
-        int margin=style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, this);
-        if (margin<2) {
-            margin=2;
-        }
-        diStatusLabel->setStyleSheet(QString("QLabel { color : %1; border-radius: %4px; border: 2px solid %2; margin: %3px}")
-                                     .arg(col).arg(col).arg(margin).arg(margin*2));
+        diStatusLabel->setStatus(DigitallyImported::self()->loggedIn());
     }
 }
+
+void StreamsPage::showPreferencesPage()
+{
+    emit showPreferencesPage(QLatin1String("streams"));
+}
+
+void StreamsPage::expandFavourites()
+{
+    view->expand(proxy->mapFromSource(StreamsModel::self()->favouritesIndex()), true);
+}
+
+void StreamsPage::addedToFavourites(const QString &name)
+{
+    itemView()->showMessage(i18n("Added '%1'' to favorites", name), constMsgDisplayTime);
+}
+
