@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,14 +25,16 @@
 #define STREAMSPAGE_H
 
 #include "ui_streamspage.h"
-#include "streamsproxymodel.h"
-#include "streamsearchmodel.h"
+#include "models/streamsproxymodel.h"
+#include "models/streamsearchmodel.h"
+#include "models/streamsmodel.h"
+#include "gui/page.h"
 
 class Action;
 class QAction;
 class QNetworkReply;
 
-class StreamsPage : public QWidget, public Ui::StreamsPage
+class StreamsPage : public QWidget, public Ui::StreamsPage, public Page
 {
     Q_OBJECT
 
@@ -40,24 +42,20 @@ public:
     StreamsPage(QWidget *p);
     virtual ~StreamsPage();
 
-    void setEnabled(bool e);
-    bool isEnabled() const { return enabled; }
-    void save();
-    void addSelectionToPlaylist(bool replace, quint8 priorty=0);
+    void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0, bool randomAlbums=false);
     void setView(int v) { view->setMode((ItemView::Mode)v); searchView->setMode((ItemView::Mode)v); }
     void focusSearch() { view->focusSearch(); searchView->focusSearch(); }
     void goBack() { itemView()->backActivated(); }
     ItemView *itemView() { return searching ? searchView : view; }
+    void showEvent(QShowEvent *e);
 
 Q_SIGNALS:
     void add(const QStringList &streams, bool replace, quint8 priorty);
 
     void error(const QString &str);
+    void showPreferencesPage(const QString &page);
 
 public Q_SLOTS:
-    void mpdDirChanged();
-    void checkWritable();
-    void refresh();
     void removeItems();
     void controlActions();
 
@@ -75,12 +73,15 @@ private Q_SLOTS:
     void controlSearch(bool on);
     void itemDoubleClicked(const QModelIndex &index);
     void updateDiStatus();
+    void showPreferencesPage();
+    void expandFavourites();
+    void addedToFavourites(const QString &name);
 
 private:
     void addItemsToPlayQueue(const QModelIndexList &indexes, bool replace, quint8 priorty=0);
+    StreamsModel::CategoryItem *getSearchCategory();
 
 private:
-    bool enabled;
     bool searching;
     Action *importAction;
     Action *exportAction;
