@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,7 +26,7 @@
 
 #include <QComboBox>
 
-#ifdef Q_OS_WIN
+#if defined Q_OS_WIN || defined Q_OS_MAC
 class ComboBox : public QComboBox { public: ComboBox(QWidget *p) : QComboBox(p) { } };
 #else
 #ifdef ENABLE_KDE_SUPPORT
@@ -44,8 +44,6 @@ public:
     virtual ~ComboBox() { }
 
     void setEditable(bool editable);
-
-private:
     void showPopup();
     void hidePopup();
 
