@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,7 +25,7 @@
 #include "acceleratormanager.h"
 #include <QMenu>
 
-static const char * constAccelProp="catata-accel";
+static const char * constAccelProp="managed-accel";
 
 void ProxyStyle::polish(QWidget *widget)
 {
