@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,35 +23,58 @@
 
 #include "tageditor.h"
 #include "tags.h"
-#include "musiclibrarymodel.h"
-#include "mpdconnection.h"
-#include "settings.h"
-#include "messagebox.h"
-#include "inputdialog.h"
-#include "localize.h"
+#include "widgets/tagspinbox.h"
+#include "models/musiclibrarymodel.h"
+#include "mpd/mpdconnection.h"
+#include "gui/settings.h"
+#include "support/messagebox.h"
+#include "support/inputdialog.h"
+#include "support/localize.h"
 #include "trackorganiser.h"
-#include "cuefile.h"
+#include "mpd/cuefile.h"
+#include "support/utils.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "devicesmodel.h"
-#include "device.h"
+#include "models/devicesmodel.h"
+#include "devices/device.h"
 #endif
 #include <QMenu>
 #include <QCloseEvent>
 #include <QCoreApplication>
 #include <QEventLoop>
 #include <QDir>
+#include <QTimer>
 
-static bool equalTags(const Song &a, const Song &b, bool compareCommon)
+#define REMOVE(w) \
+    w->setVisible(false); \
+    w->deleteLater(); \
+    w=0;
+
+//
+// NOTE: Cantata does NOT read, and store, comments from MPD. For comment support, Cantata will read these from the files when the tag editor is opened
+//       These are placed into the Song's 'name' field (as this is not currently used in this editor)
+//
+static bool equalTags(const Song &a, const Song &b, bool compareCommon, bool composerSupport, bool commentSupport)
 {
     return (compareCommon || a.track==b.track) && a.year==b.year && a.disc==b.disc &&
            a.artist==b.artist && a.genre==b.genre && a.album==b.album &&
-           a.albumartist==b.albumartist && (compareCommon || a.title==b.title);
+           a.albumartist==b.albumartist && (!composerSupport || a.composer()==b.composer()) &&
+           (!commentSupport || a.comment()==b.comment()) && (compareCommon || a.title==b.title);
 }
 
-static void setString(QString &str, const QString &v, bool skipEmpty) {
+static bool setString(QString &str, const QString &v, bool skipEmpty) {
     if (!skipEmpty || !v.isEmpty()) {
         str=v;
+        return true;
     }
+    return false;
+}
+
+static QString trim(QString str) {
+    str=str.trimmed();
+    str=str.simplified();
+    str=str.replace(QLatin1String(" ;"), QLatin1String(";"));
+    str=str.replace(QLatin1String("; "), QLatin1String(";"));
+    return str;
 }
 
 static int iCount=0;
@@ -62,9 +85,9 @@ int TagEditor::instanceCount()
 }
 
 TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
-                     const QSet<QString> &existingArtists, const QSet<QString> &existingAlbumArtists,
+                     const QSet<QString> &existingArtists, const QSet<QString> &existingAlbumArtists, const QSet<QString> &existingComposers,
                      const QSet<QString> &existingAlbums, const QSet<QString> &existingGenres, const QString &udi)
-    : Dialog(parent, "TagEditor", QSize(500, 200))
+    : SongDialog(parent)
     #ifdef ENABLE_DEVICES_SUPPORT
     , deviceUdi(udi)
     #endif
@@ -72,32 +95,24 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     , updating(false)
     , haveArtists(false)
     , haveAlbumArtists(false)
+    , haveComposers(false)
+    , haveComments(false)
     , haveAlbums(false)
     , haveGenres(false)
+    , haveDiscs(false)
+    , haveYears(false)
     , saving(false)
+    , composerSupport(false)
+    , commentSupport(false)
 {
     iCount++;
-    foreach (const Song &s, songs) {
-        if (CueFile::isCue(s.file)) {
-            continue;
-        }
-        if (s.guessed) {
-            Song song(s);
-            song.revertGuessedTags();
-            original.append(song);
-        } else {
-            original.append(s);
-        }
-    }
-
-    if (original.isEmpty()) {
-        deleteLater();
-        return;
-    }
-
+    bool isMopidy=false;
     #ifdef ENABLE_DEVICES_SUPPORT
     if (deviceUdi.isEmpty()) {
         baseDir=MPDConnection::self()->getDetails().dir;
+        composerSupport=MPDConnection::self()->composerTagSupported();
+        commentSupport=MPDConnection::self()->commentTagSupported();
+        isMopidy=MPDConnection::self()->isMopdidy();
     } else {
         Device *dev=getDevice(udi, parentWidget());
 
@@ -109,16 +124,40 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         baseDir=dev->path();
     }
     #else
-    Q_UNUSED(udi)
     baseDir=MPDConnection::self()->getDetails().dir;
+    composerSupport=MPDConnection::self()->composerTagSupported();
+    commentSupport=MPDConnection::self()->commentTagSupported();
+    isMopidy=MPDConnection::self()->isMopdidy();
     #endif
+
+    foreach (const Song &s, songs) {
+        if (CueFile::isCue(s.file)) {
+            continue;
+        }
+        Song song(s);
+        if (s.guessed) {
+            song.revertGuessedTags();
+        }
+        original.append(song);
+    }
+
+    if (original.isEmpty()) {
+        deleteLater();
+        return;
+    }
     qSort(original);
 
+    if (!songsOk(original, baseDir, udi.isEmpty())) {
+        return;
+    }
+
     QWidget *mainWidet = new QWidget(this);
     setupUi(mainWidet);
-    track->setAllowEmpty();
-    disc->setAllowEmpty();
-    year->setAllowEmpty();
+    if (isMopidy) {
+        connect(mopidyNote, SIGNAL(leftClickedUrl()), SLOT(showMopidyMessage()));
+    } else {
+        REMOVE(mopidyNote);
+    }
     setMainWidget(mainWidet);
     ButtonCodes buttons=Ok|Cancel|Reset|User3;
     if (songs.count()>1) {
@@ -157,6 +196,24 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     albumArtist->clear();
     albumArtist->insertItems(0, strings);
 
+    if (composerSupport) {
+        strings=existingComposers.toList();
+        strings.sort();
+        composer->clear();
+        composer->insertItems(0, strings);
+    } else {
+        REMOVE(composer)
+        REMOVE(composerLabel)
+    }
+
+    if (commentSupport) {
+        comment->clear();
+        composer->insertItems(0, strings);
+    } else {
+        REMOVE(comment)
+        REMOVE(commentLabel)
+    }
+
     strings=existingAlbums.toList();
     strings.sort();
     album->clear();
@@ -176,25 +233,27 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         QSet<QString> songAlbumArtists;
         QSet<QString> songAlbums;
         QSet<QString> songGenres;
+        QSet<QString> songComposers;
+        QSet<QString> songComments;
         QSet<int> songYears;
         QSet<int> songDiscs;
 
         foreach (const Song &s, original) {
-            if (!s.artist.isEmpty()) {
-                songArtists.insert(s.artist);
+            songArtists.insert(s.artist);
+            songAlbumArtists.insert(s.albumartist);
+            songAlbums.insert(s.album);
+            songGenres.insert(s.genre);
+            if (composerSupport) {
+                songComposers.insert(s.composer());
             }
-            if (!s.albumartist.isEmpty()) {
-                songAlbumArtists.insert(s.albumartist);
-            }
-            if (!s.album.isEmpty()) {
-                songAlbums.insert(s.album);
-            }
-            if (!s.genre.isEmpty()) {
-                songGenres.insert(s.genre);
+            if (commentSupport) {
+                songComments.insert(s.comment());
             }
             songYears.insert(s.year);
             songDiscs.insert(s.disc);
-            if (songArtists.count()>1 && songAlbumArtists.count()>1 && songAlbums.count()>1 && songGenres.count()>1 && songYears.count()>1 && songDiscs.count()>1) {
+            if (songArtists.count()>1 && songAlbumArtists.count()>1 && songAlbums.count()>1 &&
+                songGenres.count()>1 && songYears.count()>1 && songDiscs.count()>1 &&
+                (!composerSupport || songComposers.count()>1) && (!commentSupport || songComments.count()>1)) {
                 break;
             }
         }
@@ -203,6 +262,10 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         all.title.clear();
         all.track=0;
         all.artist=1==songArtists.count() ? *(songArtists.begin()) : QString();
+        if (1==songComposers.count()) {
+            all.setComposer(*(songComposers.begin()));
+        }
+        all.setComment(1==songComments.count() ? *(songComments.begin()) : QString());
         all.albumartist=1==songAlbumArtists.count() ? *(songAlbumArtists.begin()) : QString();
         all.album=1==songAlbums.count() ? *(songAlbums.begin()) : QString();
         all.genre=1==songGenres.count() ? *(songGenres.begin()) : QString();
@@ -210,10 +273,13 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         all.disc=1==songDiscs.count() ? *(songDiscs.begin()) : 0;
         original.prepend(all);
         artist->setFocus();
-        haveArtists=!songArtists.isEmpty();
-        haveAlbumArtists=!songAlbumArtists.isEmpty();
-        haveAlbums=!songAlbums.isEmpty();
-        haveGenres=!songGenres.isEmpty();
+        haveArtists=songArtists.count()>1;
+        haveAlbumArtists=songAlbumArtists.count()>1;
+        haveAlbums=songAlbums.count()>1;
+        haveGenres=songGenres.count()>1;
+        haveComposers=songComposers.count()>1;
+        haveDiscs=songDiscs.count()>1;
+        haveYears=songYears.count()>1;
     } else {
         title->setFocus();
     }
@@ -225,7 +291,7 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
             trackName->insertItem(trackName->count(), i18n("All tracks"));
             first=false;
         } else {
-            trackName->insertItem(trackName->count(), s.file);
+            trackName->insertItem(trackName->count(), s.filePath());
         }
     }
     connect(title, SIGNAL(textChanged(const QString &)), SLOT(checkChanged()));
@@ -233,6 +299,13 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     connect(artist, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
     connect(albumArtist, SIGNAL(activated(int)), SLOT(checkChanged()));
     connect(albumArtist, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
+    if (composerSupport) {
+        connect(composer, SIGNAL(activated(int)), SLOT(checkChanged()));
+        connect(composer, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
+    }
+    if (commentSupport) {
+        connect(comment, SIGNAL(textChanged(const QString &)), SLOT(checkChanged()));
+    }
     connect(album, SIGNAL(activated(int)), SLOT(checkChanged()));
     connect(album, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
     connect(genre, SIGNAL(activated(int)), SLOT(checkChanged()));
@@ -242,11 +315,13 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     connect(year, SIGNAL(valueChanged(int)), SLOT(checkChanged()));
     connect(trackName, SIGNAL(activated(int)), SLOT(setIndex(int)));
     connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(update()));
-
-    if (original.count()>0) {
-        saveable=QDir(baseDir).isReadable();
-    } else {
-        saveable=false;
+    adjustSize();
+    int w=Utils::scaleForDpi(600);
+    if (width()<w) {
+        resize(w, height());
+    }
+    if (commentSupport) {
+        QTimer::singleShot(0, this, SLOT(readComments()));
     }
 }
 
@@ -266,33 +341,51 @@ void TagEditor::fillSong(Song &s, bool isAll, bool skipEmpty) const
     setString(s.artist, artist->text().trimmed(), skipEmpty && (!haveAll || all.artist.isEmpty()));
     setString(s.album, album->text().trimmed(), skipEmpty && (!haveAll || all.album.isEmpty()));
     setString(s.albumartist, albumArtist->text().trimmed(), skipEmpty && (!haveAll || all.albumartist.isEmpty()));
+    if (composerSupport) {
+        QString str;
+        if (setString(str, composer->text().trimmed(), skipEmpty && (!haveAll || all.composer().isEmpty()))) {
+            s.setComposer(str);
+        }
+    }
+    if (commentSupport) {
+        QString str;
+        if (setString(str, comment->text().trimmed(), skipEmpty && (!haveAll || all.comment().isEmpty()))) {
+            s.setComment(str);
+        }
+    }
     if (!isAll) {
         s.track=track->value();
     }
-    s.disc=disc->value();
-    setString(s.genre, genre->text().trimmed(), skipEmpty && (!haveAll || all.genre.isEmpty()));
-    s.year=year->value();
+    if (!isAll || 0!=disc->value()) {
+        s.disc=disc->value();
+    }
+    setString(s.genre, trim(genre->text()), skipEmpty && (!haveAll || all.genre.isEmpty()));
+    if (!isAll || 0!=year->value()) {
+        s.year=year->value();
+    }
 }
 
-void TagEditor::setPlaceholderTexts()
+void TagEditor::setVariousHint()
 {
-    QString various=i18n("(Various)");
-
     if(0==currentSongIndex && original.count()>1) {
         Song all=original.at(0);
-        artist->setPlaceholderText(all.artist.isEmpty() && haveArtists ? various : QString());
-        album->setPlaceholderText(all.album.isEmpty() && haveAlbums ? various : QString());
-        albumArtist->setPlaceholderText(all.albumartist.isEmpty() && haveAlbumArtists ? various : QString());
-        genre->setPlaceholderText(all.genre.isEmpty() && haveGenres ? various : QString());
+        artist->setPlaceholderText(all.artist.isEmpty() && haveArtists ? TagSpinBox::variousStr() : QString());
+        album->setPlaceholderText(all.album.isEmpty() && haveAlbums ? TagSpinBox::variousStr() : QString());
+        albumArtist->setPlaceholderText(all.albumartist.isEmpty() && haveAlbumArtists ? TagSpinBox::variousStr() : QString());
+        if (composerSupport) {
+            composer->setPlaceholderText(all.composer().isEmpty() && haveComposers ? TagSpinBox::variousStr() : QString());
+        }
+        if (commentSupport) {
+            comment->setPlaceholderText(all.comment().isEmpty() && haveComments ? TagSpinBox::variousStr() : QString());
+        }
+        genre->setPlaceholderText(all.genre.isEmpty() && haveGenres ? TagSpinBox::variousStr() : QString());
+        disc->setVarious(0==all.disc && haveDiscs);
+        year->setVarious(0==all.year && haveYears);
     }
 }
 
 void TagEditor::enableOkButton()
 {
-    if (!saveable) {
-        return;
-    }
-
     enableButton(Ok, (editedIndexes.count()>1) ||
                      (1==original.count() && 1==editedIndexes.count()) ||
                      (1==editedIndexes.count() && !editedIndexes.contains(0)) );
@@ -307,6 +400,12 @@ void TagEditor::setLabelStates()
 
     titleLabel->setOn(!isAll && o.title!=e.title);
     artistLabel->setOn(o.artist!=e.artist);
+    if (composerSupport) {
+        composerLabel->setOn(o.composer()!=e.composer());
+    }
+    if (commentSupport) {
+        commentLabel->setOn(o.comment()!=e.comment());
+    }
     albumArtistLabel->setOn(o.albumartist!=e.albumartist);
     albumLabel->setOn(o.album!=e.album);
     trackLabel->setOn(!isAll && o.track!=e.track);
@@ -315,6 +414,42 @@ void TagEditor::setLabelStates()
     yearLabel->setOn(o.year!=e.year);
 }
 
+void TagEditor::readComments()
+{
+    progress->setVisible(true);
+    progress->setRange(0, original.count());
+    bool haveMultiple=original.count()>1;
+    bool updated=false;
+
+    for (int i=0; i<original.count(); ++i) {
+        progress->setValue(i+1);
+        if (i && 0==i%10) {
+            QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        }
+
+        if (0==i && haveMultiple) {
+            continue;
+        }
+        Song song=original.at(i);
+        QString comment=Tags::readComment(baseDir+song.file);
+        if (!comment.isEmpty()) {
+            song.setComment(comment);
+            original.replace(i, song);
+            haveComments=true;
+            updated=true;
+        }
+    }
+    if (updated) {
+        edited=original;
+    }
+    progress->setVisible(false);
+    if (haveMultiple) {
+        setVariousHint();
+    } else {
+        setSong(original.at(0));
+    }
+}
+
 void TagEditor::applyVa()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
@@ -539,13 +674,13 @@ void TagEditor::updateTrackName(int index, bool edited)
         if (isAll) {
             trackName->setItemText(index, i18n("All tracks [modified]"));
         } else {
-            trackName->setItemText(index, i18n("%1 [modified]", original.at(index).file));
+            trackName->setItemText(index, i18n("%1 [modified]", original.at(index).filePath()));
         }
     } else {
         if (isAll) {
             trackName->setItemText(index, i18n("All tracks"));
         } else {
-            trackName->setItemText(index, original.at(index).file);
+            trackName->setItemText(index, original.at(index).filePath());
         }
     }
 }
@@ -554,7 +689,7 @@ void TagEditor::updateEditedStatus(int index)
 {
     bool isAll=0==index && original.count()>1;
     Song s=edited.at(index);
-    if (equalTags(s, original.at(index), isAll)) {
+    if (equalTags(s, original.at(index), isAll, composerSupport, commentSupport)) {
         if (editedIndexes.contains(index)) {
             editedIndexes.remove(index);
             updateTrackName(index, false);
@@ -584,6 +719,12 @@ void TagEditor::updateEdited(bool isFromAll)
         if (all.albumartist.isEmpty() && s.albumartist.isEmpty() && !o.albumartist.isEmpty()) {
             s.albumartist=o.albumartist;
         }
+        if (composerSupport && all.composer().isEmpty() && s.composer().isEmpty() && !o.composer().isEmpty()) {
+            s.setComposer(o.composer());
+        }
+        if (commentSupport && all.comment().isEmpty() && s.comment().isEmpty() && !o.comment().isEmpty()) {
+            s.setComment(o.comment());
+        }
         if (all.album.isEmpty() && s.album.isEmpty() && !o.album.isEmpty()) {
             s.album=o.album;
         }
@@ -592,7 +733,7 @@ void TagEditor::updateEdited(bool isFromAll)
         }
     }
 
-    if (equalTags(s, original.at(currentSongIndex), isFromAll || isAll)) {
+    if (equalTags(s, original.at(currentSongIndex), isFromAll || isAll, composerSupport, commentSupport)) {
         if (editedIndexes.contains(currentSongIndex)) {
             editedIndexes.remove(currentSongIndex);
             updateTrackName(currentSongIndex, false);
@@ -612,6 +753,12 @@ void TagEditor::setSong(const Song &s)
     title->setText(s.title);
     artist->setText(s.artist);
     albumArtist->setText(s.albumartist);
+    if (composerSupport) {
+        composer->setText(s.composer());
+    }
+    if (commentSupport) {
+        comment->setText(s.comment());
+    }
     album->setText(s.album);
     track->setValue(s.track);
     disc->setValue(s.disc);
@@ -649,14 +796,23 @@ void TagEditor::setIndex(int idx)
         enableButton(User1, !isMultiple && idx<(original.count()-1)); // Next
         enableButton(User2, !isMultiple && idx>1); // Prev
     }
-    setPlaceholderTexts();
+    setVariousHint();
     enableOkButton();
     trackName->setCurrentIndex(idx);
     setLabelStates();
     updating=false;
 }
 
-void TagEditor::applyUpdates()
+void TagEditor::showMopidyMessage()
+{
+    MessageBox::information(this, i18n("Cantata has detected that you are connected to a Mopidy server.\n\n"
+                                       "Currently it is not possible for Cantata to force Mopidy to refresh its local "
+                                       "music listing. Therefore, you will need to stop Cantata, manually refresh "
+                                       "Mopidy's database, and restart Cantata for any changes to be active."),
+                            QLatin1String("Mopidy"));
+}
+
+bool TagEditor::applyUpdates()
 {
     bool skipFirst=original.count()>1;
     QStringList failed;
@@ -667,7 +823,7 @@ void TagEditor::applyUpdates()
     if (!deviceUdi.isEmpty()) {
         dev=getDevice(deviceUdi, this);
         if (!dev) {
-            return;
+            return true;
         }
         opts=dev->options();
         udi=dev->id();
@@ -690,6 +846,7 @@ void TagEditor::applyUpdates()
     progress->setRange(1, toSave);
 
     int count=0;
+    bool someTimedout=false;
     foreach (int idx, editedIndexes) {
         progress->setValue(progress->value()+1);
 
@@ -702,12 +859,14 @@ void TagEditor::applyUpdates()
         }
         Song orig=original.at(idx);
         Song edit=edited.at(idx);
-        if (equalTags(orig, edit, false)) {
+        if (equalTags(orig, edit, false, composerSupport, commentSupport)) {
             continue;
         }
 
-        switch(Tags::update(baseDir+orig.file, orig, edit)) {
+        QString file=orig.filePath();
+        switch(Tags::update(baseDir+file, orig, edit, -1, commentSupport)) {
         case Tags::Update_Modified:
+            edit.setComment(QString());
             #ifdef ENABLE_DEVICES_SUPPORT
             if (!deviceUdi.isEmpty()) {
                 if (!dev->updateSong(orig, edit)) {
@@ -723,12 +882,15 @@ void TagEditor::applyUpdates()
                 }
             }
             updatedSongs.append(edit);
-            if (!renameFiles && orig.file!=opts.createFilename(edit)) {
+            if (!renameFiles && file!=opts.createFilename(edit)) {
                 renameFiles=true;
             }
             break;
         case Tags::Update_Failed:
-            failed.append(orig.file);
+            failed.append(file);
+            break;
+        case Tags::Update_BadFile:
+            failed.append(i18nc("filename (Corrupt tags?)", "%1 (Corrupt tags?)", file));
             break;
         default:
             break;
@@ -741,31 +903,35 @@ void TagEditor::applyUpdates()
     }
 
     if (updatedSongs.count()) {
-        #ifdef ENABLE_DEVICES_SUPPORT
-        if (!deviceUdi.isEmpty()) {
-            dev->saveCache();
-        } else
-        #endif
-        {
-//             MusicLibraryModel::self()->removeCache();
-            emit update();
-        }
-
+        // If we call tag-editor, no need to do MPD update - as this will be done from that dialog...
         if (renameFiles &&
             MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Would you also like to rename your song files, so as to match your tags?"),
                                                        i18n("Rename Files"), GuiItem(i18n("Rename")), StdGuiItem::cancel())) {
             TrackOrganiser *dlg=new TrackOrganiser(parentWidget());
-            dlg->show(updatedSongs, udi);
+            dlg->show(updatedSongs, udi, true);
+        } else {
+            #ifdef ENABLE_DEVICES_SUPPORT
+            if (!deviceUdi.isEmpty()) {
+                dev->saveCache();
+            } else
+            #endif
+            {
+            //             MusicLibraryModel::self()->removeCache();
+                emit update();
+            }
         }
     }
+
+    return !someTimedout;
 }
 
 void TagEditor::slotButtonClicked(int button)
 {
     switch (button) {
     case Ok: {
-        applyUpdates();
-        accept();
+        if (applyUpdates()) {
+            accept();
+        }
         break;
     }
     case Reset: // Reset
