@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,7 +24,7 @@
 #include "tags.h"
 #include "config.h"
 #include "filetyperesolver.h"
-#include "utils.h"
+#include "support/utils.h"
 #include <algorithm>
 #include <cctype>
 #include <cmath>
@@ -33,11 +33,11 @@
 #include <ios>
 #include <QPair>
 #include <QFile>
-#include <QMutex>
-#include <QMutexLocker>
 #include <QString>
 #include <QStringList>
 #include <QTextCodec>
+#include <QDebug>
+#include <taglib/tpropertymap.h>
 #include <taglib/fileref.h>
 #include <taglib/aifffile.h>
 #ifdef TAGLIB_ASF_FOUND
@@ -47,6 +47,9 @@
 #ifdef TAGLIB_MP4_FOUND
 #include <taglib/mp4file.h>
 #endif
+#ifdef TAGLIB_OPUS_FOUND
+#include <taglib/opusfile.h>
+#endif
 #include <taglib/mpcfile.h>
 #include <taglib/mpegfile.h>
 #include <taglib/oggfile.h>
@@ -62,7 +65,7 @@
 #endif
 #include <taglib/apetag.h>
 #include <taglib/id3v2tag.h>
-#include <taglib/id3v1tag.h>
+#include <taglib/id3v1genres.h>
 #ifdef TAGLIB_MP4_FOUND
 #include <taglib/mp4tag.h>
 #endif
@@ -76,9 +79,27 @@
 #include <taglib-extras/realmediafiletyperesolver.h>
 #endif
 
+#ifdef ENABLE_EXTERNAL_TAGS
+// Mutex is locked in client, and server (that accesses this class) is single threaded
+#define LOCK_MUTEX
+#else
+#include <QMutex>
+#include <QMutexLocker>
+static QMutex mutex;
+#define LOCK_MUTEX QMutexLocker locker(&mutex);
+#endif
+
 namespace Tags
 {
 
+static bool debugEnabled=false;
+#define DBUG if (debugEnabled) qWarning() << "Tags" << __FUNCTION__
+
+void enableDebug()
+{
+    debugEnabled=true;
+}
+
 static QString tString2QString(const TagLib::String &str)
 {
     static QTextCodec *codec = QTextCodec::codecForName("UTF-8");
@@ -109,11 +130,6 @@ static double parseRgString(const TagLib::String &str) {
     return ok ? v : 0.0;
 }
 
-bool ReplayGain::isEmpty() const
-{
-    return Utils::equal(trackGain, 0.0) && Utils::equal(trackPeak, 0.0) && Utils::equal(albumGain, 0.0) && Utils::equal(albumPeak, 0.0);
-}
-
 struct RgTags : public ReplayGain
 {
     RgTags(const ReplayGain &r)
@@ -144,19 +160,6 @@ struct RgTagsStrings
     std::string trackGain, trackPeak, albumGain, albumPeak;
 };
 
-static TagLib::FileRef getFileRef(const QString &path)
-{
-    #ifdef Q_OS_WIN32
-    const wchar_t *encodedName = reinterpret_cast< const wchar_t * >(path.utf16());
-    #elif defined COMPLEX_TAGLIB_FILENAME
-    const wchar_t *encodedName = reinterpret_cast< const wchar_t * >(path.utf16());
-    #else
-    QByteArray fileName = QFile::encodeName(path);
-    const char *encodedName = fileName.constData(); // valid as long as fileName exists
-    #endif
-    return TagLib::FileRef(encodedName, true, TagLib::AudioProperties::Fast);
-}
-
 static void ensureFileTypeResolvers()
 {
     static bool alreadyAdded = false;
@@ -171,6 +174,17 @@ static void ensureFileTypeResolvers()
     }
 }
 
+static TagLib::FileRef getFileRef(const QString &path)
+{
+    ensureFileTypeResolvers();
+
+    #ifdef Q_OS_WIN
+    return TagLib::FileRef(reinterpret_cast<const wchar_t*>(path.constData()), true, TagLib::AudioProperties::Fast);
+    #else
+    return TagLib::FileRef(QFile::encodeName(path).constData(), true, TagLib::AudioProperties::Fast);
+    #endif
+}
+
 static QPair<int, int> splitDiscNumber(const QString &value)
 {
     int disc;
@@ -191,7 +205,8 @@ static QPair<int, int> splitDiscNumber(const QString &value)
 }
 
 // -- taken from rgtag.cpp from libebur128 -- START
-static bool clearTxxxTag(TagLib::ID3v2::Tag* tag, TagLib::String tagName) {
+static bool clearTxxxTag(TagLib::ID3v2::Tag *tag, TagLib::String tagName)
+{
     TagLib::ID3v2::FrameList l = tag->frameList("TXXX");
     for (TagLib::ID3v2::FrameList::Iterator it = l.begin(); it != l.end(); ++it) {
         TagLib::ID3v2::UserTextIdentificationFrame *fr=dynamic_cast<TagLib::ID3v2::UserTextIdentificationFrame*>(*it);
@@ -203,7 +218,8 @@ static bool clearTxxxTag(TagLib::ID3v2::Tag* tag, TagLib::String tagName) {
     return false;
 }
 
-static bool clearRva2Tag(TagLib::ID3v2::Tag* tag, TagLib::String tagName) {
+static bool clearRva2Tag(TagLib::ID3v2::Tag *tag, TagLib::String tagName)
+{
     TagLib::ID3v2::FrameList l = tag->frameList("RVA2");
     for (TagLib::ID3v2::FrameList::Iterator it = l.begin(); it != l.end(); ++it) {
         TagLib::ID3v2::RelativeVolumeFrame *fr=dynamic_cast<TagLib::ID3v2::RelativeVolumeFrame*>(*it);
@@ -212,10 +228,11 @@ static bool clearRva2Tag(TagLib::ID3v2::Tag* tag, TagLib::String tagName) {
             return true;
         }
     }
-  return false;
+    return false;
 }
 
-static void setTxxxTag(TagLib::ID3v2::Tag* tag, const std::string &tagName, const std::string &value) {
+static void setTxxxTag(TagLib::ID3v2::Tag *tag, const std::string &tagName, const std::string &value)
+{
     TagLib::ID3v2::UserTextIdentificationFrame *frame = TagLib::ID3v2::UserTextIdentificationFrame::find(tag, tagName);
     if (!frame) {
         frame = new TagLib::ID3v2::UserTextIdentificationFrame;
@@ -225,13 +242,14 @@ static void setTxxxTag(TagLib::ID3v2::Tag* tag, const std::string &tagName, cons
     frame->setText(value);
 }
 
-static void setRva2Tag(TagLib::ID3v2::Tag* tag, const std::string &tagName, double gain, double peak) {
+static void setRva2Tag(TagLib::ID3v2::Tag *tag, const std::string &tagName, double gain, double peak)
+{
     TagLib::ID3v2::RelativeVolumeFrame *frame = NULL;
     TagLib::ID3v2::FrameList frameList = tag->frameList("RVA2");
     TagLib::ID3v2::FrameList::ConstIterator it = frameList.begin();
     for (; it != frameList.end(); ++it) {
         TagLib::ID3v2::RelativeVolumeFrame *fr=dynamic_cast<TagLib::ID3v2::RelativeVolumeFrame*>(*it);
-        if (fr->identification() == tagName) {
+        if (fr && fr->identification() == tagName) {
             frame = fr;
             break;
         }
@@ -262,11 +280,53 @@ static void readID3v2Tags(TagLib::ID3v2::Tag *tag, Song *song, ReplayGain *rg, Q
             song->albumartist=tString2QString(albumArtist.front()->toString());
         }
 
+        const TagLib::ID3v2::FrameList &composer = tag->frameListMap()["TCOM"];
+
+        if (!composer.isEmpty()) {
+            song->setComposer(tString2QString(composer.front()->toString()));
+        }
+
         const TagLib::ID3v2::FrameList &disc = tag->frameListMap()["TPOS"];
 
         if (!disc.isEmpty()) {
             song->disc=splitDiscNumber(tString2QString(disc.front()->toString())).first;
         }
+
+        const TagLib::ID3v2::FrameList &tcon = tag->frameListMap()["TCON"];
+        if (!tcon.isEmpty()) {
+            TagLib::ID3v2::FrameList::ConstIterator it = tcon.begin();
+            TagLib::ID3v2::FrameList::ConstIterator end = tcon.end();
+            QSet<QString> genres;
+
+            for (; it!=end; ++it) {
+                TagLib::ID3v2::TextIdentificationFrame *f = static_cast<TagLib::ID3v2::TextIdentificationFrame *>(*it);
+                TagLib::StringList fields = f->fieldList();
+                TagLib::StringList::ConstIterator fIt = fields.begin();
+                TagLib::StringList::ConstIterator fEnd = fields.end();
+
+                for (; fIt!=fEnd; ++fIt) {
+                    if ((*fIt).isEmpty()) {
+                        continue;
+                    }
+
+                    bool ok;
+                    int number = (*fIt).toInt(&ok);
+                    QString genre;
+                    if (ok && number >= 0 && number <= 255) {
+                        genre = tString2QString(TagLib::ID3v1::genre(number));
+                    } else {
+                        genre = tString2QString(*fIt);
+                    }
+                    if (!genres.contains(genre)) {
+                        genres.insert(genre);
+                        if (!song->genre.isEmpty()) {
+                            song->genre+=QLatin1Char(';');
+                        }
+                        song->genre+=genre;
+                    }
+                }
+            }
+        }
     }
 
     if (rg) {
@@ -375,9 +435,30 @@ static bool writeID3v2Tags(TagLib::ID3v2::Tag *tag, const Song &from, const Song
         if (from.albumartist!=to.albumartist && updateID3v2Tag(tag, "TPE2", to.albumartist)) {
             changed=true;
         }
+        if (from.composer()!=to.composer() && updateID3v2Tag(tag, "TCOM", to.composer())) {
+            changed=true;
+        }
         if (from.disc!=to.disc&& updateID3v2Tag(tag, "TPOS", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        DBUG << "genres" << from.genre << to.genre;
+        if (from.genre!=to.genre) {
+            tag->removeFrames("TCON");
+            QStringList genres=to.genres();
+            DBUG << "num genres:" << genres.count();
+            if (genres.count()<2) {
+                DBUG << "set genre" << (genres.isEmpty() ? QString() : genres.first().trimmed());
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    TagLib::ID3v2::TextIdentificationFrame *frame = new TagLib::ID3v2::TextIdentificationFrame("TCON");
+                    tag->addFrame(frame);
+                    DBUG << "add genre" << genre.trimmed();
+                    frame->setText(qString2TString(genre.trimmed()));
+                }
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -416,7 +497,7 @@ static bool writeID3v2Tags(TagLib::ID3v2::Tag *tag, const Song &from, const Song
     return changed;
 }
 
-static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg)
+static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg, QImage *img)
 {
     const TagLib::APE::ItemListMap &map = tag->itemListMap();
 
@@ -424,10 +505,20 @@ static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg)
         if (map.contains("Album Artist")) {
             song->albumartist=tString2QString(map["Album Artist"].toString());
         }
-
+        if (map.contains("Composer")) {
+            song->setComposer(tString2QString(map["Composer"].toString()));
+        }
         if (map.contains("Disc")) {
             song->disc=splitDiscNumber(tString2QString(map["Disc"].toString())).first;
         }
+        if (map.contains("GENRE")) {
+            TagLib::StringList genres=map["GENRE"].values();
+            TagLib::StringList::ConstIterator it=genres.begin();
+            TagLib::StringList::ConstIterator end=genres.end();
+            for (; it!=end; ++it) {
+                song->addGenre(tString2QString(*it));
+            }
+        }
     }
 
     if (rg) {
@@ -444,6 +535,21 @@ static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg)
             rg->albumPeak=parseRgString(map["replaygain_album_peak"].toString());
         }
     }
+
+    if (img) {
+        if (map.contains("COVER ART (FRONT)")) {
+            const TagLib::ByteVector nullStringTerminator(1, 0);
+
+            TagLib::ByteVector item = map["COVER ART (FRONT)"].value();
+            int pos = item.find(nullStringTerminator);   // Skip the filename
+
+            if (++pos > 0) {
+                const TagLib::ByteVector &bytes=item.mid(pos);
+                QByteArray data(bytes.data(), bytes.size());
+                img->loadFromData(data);
+            }
+        }
+    }
 }
 
 static bool updateAPETag(TagLib::APE::Tag *tag, const char *tagName, const QString &value)
@@ -470,9 +576,24 @@ static bool writeAPETags(TagLib::APE::Tag *tag, const Song &from, const Song &to
         if (from.albumartist!=to.albumartist && updateAPETag(tag, "Album Artist", to.albumartist)) {
             changed=true;
         }
+        if (from.composer()!=to.composer() && updateAPETag(tag, "Composer", to.composer())) {
+            changed=true;
+        }
         if (from.disc!=to.disc && updateAPETag(tag, "Disc", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            tag->removeItem("GENRE");
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    tag->addValue("GENRE", qString2TString(genre.trimmed()), false);
+                }
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -495,7 +616,6 @@ static TagLib::String readVorbisTag(TagLib::Ogg::XiphComment *tag, const char *f
 {
     if (tag->contains(field)) {
         const TagLib::StringList &list = tag->fieldListMap()[field];
-
         if (!list.isEmpty()) {
             return list.front();
         }
@@ -504,18 +624,31 @@ static TagLib::String readVorbisTag(TagLib::Ogg::XiphComment *tag, const char *f
     return TagLib::String();
 }
 
-static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, ReplayGain *rg, QImage *img)
+static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, ReplayGain *rg, QImage *img, QString *lyrics)
 {
     if (song) {
         TagLib::String str=readVorbisTag(tag, "ALBUMARTIST");
         if (str.isEmpty()) {
             song->albumartist=tString2QString(str);
         }
-
+        str=readVorbisTag(tag, "COMPOSER");
+        if (str.isEmpty()) {
+            song->setComposer(tString2QString(str));
+        }
         str=readVorbisTag(tag, "DISCNUMBER");
         if (str.isEmpty()) {
             song->disc=splitDiscNumber(tString2QString(str)).first;
         }
+        if (tag->contains("GENRE")) {
+            const TagLib::StringList &genres = tag->fieldListMap()["GENRE"];
+            if (!genres.isEmpty()) {
+                TagLib::StringList::ConstIterator it=genres.begin();
+                TagLib::StringList::ConstIterator end=genres.end();
+                for (; it!=end; ++it) {
+                    song->addGenre(tString2QString(*it));
+                }
+            }
+        }
     }
 
     if (rg) {
@@ -527,9 +660,30 @@ static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, Rep
 
     if (img) {
         TagLib::Ogg::FieldListMap map = tag->fieldListMap();
-        // Ogg lacks a definitive standard for embedding cover art, but it seems
-        // b64 encoding a field called COVERART is the general convention
-        if (map.contains("COVERART")) {
+        #if (TAGLIB_MAJOR_VERSION > 1) || (TAGLIB_MAJOR_VERSION == 1 && TAGLIB_MINOR_VERSION >= 7)
+        // METADATA_BLOCK_PICTURE is the Ogg standard way of encoding a covers.
+        // https://wiki.xiph.org/index.php/VorbisComment#Cover_art
+        if (map.contains("METADATA_BLOCK_PICTURE")) {
+            TagLib::StringList block=map["METADATA_BLOCK_PICTURE"];
+            if (!block.isEmpty()) {
+                TagLib::StringList::ConstIterator i = block.begin();
+                TagLib::StringList::ConstIterator end = block.end();
+
+                for (; i != end; ++i ) {
+                    QByteArray data(QByteArray::fromBase64(i->to8Bit().c_str()));
+                    TagLib::ByteVector bytes(data.data(), data.size());
+                    TagLib::FLAC::Picture pic;
+
+                    if (pic.parse(bytes)) {
+                        img->loadFromData(QByteArray(pic.data().data(), pic.data().size()));
+                    }
+                }
+            }
+        }
+        #endif
+
+        // COVERART is an older (now deprecated) way of storing covers...
+        if (img->isNull() && map.contains("COVERART")) {
             QByteArray data=map["COVERART"].toString().toCString();
             img->loadFromData(QByteArray::fromBase64(data));
             if (img->isNull()) {
@@ -537,6 +691,13 @@ static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, Rep
             }
         }
     }
+
+    if (lyrics) {
+        TagLib::String str=readVorbisTag(tag, "LYRICS");
+        if (!str.isEmpty()) {
+            *lyrics=tString2QString(str);
+        }
+    }
 }
 
 #if (TAGLIB_MAJOR_VERSION > 1) || (TAGLIB_MAJOR_VERSION == 1 && TAGLIB_MINOR_VERSION >= 7)
@@ -573,9 +734,24 @@ static bool writeVorbisCommentTags(TagLib::Ogg::XiphComment *tag, const Song &fr
         if (from.albumartist!=to.albumartist && updateVorbisCommentTag(tag, "ALBUMARTIST", to.albumartist)) {
             changed=true;
         }
+        if (from.composer()!=to.composer() && updateVorbisCommentTag(tag, "COMPOSER", to.composer())) {
+            changed=true;
+        }
         if (from.disc!=to.disc && updateVorbisCommentTag(tag, "DISCNUMBER", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            tag->removeField("GENRE");
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    tag->addField("GENRE", qString2TString(genre.trimmed()), false);
+                }
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -612,9 +788,20 @@ static void readMP4Tags(TagLib::MP4::Tag *tag, Song *song, ReplayGain *rg, QImag
         if (map.contains("aART") && !map["aART"].toStringList().isEmpty()) {
             song->albumartist=tString2QString(map["aART"].toStringList().front());
         }
+        if (map.contains("\xA9wrt") && !map["\xA9wrt"].toStringList().isEmpty()) {
+            song->setComposer(tString2QString(map["\xA9wrt"].toStringList().front()));
+        }
         if (map.contains("disk")) {
             song->disc=map["disk"].toIntPair().first;
         }
+        if (map.contains("\251gen")) {
+            TagLib::StringList genres=map["\251gen"].toStringList();
+            TagLib::StringList::ConstIterator it=genres.begin();
+            TagLib::StringList::ConstIterator end=genres.end();
+            for (; it!=end; ++it) {
+                song->addGenre(tString2QString(*it));
+            }
+        }
     }
     if (rg) {
         if (map.contains("----:com.apple.iTunes:replaygain_track_gain")) {
@@ -652,7 +839,7 @@ static bool updateMP4Tag(TagLib::MP4::Tag *tag, const char *tagName, const QStri
             return true;
         }
     } else {
-        map.insert("aART", TagLib::StringList(qString2TString(value)));
+        map.insert(tagName, TagLib::StringList(qString2TString(value)));
         return true;
     }
     return false;
@@ -666,9 +853,25 @@ static bool writeMP4Tags(TagLib::MP4::Tag *tag, const Song &from, const Song &to
         if (from.albumartist!=to.albumartist && updateMP4Tag(tag, "aART", to.albumartist)) {
             changed=true;
         }
+        if (from.composer()!=to.composer() && updateMP4Tag(tag, "\xA9wrt", to.composer())) {
+            changed=true;
+        }
         if (from.disc!=to.disc && updateMP4Tag(tag, "disk", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                TagLib::StringList tagGenres;
+                foreach (const QString &genre, genres) {
+                    tagGenres.append(qString2TString(genre.trimmed()));
+                }
+                tag->itemListMap()["\251gen"]=tagGenres;
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -711,10 +914,20 @@ static void readASFTags(TagLib::ASF::Tag *tag, Song *song, ReplayGain *rg)
         if (map.contains("WM/AlbumTitle") && !map["WM/AlbumTitle"].isEmpty()) {
             song->albumartist=tString2QString(map["WM/AlbumTitle"].front().toString());
         }
-
-        if (map.contains("WM/PartOfSet")) {
+        if (map.contains("WM/Composer") && !map["WM/Composer"].isEmpty()) {
+            song->setComposer(tString2QString(map["WM/Composer"].front().toString()));
+        }
+        if (map.contains("WM/PartOfSet") && !map["WM/PartOfSet"].isEmpty()) {
             song->albumartist=map["WM/PartOfSet"].front().toUInt();
         }
+        if (map.contains("WM/Genre") && !map["WM/Genre"].isEmpty()) {
+            const TagLib::ASF::AttributeList &genres=map["WM/Genre"];
+            TagLib::ASF::AttributeList::ConstIterator it=genres.begin();
+            TagLib::ASF::AttributeList::ConstIterator end=genres.end();
+            for (; it!=end; ++it) {
+                song->addGenre(tString2QString((*it).toString()));
+            }
+        }
     }
 }
 
@@ -743,9 +956,24 @@ static bool writeASFTags(TagLib::ASF::Tag *tag, const Song &from, const Song &to
         if (from.albumartist!=to.albumartist && updateASFTag(tag, "WM/AlbumTitle", to.albumartist)) {
             changed=true;
         }
+        if (from.composer()!=to.composer() && updateASFTag(tag, "WM/Composer", to.composer())) {
+            changed=true;
+        }
         if (from.disc!=to.disc && updateASFTag(tag, "WM/PartOfSet", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            tag->removeItem("WM/Genre");
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    tag->addAttribute("WM/Genre", qString2TString(genre.trimmed()));
+                }
+            }
+            changed=true;
+        }
     }
     return changed;
 }
@@ -758,40 +986,46 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
         song->title=tString2QString(tag->title());
         song->artist=tString2QString(tag->artist());
         song->album=tString2QString(tag->album());
-        song->genre=tString2QString(tag->genre());
+//        song->genre=tString2QString(tag->genre());
         song->track=tag->track();
         song->year=tag->year();
     }
 
     if (TagLib::MPEG::File *file = dynamic_cast< TagLib::MPEG::File * >(fileref.file())) {
-        if (file->ID3v2Tag()) {
+        if (file->ID3v2Tag() && !file->ID3v2Tag()->isEmpty()) {
             readID3v2Tags(file->ID3v2Tag(), song, rg, img, lyrics);
         } else if (file->APETag()) {
-            readAPETags(file->APETag(), song, rg);
+            readAPETags(file->APETag(), song, rg, img);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
         }
     } else if (TagLib::Ogg::Vorbis::File *file = dynamic_cast< TagLib::Ogg::Vorbis::File * >(fileref.file()))  {
         if (file->tag()) {
-            readVorbisCommentTags(file->tag(), song, rg, img);
+            readVorbisCommentTags(file->tag(), song, rg, img, lyrics);
         }
     } else if (TagLib::Ogg::FLAC::File *file = dynamic_cast< TagLib::Ogg::FLAC::File * >(fileref.file())) {
         if (file->tag()) {
-            readVorbisCommentTags(file->tag(), song, rg, img);
+            readVorbisCommentTags(file->tag(), song, rg, img, lyrics);
         }
     } else if (TagLib::Ogg::Speex::File *file = dynamic_cast< TagLib::Ogg::Speex::File * >(fileref.file())) {
         if (file->tag()) {
-            readVorbisCommentTags(file->tag(), song, rg, img);
+            readVorbisCommentTags(file->tag(), song, rg, img, lyrics);
         }
+    #ifdef TAGLIB_OPUS_FOUND
+    } else if (TagLib::Ogg::Opus::File *file = dynamic_cast< TagLib::Ogg::Opus::File * >(fileref.file())) {
+        if (file->tag()) {
+            readVorbisCommentTags(file->tag(), song, rg, img, lyrics);
+        }
+    #endif
     } else if (TagLib::FLAC::File *file = dynamic_cast< TagLib::FLAC::File * >(fileref.file())) {
         if (file->xiphComment()) {
-            readVorbisCommentTags(file->xiphComment(), song, rg, img);
+            readVorbisCommentTags(file->xiphComment(), song, rg, img, lyrics);
             #if (TAGLIB_MAJOR_VERSION > 1) || (TAGLIB_MAJOR_VERSION == 1 && TAGLIB_MINOR_VERSION >= 7)
             if (img && img->isNull()) {
                 readFlacPicture(file->pictureList(), img);
             }
             #endif
-        } else if (file->ID3v2Tag()) {
+        } else if (file->ID3v2Tag() && !file->ID3v2Tag()->isEmpty()) {
             readID3v2Tags(file->ID3v2Tag(), song, rg, img, lyrics);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
@@ -805,7 +1039,7 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
     #endif
     } else if (TagLib::MPC::File *file = dynamic_cast< TagLib::MPC::File * >(fileref.file())) {
         if (file->APETag()) {
-            readAPETags(file->APETag(), song, rg);
+            readAPETags(file->APETag(), song, rg, img);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
         }
@@ -832,14 +1066,17 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
         }
     } else if (TagLib::WavPack::File *file = dynamic_cast< TagLib::WavPack::File * >(fileref.file())) {
         if (file->APETag()) {
-            readAPETags(file->APETag(), song, rg);
+            readAPETags(file->APETag(), song, rg, img);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
         }
     }
+    if (song && song->genre.isEmpty()) {
+        song->genre=tString2QString(tag->genre());
+    }
 }
 
-static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img)
+static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img, bool saveComment)
 {
     bool changed=false;
     TagLib::Tag *tag=fileref.tag();
@@ -856,10 +1093,10 @@ static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Son
             tag->setAlbum(qString2TString(to.album));
             changed=true;
         }
-        if (from.genre!=to.genre) {
-            tag->setGenre(qString2TString(to.genre));
-            changed=true;
-        }
+//        if (from.genre!=to.genre) {
+//            tag->setGenre(qString2TString(to.genre));
+//            changed=true;
+//        }
         if (from.track!=to.track) {
             tag->setTrack(to.track);
             changed=true;
@@ -868,9 +1105,13 @@ static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Son
             tag->setYear(to.year);
             changed=true;
         }
+        if (saveComment && from.comment()!=to.comment()) {
+            tag->setComment(qString2TString(to.comment()));
+            changed=true;
+        }
     }
     if (TagLib::MPEG::File *file = dynamic_cast< TagLib::MPEG::File * >(fileref.file())) {
-        if (file->ID3v2Tag()) {
+        if (file->ID3v2Tag() && !file->ID3v2Tag()->isEmpty()) {
             changed=writeID3v2Tags(file->ID3v2Tag(), from, to, rg, img) || changed;
         } else if (file->APETag()) {
             changed=writeAPETags(file->APETag(), from, to, rg) || changed;
@@ -891,10 +1132,16 @@ static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Son
         if (file->tag()) {
             changed=writeVorbisCommentTags(file->tag(), from, to, rg, img) || changed;
         }
+    #ifdef TAGLIB_OPUS_FOUND
+    } else if (TagLib::Ogg::Opus::File *file = dynamic_cast< TagLib::Ogg::Opus::File * >(fileref.file())) {
+        if (file->tag()) {
+            changed=writeVorbisCommentTags(file->tag(), from, to, rg, img) || changed;
+        }
+    #endif
     } else if (TagLib::FLAC::File *file = dynamic_cast< TagLib::FLAC::File * >(fileref.file())) {
         if (file->xiphComment()) {
             changed=writeVorbisCommentTags(file->xiphComment(), from, to, rg, img) || changed;
-        } else if (file->ID3v2Tag()) {
+        } else if (file->ID3v2Tag() && !file->ID3v2Tag()->isEmpty()) {
             changed=writeID3v2Tags(file->ID3v2Tag(), from, to, rg, img) || changed;
 //         } else if (file->ID3v1Tag()) {
 //             changed=writeID3v1Tags(fileref, from, to, rg, img) || changed;
@@ -946,16 +1193,11 @@ static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Son
     return changed;
 }
 
-static QMutex mutex;
-
 Song read(const QString &fileName)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     Song song;
     TagLib::FileRef fileref = getFileRef(fileName);
-
     if (fileref.isNull()) {
         return song;
     }
@@ -968,12 +1210,9 @@ Song read(const QString &fileName)
 
 QImage readImage(const QString &fileName)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     QImage img;
     TagLib::FileRef fileref = getFileRef(fileName);
-
     if (fileref.isNull()) {
         return img;
     }
@@ -984,12 +1223,9 @@ QImage readImage(const QString &fileName)
 
 QString readLyrics(const QString &fileName)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     QString lyrics;
     TagLib::FileRef fileref = getFileRef(fileName);
-
     if (fileref.isNull()) {
         return lyrics;
     }
@@ -998,21 +1234,27 @@ QString readLyrics(const QString &fileName)
     return lyrics;
 }
 
-static Update update(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img, int id3Ver=-1)
+QString readComment(const QString &fileName)
 {
-    if (writeTags(fileref, from, to, rg, img)) {
+    LOCK_MUTEX
+    TagLib::FileRef fileref = getFileRef(fileName);
+    return fileref.isNull() ? QString() : tString2QString(fileref.tag()->comment());
+}
+
+static Update update(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img, int id3Ver=-1, bool saveComment=false)
+{
+    if (writeTags(fileref, from, to, rg, img, saveComment)) {
         TagLib::MPEG::File *mpeg=dynamic_cast<TagLib::MPEG::File *>(fileref.file());
         if (mpeg) {
-            TagLib::ID3v1::Tag *v1=mpeg->ID3v1Tag(false);
-            bool haveV1=v1 && (!v1->title().isEmpty() || !v1->artist().isEmpty() || !v1->album().isEmpty());
             #ifdef TAGLIB_CAN_SAVE_ID3VER
             TagLib::ID3v2::Tag *v2=mpeg->ID3v2Tag(false);
             bool isID3v24=v2 && isId3V24(v2->header());
             int ver=id3Ver==3 ? 3 : (id3Ver==4 ? 4 : (isID3v24 ? 4 : 3));
-            return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
+            DBUG << "isID3v24" << isID3v24 << "reqVer:" << id3Ver << "use:" << ver;
+            return mpeg->save(TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
             #else
             Q_UNUSED(id3Ver)
-            return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
+            return mpeg->save(TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
             #endif
         }
         return fileref.file()->save() ? Update_Modified : Update_Failed;
@@ -1022,11 +1264,8 @@ static Update update(const TagLib::FileRef fileref, const Song &from, const Song
 
 Update updateArtistAndTitle(const QString &fileName, const Song &song)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     TagLib::FileRef fileref = getFileRef(fileName);
-
     if (fileref.isNull()) {
         return Update_Failed;
     }
@@ -1037,35 +1276,29 @@ Update updateArtistAndTitle(const QString &fileName, const Song &song)
     tag->setArtist(qString2TString(song.artist));
 
     if (mpeg) {
-        TagLib::ID3v1::Tag *v1=mpeg->ID3v1Tag(false);
-        bool haveV1=v1 && (!v1->title().isEmpty() || !v1->artist().isEmpty() || !v1->album().isEmpty());
         #ifdef TAGLIB_CAN_SAVE_ID3VER
         TagLib::ID3v2::Tag *v2=mpeg->ID3v2Tag(false);
         int ver=v2 && isId3V24(v2->header()) ? 4 : 3;
-        return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
+        DBUG << "useId3ver:" << ver;
+        return mpeg->save(TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
         #else
-        return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
+        return mpeg->save(TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
         #endif
     }
     return fileref.file()->save() ? Update_Modified : Update_Failed;
 }
 
-Update update(const QString &fileName, const Song &from, const Song &to, int id3Ver)
+Update update(const QString &fileName, const Song &from, const Song &to, int id3Ver, bool saveComment)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     TagLib::FileRef fileref = getFileRef(fileName);
-    return fileref.isNull() ? Update_Failed : update(fileref, from, to, RgTags(), QByteArray(), id3Ver);
+    return fileref.isNull() ? Update_Failed : update(fileref, from, to, RgTags(), QByteArray(), id3Ver, saveComment);
 }
 
 ReplayGain readReplaygain(const QString &fileName)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     TagLib::FileRef fileref = getFileRef(fileName);
-
     if (fileref.isNull()) {
         return false;
     }
@@ -1077,20 +1310,79 @@ ReplayGain readReplaygain(const QString &fileName)
 
 Update updateReplaygain(const QString &fileName, const ReplayGain &rg)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     TagLib::FileRef fileref = getFileRef(fileName);
     return fileref.isNull() ? Update_Failed : update(fileref, Song(), Song(), RgTags(rg), QByteArray());
 }
 
 Update embedImage(const QString &fileName, const QByteArray &cover)
 {
-    QMutexLocker locker(&mutex);
-    ensureFileTypeResolvers();
-
+    LOCK_MUTEX
     TagLib::FileRef fileref = getFileRef(fileName);
     return fileref.isNull() ? Update_Failed : update(fileref, Song(), Song(), RgTags(), cover);
 }
 
+QString oggMimeType(const QString &fileName)
+{
+    LOCK_MUTEX
+    #ifdef Q_OS_WIN
+    const wchar_t*encodedName=reinterpret_cast<const wchar_t*>(fileName.constData());
+    #else
+    const char *encodedName=QFile::encodeName(fileName).constData();
+    #endif
+
+    TagLib::Ogg::Vorbis::File vorbis(encodedName, false, TagLib::AudioProperties::Fast);
+    if (vorbis.isValid()) {
+        return QLatin1String("audio/x-vorbis+ogg");
+    }
+
+    TagLib::Ogg::FLAC::File flac(encodedName, false, TagLib::AudioProperties::Fast);
+    if (flac.isValid()) {
+        return QLatin1String("audio/x-flac+ogg");
+    }
+
+    TagLib::Ogg::Speex::File speex(encodedName, false, TagLib::AudioProperties::Fast);
+    if (speex.isValid()) {
+        return QLatin1String("audio/x-speex+ogg");
+    }
+
+    #ifdef TAGLIB_OPUS_FOUND
+    TagLib::Ogg::Opus::File opus(encodedName, false, TagLib::AudioProperties::Fast);
+    if (opus.isValid()) {
+        return QLatin1String("audio/x-opus+ogg");
+    }
+    #endif
+    return QLatin1String("audio/ogg");
+}
+
+QMap<QString, QString> readAll(const QString &fileName)
+{
+    QMap<QString, QString> allTags;
+    TagLib::FileRef fileref = getFileRef(fileName);
+    if (fileref.isNull()) {
+        return allTags;
+    }
+
+    TagLib::PropertyMap properties=fileref.file()->properties();
+    TagLib::PropertyMap::ConstIterator it = properties.begin();
+    TagLib::PropertyMap::ConstIterator end = properties.end();
+    for (; it!=end; ++it) {
+        allTags.insert(tString2QString(it->first.upper()), tString2QString(it->second.toString(", ")));
+    }
+
+    if (fileref.audioProperties()) {
+        TagLib::AudioProperties *properties = fileref.audioProperties();
+        allTags.insert(QLatin1String("X-AUDIO:BITRATE"), QString("%1 kb/s").arg(properties->bitrate()));
+        allTags.insert(QLatin1String("X-AUDIO:SAMPLERATE"), QString("%1 Hz").arg(properties->sampleRate()));
+        allTags.insert(QLatin1String("X-AUDIO:CHANNELS"), QString::number(properties->channels()));
+    }
+    return allTags;
+}
+
+QString id3Genre(int id)
+{
+    // Clementine: In theory, genre 0 is "blues"; in practice it's invalid.
+    return 0==id ? QString() : tString2QString(TagLib::ID3v1::genre(id));
+}
+
 }
