@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  * This file (c) 2012 Piotr Wicijowski <piotr.wicijowski@gmail.com>
  *
  * ----
@@ -23,12 +23,40 @@
  */
 
 #include "autohidingsplitter.h"
+#include "support/gtkstyle.h"
+#include "support/utils.h"
 #include <QSplitterHandle>
 #include <QTimer>
 #include <QChildEvent>
 #include <QResizeEvent>
 #include <QComboBox>
 #include <QMenu>
+#include <QApplication>
+#include <QStyle>
+
+static int splitterSize(const QWidget *w)
+{
+    static int size=-1;
+
+    if (-1==size || !w || !w->isVisible()) {
+        if (Utils::touchFriendly()) {
+            size=4;
+        } else if (qApp->style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents)) {
+            int spacing=qApp->style()->pixelMetric(QStyle::PM_ScrollView_ScrollBarSpacing);
+            int splitterSize=qApp->style()->pixelMetric(QStyle::PM_SplitterWidth);
+            size=qMin(spacing+2, splitterSize);
+        } else {
+            size=GtkStyle::isActive() ? 0 : 1;
+        }
+    }
+    return size;
+}
+
+QSize AutohidingSplitterHandle::sizeHint() const
+{
+    int sz=splitterSize(this);
+    return QSize(sz, sz);
+}
 
 class SplitterSizeAnimation:public QVariantAnimation
 {
@@ -73,6 +101,7 @@ AutohidingSplitter::AutohidingSplitter(Qt::Orientation orientation, QWidget *par
     autohideAnimation->setEasingCurve(QEasingCurve::Linear);
     //connect(this, SIGNAL(splitterMoved(int, int)), this, SLOT(updateAfterSplitterMoved(int, int)));
     setMinimumWidth(32);
+    setHandleWidth(splitterSize(this));
 }
 
 AutohidingSplitter::AutohidingSplitter(QWidget *parent)
@@ -86,6 +115,7 @@ AutohidingSplitter::AutohidingSplitter(QWidget *parent)
     autohideAnimation->setEasingCurve(QEasingCurve::Linear);
     //connect(this, SIGNAL(splitterMoved(int, int)), this, SLOT(updateAfterSplitterMoved(int, int)));
     setMinimumWidth(32);
+    setHandleWidth(Utils::touchFriendly() ? 4 : 1);
 }
 
 AutohidingSplitter::~AutohidingSplitter()
