@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -21,17 +21,20 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include "covers.h"
+#include "gui/covers.h"
 #include "groupedview.h"
-#include "mpdstatus.h"
-#include "song.h"
+#include "mpd/mpdstatus.h"
+#include "mpd/song.h"
 #include "actionitemdelegate.h"
+#include "basicitemdelegate.h"
 #include "itemview.h"
 #include "config.h"
-#include "localize.h"
-#include "qtplural.h"
-#include "icons.h"
-#include "gtkstyle.h"
+#include "support/localize.h"
+#include "gui/plurals.h"
+#include "widgets/icons.h"
+#include "support/gtkstyle.h"
+#include "support/utils.h"
+#include "models/roles.h"
 #include <QStyledItemDelegate>
 #include <QApplication>
 #include <QFontMetrics>
@@ -44,13 +47,15 @@
 static int constCoverSize=32;
 static int constIconSize=16;
 static int constBorder=1;
+static double sizeAdjust=1.25;
 
 void GroupedView::setup()
 {
     int height=QApplication::fontMetrics().height();
+    sizeAdjust=Utils::touchFriendly() ? 1.5 : 1.25;
 
     if (height>17) {
-        constCoverSize=((int)((height*2)/4))*4;
+        constCoverSize=(((int)((height*2)/4))*4);
         constIconSize=Icon::stdSize(((int)(height/4))*4);
         constBorder=constCoverSize>48 ? 2 : 1;
     } else {
@@ -58,6 +63,83 @@ void GroupedView::setup()
         constIconSize=16;
         constBorder=1;
     }
+    constCoverSize*=sizeAdjust;
+}
+
+int GroupedView::coverSize()
+{
+    return constCoverSize;
+}
+
+int GroupedView::borderSize()
+{
+    return constBorder;
+}
+
+int GroupedView::iconSize()
+{
+    return constIconSize;
+}
+
+void GroupedView::drawPlayState(QPainter *painter, const QStyleOptionViewItem &option, const QRect &r, int state)
+{
+    if (state) {
+        int size=constIconSize-7;
+        int hSize=size/2;
+        QRect ir(r.x()-(size+6), r.y()+(((r.height()-size)/2.0)+0.5), size, size);
+        QColor inside(option.palette.color(QPalette::Text));
+        QColor border=inside.red()>100 && inside.blue()>100 && inside.green()>100 ? Qt::black : Qt::white;
+        if (QApplication::isRightToLeft()) {
+            ir.adjust(r.width()-size, 0, r.width()-size, 0);
+        }
+        switch (state) {
+        case GroupedView::State_Stopped:
+            painter->fillRect(ir, border);
+            painter->fillRect(ir.adjusted(1, 1, -1, -1), inside);
+            break;
+        case GroupedView::State_StopAfter: {
+            ir.adjust(2, 0, -2, 0);
+            QPoint p1[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+(ir.height())), QPoint(ir.x()-2, ir.y()+(ir.height())) };
+            QPoint p2[5]={ QPoint(ir.x()-1, ir.y()), QPoint(ir.x(), ir.y()), QPoint(ir.x()+(size-hSize)-1, ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()-1), QPoint(ir.x()-1, ir.y()+ir.height()-1) };
+            QPoint p3[3]={ QPoint(ir.x(), ir.y()+1), QPoint(ir.x()+(size-hSize)-2, ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()-2) };
+            painter->save();
+            painter->setPen(border);
+            painter->drawPolygon(p1, 5);
+            painter->drawPolygon(p3, 3);
+            painter->setPen(inside);
+            painter->drawPolygon(p2, 5);
+            painter->restore();
+            break;
+        }
+        case GroupedView::State_StopAfterTrack:
+            painter->setPen(border);
+            painter->drawRect(ir.adjusted(0, 0, -1, -1));
+            painter->drawRect(ir.adjusted(2, 2, -3, -3));
+            painter->setPen(inside);
+            painter->drawRect(ir.adjusted(1, 1, -2, -2));
+            break;
+        case GroupedView::State_Paused: {
+            int blockSize=hSize-1;
+            painter->fillRect(ir, border);
+            painter->fillRect(ir.x()+1, ir.y()+1, blockSize, size-2, inside);
+            painter->fillRect(ir.x()+size-blockSize-1, ir.y()+1, blockSize, size-2, inside);
+            break;
+        }
+        case GroupedView::State_Playing: {
+            ir.adjust(2, 0, -2, 0);
+            QPoint p1[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+(ir.height()-1)), QPoint(ir.x()-2, ir.y()+(ir.height()-1)) };
+            QPoint p2[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()), QPoint(ir.x()-2, ir.y()+ir.height()) };
+            painter->save();
+            painter->setBrush(border);
+            painter->setPen(border);
+            painter->drawPolygon(p1, 5);
+            painter->setBrush(inside);
+            painter->drawPolygon(p2, 5);
+            painter->restore();
+            break;
+        }
+        }
+    }
 }
 
 enum Type {
@@ -68,33 +150,35 @@ enum Type {
 static Type getType(const QModelIndex &index)
 {
     QModelIndex prev=index.row()>0 ? index.sibling(index.row()-1, 0) : QModelIndex();
-    quint16 thisKey=index.data(GroupedView::Role_Key).toUInt();
-    quint16 prevKey=prev.isValid() ? prev.data(GroupedView::Role_Key).toUInt() : Song::constNullKey;
+    quint16 thisKey=index.data(Cantata::Role_Key).toUInt();
+    quint16 prevKey=prev.isValid() ? prev.data(Cantata::Role_Key).toUInt() : Song::constNullKey;
 
     return thisKey==prevKey ? AlbumTrack : AlbumHeader;
 }
 
 static bool isAlbumHeader(const QModelIndex &index)
 {
-    return !index.data(GroupedView::Role_IsCollection).toBool() && AlbumHeader==getType(index);
+    return !index.data(Cantata::Role_IsCollection).toBool() && AlbumHeader==getType(index);
 }
 
 static QString streamText(const Song &song, const QString &trackTitle, bool useName=true)
 {
     if (song.album.isEmpty() && song.albumArtist().isEmpty()) {
-        return song.title.isEmpty() && song.name.isEmpty()
+        QString songName=song.name();
+        return song.title.isEmpty() && songName.isEmpty()
                 ? song.file
-                : !useName || song.name.isEmpty()
+                : !useName || songName.isEmpty()
                   ? song.title
                   : song.title.isEmpty()
-                    ? song.name
-                    : (song.title + " - " + song.name);
+                    ? songName
+                    : (song.title + " - " + songName);
     } else if (!song.title.isEmpty() && !song.artist.isEmpty()) {
-        return song.artist + " - " + (!useName || song.name.isEmpty()
+        QString name=song.name();
+        return song.artist + " - " + (!useName || name.isEmpty()
                                         ? song.title
                                         : song.title.isEmpty()
-                                            ? song.name
-                                            : (song.title + " - " + song.name));
+                                            ? name
+                                            : (song.title + " - " + name));
     } else {
         return trackTitle;
     }
@@ -115,7 +199,7 @@ public:
 
     QSize sizeHint(Type type, bool isCollection) const
     {
-        int textHeight = QApplication::fontMetrics().height();
+        int textHeight = QApplication::fontMetrics().height()*sizeAdjust;
 
         if (isCollection || AlbumHeader==type) {
             return QSize(64, qMax(constCoverSize, (qMax(constIconSize, textHeight)*2)+constBorder)+(2*constBorder));
@@ -126,58 +210,11 @@ public:
     QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
     {
         if (0==index.column()) {
-            return sizeHint(getType(index), index.data(GroupedView::Role_IsCollection).toBool());
+            return sizeHint(getType(index), index.data(Cantata::Role_IsCollection).toBool());
         }
         return QStyledItemDelegate::sizeHint(option, index);
     }
 
-    inline QString formatNumber(int num) const
-    {
-        QString text=QString::number(num);
-        return num<10 ? "0"+text : text;
-    }
-
-    static void drawFadedLine(QPainter *p, const QRect &r, const QColor &col)
-    {
-        QPoint start(r.x(), r.y());
-        QPoint end(r.x()+r.width()-1, r.y());
-        QLinearGradient grad(start, end);
-        QColor c(col);
-        c.setAlphaF(0.45);
-        QColor fade(c);
-        const int fadeSize=64;
-        double fadePos=1.0-((r.width()-fadeSize)/(r.width()*1.0));
-        bool rtl=Qt::RightToLeft==QApplication::layoutDirection();
-
-        fade.setAlphaF(0.0);
-        grad.setColorAt(0, rtl ? fade : c);
-        if(fadePos>=0 && fadePos<=1.0) {
-            if (rtl) {
-                grad.setColorAt(fadePos, c);
-            } else {
-                grad.setColorAt(1.0-fadePos, c);
-            }
-        }
-        grad.setColorAt(1, rtl ? c : fade);
-        p->save();
-        p->setPen(QPen(QBrush(grad), 1));
-        p->drawLine(start, end);
-        p->restore();
-    }
-
-    static QPainterPath buildPath(const QRectF &r, double radius)
-    {
-        QPainterPath path;
-        double diameter(radius*2);
-
-        path.moveTo(r.x()+r.width(), r.y()+r.height()-radius);
-        path.arcTo(r.x()+r.width()-diameter, r.y(), diameter, diameter, 0, 90);
-        path.arcTo(r.x(), r.y(), diameter, diameter, 90, 90);
-        path.arcTo(r.x(), r.y()+r.height()-diameter, diameter, diameter, 180, 90);
-        path.arcTo(r.x()+r.width()-diameter, r.y()+r.height()-diameter, diameter, diameter, 270, 90);
-        return path;
-    }
-
     void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
     {
         if (!index.isValid()) {
@@ -185,14 +222,14 @@ public:
         }
 
         Type type=getType(index);
-        bool isCollection=index.data(GroupedView::Role_IsCollection).toBool();
-        Song song=index.data(GroupedView::Role_Song).value<Song>();
-        int state=index.data(GroupedView::Role_Status).toInt();
-        quint32 collection=index.data(GroupedView::Role_CollectionId).toUInt();
+        bool isCollection=index.data(Cantata::Role_IsCollection).toBool();
+        Song song=index.data(Cantata::Role_Song).value<Song>();
+        int state=index.data(Cantata::Role_Status).toInt();
+        quint32 collection=index.data(Cantata::Role_CollectionId).toUInt();
         bool selected=option.state&QStyle::State_Selected;
         bool mouseOver=underMouse && option.state&QStyle::State_MouseOver;
         bool gtk=mouseOver && GtkStyle::isActive();
-        bool rtl=Qt::RightToLeft==QApplication::layoutDirection();
+        bool rtl=QApplication::isRightToLeft();
 
         if (!isCollection && AlbumHeader==type) {
             if (mouseOver && gtk) {
@@ -212,7 +249,7 @@ public:
             }
             painter->restore();
             if (!state && !view->isExpanded(song.key, collection) && view->isCurrentAlbum(song.key)) {
-                QVariant cs=index.data(GroupedView::Role_CurrentStatus);
+                QVariant cs=index.data(Cantata::Role_CurrentStatus);
                 if (cs.isValid()) {
                     state=cs.toInt();
                 }
@@ -226,21 +263,20 @@ public:
         }
         QString title;
         QString track;
-        QString duration=song.time>0 ? Song::formattedTime(song.time) : QString();
-        bool stream=!isCollection && (song.isStream() && !song.isCantataStream());
-        bool audiocd=stream && song.isCdda();
+        QString duration=song.time>0 ? Utils::formatTime(song.time) : QString();
+        bool stream=!isCollection && song.isStandardStream();
         bool isEmpty=song.title.isEmpty() && song.artist.isEmpty() && !song.file.isEmpty();
         QString trackTitle=isEmpty
                         ? song.file
-                        : !song.albumartist.isEmpty() && song.albumartist != song.artist
-                            ? song.title + " - " + song.artist
+                        : song.diffArtist()
+                            ? song.artistSong()
                             : song.title;
         QFont f(QApplication::font());
         if (stream) {
             f.setItalic(true);
         }
         QFontMetrics fm(f);
-        int textHeight=fm.height();
+        int textHeight=fm.height()*sizeAdjust;
 
 
         if (isCollection) {
@@ -248,40 +284,38 @@ public:
         } else if (AlbumHeader==type) {
             if (stream) {
                 QModelIndex next=index.sibling(index.row()+1, 0);
-                quint16 nextKey=next.isValid() ? next.data(GroupedView::Role_Key).toUInt() : Song::constNullKey;
-                if (nextKey!=song.key && !song.name.isEmpty()) {
-                    title=song.name;
+                quint16 nextKey=next.isValid() ? next.data(Cantata::Role_Key).toUInt() : Song::constNullKey;
+                if (nextKey!=song.key && !song.name().isEmpty()) {
+                    title=song.name();
                     track=streamText(song, trackTitle, false);
                 } else {
-                    title=audiocd ? i18n("Audio CD") : i18n("Streams");
+                    title=song.isCdda() ? i18n("Audio CD") : i18n("Streams");
                     track=streamText(song, trackTitle);
                 }
             } else if (isEmpty) {
-                title=i18n("Unknown");
+                title=Song::unknown();
                 track=trackTitle;
+            } else if (song.album.isEmpty()) {
+                title=song.artistOrComposer();
+                track=song.trackAndTitleStr();
             } else {
-                quint16 year=Song::albumYear(song);
-
-                if (year>0) {
-                    if (song.isFromOnlineService()) {
-                        title=song.album+QLatin1String(" (")+QString::number(year)+QLatin1Char(')');
-                    } else {
-                        title=i18nc("artist - album (albumYear)", "%1 - %2 (%3)", song.albumArtist(), song.album, QString::number(year));
-                    }
+                if (song.isFromOnlineService()) {
+                    title=Song::displayAlbum(song.album, Song::albumYear(song));
                 } else {
-                    if (song.isFromOnlineService()) {
-                        title=song.album;
-                    } else {
-                        title=i18nc("artist - album", "%1 - %2", song.albumArtist(), song.album);
+                    title=song.artistOrComposer()+QLatin1String(" - ")+Song::displayAlbum(song.album, Song::albumYear(song));
+                }
+                if (Song::useComposer()) {
+                    while (title.contains(") (")) {
+                        title=title.replace(") (", ", ");
                     }
                 }
-                track=song.track ? formatNumber(song.track)+QChar(' ')+trackTitle : trackTitle;
+                track=song.trackAndTitleStr();
             }
         } else {
             if (stream) {
                 track=streamText(song, trackTitle);
             } else {
-                track=song.track ? formatNumber(song.track)+QChar(' ')+trackTitle : trackTitle;
+                track=song.trackAndTitleStr();
             }
         }
 
@@ -305,11 +339,11 @@ public:
             g.setColorAt(0, gradCol.dark(165));
             g.setColorAt(1, gradCol.light(165));
             painter->setRenderHint(QPainter::Antialiasing, true);
-            painter->fillPath(buildPath(border, 3), g);
+            painter->fillPath(Utils::buildPath(border, 3), g);
             painter->setPen(QPen(gradCol, 1));
-            painter->drawPath(buildPath(border.adjusted(-1, -1, 1, 1), 3.5));
+            painter->drawPath(Utils::buildPath(border.adjusted(-1, -1, 1, 1), 3.5));
             painter->setPen(QPen(QApplication::palette().color(QPalette::HighlightedText), 1));
-            painter->drawPath(buildPath(border, 3));
+            painter->drawPath(Utils::buildPath(border, 3));
             painter->setRenderHint(QPainter::Antialiasing, false);
         }
 
@@ -323,31 +357,30 @@ public:
                 pix=index.data(Qt::DecorationRole).value<QIcon>().pixmap(constCoverSize, constCoverSize);
             } else {
                 QPixmap *cover=stream ? 0 : Covers::self()->get(song, constCoverSize);
-                pix=cover ? *cover : (stream && !audiocd ? Icons::self()->streamIcon : Icons::self()->albumIcon).pixmap(constCoverSize, constCoverSize);
+                pix=cover ? *cover : (stream && !song.isCdda() ? Icons::self()->streamIcon : Icons::self()->albumIcon).pixmap(constCoverSize, constCoverSize);
             }
 
             if (rtl) {
-                painter->drawPixmap(r.x()+r.width()-(pix.width()+constBorder), r.y()+((r.height()-pix.height())/2), pix.width(), pix.height(), pix);
+                painter->drawPixmap(r.x()+r.width()-(pix.width()-constBorder), r.y()+((r.height()-pix.height())/2), pix.width(), pix.height(), pix);
                 r.adjust(0, 0, -(constCoverSize+constBorder), 0);
             } else {
                 painter->drawPixmap(r.x()-2, r.y()+((r.height()-pix.height())/2), pix.width(), pix.height(), pix);
                 r.adjust(constCoverSize+constBorder, 0, 0, 0);
             }
-            int td=index.data(GroupedView::Role_AlbumDuration).toUInt();
-            QString totalDuration=td>0 ? Song::formattedTime(td) : QString();
+            int td=index.data(Cantata::Role_AlbumDuration).toUInt();
+            QString totalDuration=td>0 ? Utils::formatTime(td) : QString();
             QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
             int totalDurationWidth=fm.width(totalDuration)+8;
             QRect textRect(r.x(), r.y(), r.width()-(rtl ? (4*constBorder) : totalDurationWidth), textHeight);
             QFont tf(f);
             tf.setBold(true);
-            tf.setItalic(true);
             title = QFontMetrics(tf).elidedText(title, Qt::ElideRight, textRect.width(), QPalette::WindowText);
             painter->setFont(tf);
             painter->drawText(textRect, title, textOpt);
             if (!totalDuration.isEmpty()) {
                 painter->drawText(duratioRect, totalDuration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
             }
-            drawFadedLine(painter, r.adjusted(0, (r.height()/2)-1, 0, 0), col);
+            BasicItemDelegate::drawLine(painter, r.adjusted(0, 0, 0, -r.height()/2), col, rtl, !rtl, 0.45);
             r.adjust(0, textHeight+constBorder, 0, 0);
             r=QRect(r.x(), r.y()+r.height()-(textHeight+1), r.width(), textHeight);
             painter->setFont(f);
@@ -357,11 +390,7 @@ public:
 
             if (isCollection || !view->isExpanded(song.key, collection)) {
                 showTrackDuration=false;
-                #ifdef ENABLE_KDE_SUPPORT
-                track=i18np("1 Track", "%1 Tracks", index.data(GroupedView::Role_SongCount).toUInt());
-                #else
-                track=QTP_TRACKS_STR(index.data(GroupedView::Role_SongCount).toUInt());
-                #endif
+                track=Plurals::tracks(index.data(Cantata::Role_SongCount).toUInt());
             }
         } else if (rtl) {
             r.adjust(0, 0, -(constBorder*4), 0);
@@ -369,64 +398,9 @@ public:
             r.adjust(constCoverSize+constBorder, 0, 0, 0);
         }
 
-        if (state) {
-            int size=constIconSize-7;
-            int hSize=size/2;
-            QRect ir(r.x()-(size+6), r.y()+(((r.height()-size)/2.0)+0.5), size, size);
-            QColor inside(option.palette.color(QPalette::Text));
-            QColor border=inside.red()>100 && inside.blue()>100 && inside.green()>100 ? Qt::black : Qt::white;
-            if (rtl) {
-                ir.adjust(r.width()-size, 0, r.width()-size, 0);
-            }
-            switch (state) {
-            case GroupedView::State_Stopped:
-                painter->fillRect(ir, border);
-                painter->fillRect(ir.adjusted(1, 1, -1, -1), inside);
-                break;
-            case GroupedView::State_StopAfter: {
-                ir.adjust(2, 0, -2, 0);
-                QPoint p1[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+(ir.height())), QPoint(ir.x()-2, ir.y()+(ir.height())) };
-                QPoint p2[5]={ QPoint(ir.x()-1, ir.y()), QPoint(ir.x(), ir.y()), QPoint(ir.x()+(size-hSize)-1, ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()-1), QPoint(ir.x()-1, ir.y()+ir.height()-1) };
-                QPoint p3[3]={ QPoint(ir.x(), ir.y()+1), QPoint(ir.x()+(size-hSize)-2, ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()-2) };
-                painter->save();
-                painter->setPen(border);
-                painter->drawPolygon(p1, 5);
-                painter->drawPolygon(p3, 3);
-                painter->setPen(inside);
-                painter->drawPolygon(p2, 5);
-                painter->restore();
-                break;
-            }
-            case GroupedView::State_StopAfterTrack:
-                painter->setPen(border);
-                painter->drawRect(ir.adjusted(0, 0, -1, -1));
-                painter->drawRect(ir.adjusted(2, 2, -3, -3));
-                painter->setPen(inside);
-                painter->drawRect(ir.adjusted(1, 1, -2, -2));
-                break;
-            case GroupedView::State_Paused: {
-                int blockSize=hSize-1;
-                painter->fillRect(ir, border);
-                painter->fillRect(ir.x()+1, ir.y()+1, blockSize, size-2, inside);
-                painter->fillRect(ir.x()+size-blockSize-1, ir.y()+1, blockSize, size-2, inside);
-                break;
-            }
-            case GroupedView::State_Playing: {
-                ir.adjust(2, 0, -2, 0);
-                QPoint p1[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+(ir.height()-1)), QPoint(ir.x()-2, ir.y()+(ir.height()-1)) };
-                QPoint p2[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()), QPoint(ir.x()-2, ir.y()+ir.height()) };
-                painter->save();
-                painter->setBrush(border);
-                painter->setPen(border);
-                painter->drawPolygon(p1, 5);
-                painter->setBrush(inside);
-                painter->drawPolygon(p2, 5);
-                painter->restore();
-                break;
-            }
-            }
-            painter->setPen(col);
-        }
+        GroupedView::drawPlayState(painter, option, r, state);
+        painter->setPen(col);
+
         int durationWidth=showTrackDuration ? fm.width(duration)+8 : 0;
         QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
         QRect textRect(r.x(), r.y(), r.width()-durationWidth, textHeight);
@@ -436,10 +410,10 @@ public:
             painter->drawText(duratioRect, duration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
         }
 
-        if (mouseOver) {
-            drawIcons(painter, option.rect, true, rtl, AlbumHeader==type || isCollection ? AP_HBottom : AP_HMiddle, index);
+        if (mouseOver || Utils::touchFriendly()) {
+            drawIcons(painter, option.rect, mouseOver || (selected && Utils::touchFriendly()), rtl, AlbumHeader==type || isCollection ? AP_HBottom : AP_HMiddle, index);
         }
-        drawDivider(painter, option.rect, col);
+        BasicItemDelegate::drawLine(painter, option.rect, col);
         painter->restore();
     }
 
@@ -449,7 +423,8 @@ private:
 
 GroupedView::GroupedView(QWidget *parent, bool isPlayQueue)
     : TreeView(parent, isPlayQueue)
-    , startClosed(true)
+    , allowClose(!Utils::touchFriendly())
+    , startClosed(allowClose)
     , autoExpand(true)
     , filterActive(false)
     , isMultiLevel(false)
@@ -464,6 +439,7 @@ GroupedView::GroupedView(QWidget *parent, bool isPlayQueue)
     setHeaderHidden(true);
     setRootIsDecorated(false);
     setSelectionBehavior(SelectRows);
+    setForceSingleColumn(true);
     connect(this, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
     setStyleSheet("QTreeView::branch { border: 0px; }");
     GroupedViewDelegate *delegate=new GroupedViewDelegate(this);
@@ -481,16 +457,15 @@ GroupedView::~GroupedView()
 
 void GroupedView::setModel(QAbstractItemModel *model)
 {
-    QTreeView::setModel(model);
+    TreeView::setModel(model);
     if (model) {
-        connect(Covers::self(), SIGNAL(coverRetrieved(const Song &)), this, SLOT(coverRetrieved(const Song &)));
-        int columnCount=model->columnCount();
-        QHeaderView *hdr=header();
-        for (int i=1; i<columnCount; ++i) {
-            hdr->setSectionHidden(i, true);
+        if (startClosed) {
+            updateCollectionRows();
         }
+        connect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
     } else {
-        disconnect(Covers::self(), SIGNAL(coverRetrieved(const Song &)), this, SLOT(coverRetrieved(const Song &)));
+        controlledAlbums.clear();
+        disconnect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
     }
 }
 
@@ -519,14 +494,17 @@ void GroupedView::setFilterActive(bool f)
 
 void GroupedView::setStartClosed(bool sc)
 {
-    if (sc==startClosed) {
+    if (!allowClose || sc==startClosed) {
         return;
     }
     controlledAlbums.clear();
     startClosed=sc;
+    if (startClosed) {
+        updateCollectionRows();
+    }
 }
 
-void GroupedView::updateRows(qint32 row, quint16 curAlbum, bool scroll, const QModelIndex &parent)
+void GroupedView::updateRows(qint32 row, quint16 curAlbum, bool scroll, const QModelIndex &parent, bool forceScroll)
 {
     currentAlbum=curAlbum;
 
@@ -546,7 +524,7 @@ void GroupedView::updateRows(qint32 row, quint16 curAlbum, bool scroll, const QM
     }
 
     updateRows(parent);
-    if (MPDState_Playing==MPDStatus::self()->state() && scroll) {
+    if (scroll && (MPDState_Playing==MPDStatus::self()->state() || forceScroll)) {
         scrollTo(model()->index(row, 0, parent), QAbstractItemView::PositionAtCenter);
     }
 }
@@ -559,11 +537,11 @@ void GroupedView::updateRows(const QModelIndex &parent)
 
     qint32 count=model()->rowCount(parent);
     quint16 lastKey=Song::constNullKey;
-    quint32 collection=parent.data(GroupedView::Role_CollectionId).toUInt();
+    quint32 collection=parent.data(Cantata::Role_CollectionId).toUInt();
     QSet<quint16> keys;
 
     for (qint32 i=0; i<count; ++i) {
-        quint16 key=model()->index(i, 0, parent).data(GroupedView::Role_Key).toUInt();
+        quint16 key=model()->index(i, 0, parent).data(Cantata::Role_Key).toUInt();
         keys.insert(key);
         bool hide=key==lastKey &&
                         !(key==currentAlbum && autoExpand) &&
@@ -592,13 +570,17 @@ void GroupedView::updateCollectionRows()
 
 void GroupedView::toggle(const QModelIndex &idx)
 {
-    quint16 indexKey=idx.data(GroupedView::Role_Key).toUInt();
+    if (!allowClose) {
+        return;
+    }
+
+    quint16 indexKey=idx.data(Cantata::Role_Key).toUInt();
 
     if (indexKey==currentAlbum && autoExpand) {
         return;
     }
 
-    quint32 collection=idx.data(GroupedView::Role_CollectionId).toUInt();
+    quint32 collection=idx.data(Cantata::Role_CollectionId).toUInt();
     bool toBeHidden=false;
     if (controlledAlbums[collection].contains(indexKey)) {
         controlledAlbums[collection].remove(indexKey);
@@ -613,7 +595,7 @@ void GroupedView::toggle(const QModelIndex &idx)
         quint32 count=model()->rowCount(idx.parent());
         for (quint32 i=0; i<count; ++i) {
             QModelIndex index=model()->index(i, 0, parent);
-            quint16 key=index.data(GroupedView::Role_Key).toUInt();
+            quint16 key=index.data(Cantata::Role_Key).toUInt();
             if (indexKey==key) {
                 if (isAlbumHeader(index)) {
                     dataChanged(index, index);
@@ -638,16 +620,16 @@ QModelIndexList GroupedView::selectedIndexes(bool sorted) const
             indexSet.insert(idx);
             sel.append(idx);
         }
-        if (!idx.data(GroupedView::Role_IsCollection).toBool()) {
-            quint16 key=idx.data(GroupedView::Role_Key).toUInt();
-            quint32 collection=idx.data(GroupedView::Role_CollectionId).toUInt();
+        if (!idx.data(Cantata::Role_IsCollection).toBool()) {
+            quint16 key=idx.data(Cantata::Role_Key).toUInt();
+            quint32 collection=idx.data(Cantata::Role_CollectionId).toUInt();
             if (!isExpanded(key, collection)) {
                 quint32 rowCount=model()->rowCount(idx.parent());
                 for (quint32 i=idx.row()+1; i<rowCount; ++i) {
                     QModelIndex next=idx.sibling(i, 0);
                     if (next.isValid()) {
-                        quint16 nextKey=next.data(GroupedView::Role_Key).toUInt();
-                        quint32 nextCollection=idx.data(GroupedView::Role_CollectionId).toUInt();
+                        quint16 nextKey=next.data(Cantata::Role_Key).toUInt();
+                        quint32 nextCollection=idx.data(Cantata::Role_CollectionId).toUInt();
 
                         if (nextCollection==collection && nextKey==key) {
                             if (!indexSet.contains(next)) {
@@ -683,15 +665,15 @@ void GroupedView::dropEvent(QDropEvent *event)
         if (idx.isValid() && isAlbumHeader(idx)) {
             QRect rect(visualRect(idx));
             if (event->pos().y()>(rect.y()+(rect.height()/2))) {
-                quint16 key=idx.data(GroupedView::Role_Key).toUInt();
-                quint32 collection=idx.data(GroupedView::Role_CollectionId).toUInt();
+                quint16 key=idx.data(Cantata::Role_Key).toUInt();
+                quint32 collection=idx.data(Cantata::Role_CollectionId).toUInt();
                 if (!isExpanded(key, collection)) {
                     parent=idx.parent();
                     quint32 rowCount=model()->rowCount(parent);
                     for (quint32 i=idx.row()+1; i<rowCount; ++i) {
                         QModelIndex next=idx.sibling(i, 0);
                         if (next.isValid()) {
-                            quint16 nextKey=next.data(GroupedView::Role_Key).toUInt();
+                            quint16 nextKey=next.data(Cantata::Role_Key).toUInt();
                             if (nextKey==key) {
                                 dropRowAdjust++;
                             } else {
@@ -701,24 +683,25 @@ void GroupedView::dropEvent(QDropEvent *event)
                             break;
                         }
                     }
-                    model()->setData(parent, dropRowAdjust, Role_DropAdjust);
+                    model()->setData(parent, dropRowAdjust, Cantata::Role_DropAdjust);
                 }
             }
         }
     }
 
     TreeView::dropEvent(event);
-    model()->setData(parent, 0, Role_DropAdjust);
+    model()->setData(parent, 0, Cantata::Role_DropAdjust);
 }
 
-void GroupedView::coverRetrieved(const Song &s)
+void GroupedView::coverLoaded(const Song &song, int size)
 {
-    if (filterActive || !isVisible()) {
+    if (filterActive || !isVisible() || size!=constCoverSize || song.isArtistImageRequest()) {
         return;
     }
-
     quint32 count=model()->rowCount();
     quint16 lastKey=Song::constNullKey;
+    QString albumArtist=song.albumArtist();
+    QString album=song.album;
 
     for (quint32 i=0; i<count; ++i) {
         QModelIndex index=model()->index(i, 0);
@@ -726,28 +709,29 @@ void GroupedView::coverRetrieved(const Song &s)
             continue;
         }
         if (isMultiLevel && model()->hasChildren(index)) {
+            lastKey=Song::constNullKey;
             quint32 childCount=model()->rowCount(index);
             for (quint32 c=0; c<childCount; ++c) {
-                QModelIndex child=model()->index(i, 0, index);
+                QModelIndex child=model()->index(c, 0, index);
                 if (!child.isValid()) {
                     continue;
                 }
-                quint16 key=child.data(GroupedView::Role_Key).toUInt();
+                quint16 key=child.data(Cantata::Role_Key).toUInt();
 
-                if (key!=lastKey && !isRowHidden(i, QModelIndex())) {
-                    Song song=child.data(GroupedView::Role_Song).value<Song>();
-                    if (song.albumArtist()==s.albumArtist() && song.album==s.album) {
+                if (key!=lastKey && !isRowHidden(c, index)) {
+                    Song song=child.data(Cantata::Role_Song).value<Song>();
+                    if (song.albumArtist()==albumArtist && song.album==album) {
                         dataChanged(child, child);
                     }
                 }
                 lastKey=key;
             }
         } else {
-            quint16 key=index.data(GroupedView::Role_Key).toUInt();
+            quint16 key=index.data(Cantata::Role_Key).toUInt();
 
             if (key!=lastKey && !isRowHidden(i, QModelIndex())) {
-                Song song=index.data(GroupedView::Role_Song).value<Song>();
-                if (song.albumArtist()==s.albumArtist() && song.album==s.album) {
+                Song song=index.data(Cantata::Role_Song).value<Song>();
+                if (song.albumArtist()==albumArtist && song.album==album) {
                     dataChanged(index, index);
                 }
             }
@@ -771,17 +755,17 @@ void GroupedView::itemClicked(const QModelIndex &idx)
         header.setHeight(header.height()/2);
         header.moveTo(viewport()->mapToGlobal(QPoint(header.x(), header.y())));
         icon.moveTo(viewport()->mapToGlobal(QPoint(icon.x(), icon.y())));
-        if (icon.contains(QCursor::pos())) {
+        if (allowClose && icon.contains(QCursor::pos())) {
             toggle(idx);
         } else if (header.contains(QCursor::pos())) {
             QModelIndexList list;
-            unsigned int key=idx.data(GroupedView::Role_Key).toUInt();
+            unsigned int key=idx.data(Cantata::Role_Key).toUInt();
             QModelIndex i=idx.sibling(idx.row()+1, 0);
 //            QModelIndexList sel=selectedIndexes();
             QItemSelectionModel *selModel=selectionModel();
             QModelIndexList unsel;
 
-            while (i.isValid() && i.data(GroupedView::Role_Key).toUInt()==key) {
+            while (i.isValid() && i.data(Cantata::Role_Key).toUInt()==key) {
                 #if 0 // The following does not seem to work from the grouped playlist view - the 2nd row never get selected!
                 if (!sel.contains(i)) {
                     unsel.append(i);
@@ -794,7 +778,7 @@ void GroupedView::itemClicked(const QModelIndex &idx)
             }
             if (list.count()) {
                 #if 0 // Commendted out as (as noted below) unselection is not working, and we always add to 'unsel' above (because of playlists)
-                if (unsel.isEmpty()) { // TODO: This is not working!!!
+                if (unsel.isEmpty()) { // TODO: This is not working!!!   CHECK selModel if re-add!!!
                     foreach(const QModelIndex &i, list) {
                         selModel->select(i, QItemSelectionModel::Deselect|QItemSelectionModel::Rows);
                     }
@@ -804,7 +788,7 @@ void GroupedView::itemClicked(const QModelIndex &idx)
                     }
                 }
                 #else
-                if (!unsel.isEmpty()) {
+                if (!unsel.isEmpty() && selModel) {
                     foreach(const QModelIndex &i, unsel) {
                         selModel->select(i, QItemSelectionModel::Select|QItemSelectionModel::Rows);
                     }
@@ -815,22 +799,38 @@ void GroupedView::itemClicked(const QModelIndex &idx)
     }
 }
 
-void GroupedView::expand(const QModelIndex &idx)
+void GroupedView::expand(const QModelIndex &idx, bool singleOnly)
 {
-    if (idx.isValid()) {
-        if (idx.data(GroupedView::Role_IsCollection).toBool()) {
+    if (allowClose && idx.isValid()) {
+        if (idx.data(Cantata::Role_IsCollection).toBool()) {
             setExpanded(idx, true);
-            quint32 count=model()->rowCount(idx);
-            for (quint32 i=0; i<count; ++i) {
-                expand(idx.child(i, 0));
+            if (!singleOnly) {
+                quint32 count=model()->rowCount(idx);
+                for (quint32 i=0; i<count; ++i) {
+                    expand(idx.child(i, 0));
+                }
             }
-        }
-        else if (AlbumHeader==getType(idx)) {
-            quint16 indexKey=idx.data(GroupedView::Role_Key).toUInt();
-            quint32 collection=idx.data(GroupedView::Role_CollectionId).toUInt();
+        } else if (AlbumHeader==getType(idx)) {
+            quint16 indexKey=idx.data(Cantata::Role_Key).toUInt();
+            quint32 collection=idx.data(Cantata::Role_CollectionId).toUInt();
             if (!isExpanded(indexKey, collection)) {
                 toggle(idx);
             }
         }
     }
 }
+
+void GroupedView::collapse(const QModelIndex &idx, bool singleOnly)
+{
+    if (allowClose && idx.isValid()) {
+        if (idx.data(Cantata::Role_IsCollection).toBool()) {
+            setExpanded(idx, false);
+            if (!singleOnly) {
+                quint32 count=model()->rowCount(idx);
+                for (quint32 i=0; i<count; ++i) {
+                    collapse(idx.child(i, 0));
+                }
+            }
+        }
+    }
+}
