@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,7 +23,9 @@
 
 #include "icons.h"
 #include "config.h"
-#include "settings.h"
+#include "gui/settings.h"
+#include "support/globalstatic.h"
+#include "support/utils.cpp"
 #include <QApplication>
 #include <QPixmap>
 #include <QFont>
@@ -32,51 +34,34 @@
 #include <QDir>
 #include <QFile>
 #include <math.h>
-#ifndef Q_OS_WIN
-#include "gtkstyle.h"
+#if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
+#include "support/gtkstyle.h"
 #endif
 
-#ifdef ENABLE_KDE_SUPPORT
-K_GLOBAL_STATIC(Icons, instance)
-#endif
-
-Icons * Icons::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static Icons *instance=0;
-    if(!instance) {
-        instance=new Icons;
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(Icons, instance)
 
 static QList<int> constStdSizes=QList<int>() << 16 << 22 << 32 << 48;
-static const double constDisabledOpacity=0.5;
-static const int constShadeFactor=75;
+
+static const int constDarkLimit=80;
+static const int constDarkValue=64;
+static const int constLightLimit=240;
+static const int constLightValue=240;
 
 static bool inline isLight(const QColor &col)
 {
     return col.red()>100 && col.blue()>100 && col.green()>100;
 }
 
-static bool inline isVeryLight(const QColor &col, int limit=200)
+static bool inline isVeryLight(const QColor &col, int limit=constLightValue)
 {
     return col.red()>=limit && col.blue()>=limit && col.green()>=limit;
 }
 
-static bool inline isVeryDark(const QColor &col, int limit=80)
+static bool inline isVeryDark(const QColor &col, int limit=constDarkValue)
 {
     return col.red()<limit && col.blue()<limit && col.green()<limit;
 }
 
-static const int constDarkLimit=80;
-static const int constDarkValue=64;
-static const int constLightLimit=240;
-static const int constLightValue=240;
-
 static QColor clampColor(const QColor &color, int darkLimit=constDarkLimit, int darkValue=constDarkValue,
                          int lightLimit=constLightLimit, int lightValue=constLightValue)
 {
@@ -136,21 +121,11 @@ static QPixmap createConsumeIconPixmap(int size, const QColor &col, double opaci
     return pix;
 }
 
-static QPainterPath buildPath(const QRectF &r, double radius)
-{
-    QPainterPath path;
-    double diameter(radius*2);
-
-    path.moveTo(r.x()+r.width(), r.y()+r.height()-radius);
-    path.arcTo(r.x()+r.width()-diameter, r.y(), diameter, diameter, 0, 90);
-    path.arcTo(r.x(), r.y(), diameter, diameter, 90, 90);
-    path.arcTo(r.x(), r.y()+r.height()-diameter, diameter, diameter, 180, 90);
-    path.arcTo(r.x()+r.width()-diameter, r.y()+r.height()-diameter, diameter, diameter, 270, 90);
-    return path;
-}
-
+#ifndef USE_SYSTEM_MENU_ICON
 static QPixmap createMenuIconPixmap(int size, QColor col, double opacity=1.0)
 {
+    static const int constShadeFactor=75;
+
     QPixmap pix(size, size);
     pix.fill(Qt::transparent);
     QPainter p(&pix);
@@ -170,6 +145,7 @@ static QPixmap createMenuIconPixmap(int size, QColor col, double opacity=1.0)
         space=6;
         borderX=4;
     }
+
     int borderY=((size-((3*lineWidth)+(2*space)))/2.0)+0.5;
 
     p.setOpacity(opacity);
@@ -188,58 +164,48 @@ static QPixmap createMenuIconPixmap(int size, QColor col, double opacity=1.0)
         grad.setColorAt(0, col);
         col.setAlphaF(light ? 1.0 : 0.5);
         grad.setColorAt(1, col);
-        p.fillPath(buildPath(rect, lineWidth/2.0), grad);
+        p.fillPath(Utils::buildPath(rect, lineWidth/2.0), grad);
     }
     p.end();
     return pix;
 }
+#endif
 
-static void calcIconColors(QColor &stdColor, QColor &highlightColor)
+static QColor calcIconColor()
 {
     QColor bgnd=QApplication::palette().color(QPalette::Active, QPalette::Background);
-    QColor text=QApplication::palette().color(QPalette::Active, QPalette::ButtonText);
-    stdColor=clampColor(text, constDarkLimit, bgnd.value()<224 ? 32 : 48);
-    highlightColor=isLight(stdColor) ? stdColor.lighter(constShadeFactor) : stdColor.darker(constShadeFactor);
+    QColor text=QApplication::palette().color(QPalette::Active, QPalette::WindowText);
+    return clampColor(text, constDarkLimit, bgnd.value()<224 ? 32 : 48);
 }
 
-static Icon createSingleIcon(const QColor &stdColor, const QColor &highlightColor)
+static Icon createSingleIcon(const QColor &stdColor)
 {
     Icon icon;
-
     foreach (int s, constStdSizes) {
         icon.addPixmap(createSingleIconPixmap(s, stdColor));
-        icon.addPixmap(createSingleIconPixmap(s, stdColor, constDisabledOpacity), QIcon::Disabled);
-        icon.addPixmap(createSingleIconPixmap(s, highlightColor), QIcon::Active);
     }
-
     return icon;
 }
 
-static Icon createConsumeIcon(const QColor &stdColor, const QColor &highlightColor)
+static Icon createConsumeIcon(const QColor &stdColor)
 {
     Icon icon;
-
     foreach (int s, constStdSizes) {
         icon.addPixmap(createConsumeIconPixmap(s, stdColor));
-        icon.addPixmap(createConsumeIconPixmap(s, stdColor, constDisabledOpacity), QIcon::Disabled);
-        icon.addPixmap(createConsumeIconPixmap(s, highlightColor), QIcon::Active);
     }
-
     return icon;
 }
 
-static Icon createMenuIcon(const QColor &stdColor, const QColor &highlightColor)
+#ifndef USE_SYSTEM_MENU_ICON
+static Icon createMenuIcon(const QColor &stdColor)
 {
     Icon icon;
-
     foreach (int s, constStdSizes) {
         icon.addPixmap(createMenuIconPixmap(s, stdColor));
-        icon.addPixmap(createMenuIconPixmap(s, stdColor, constDisabledOpacity), QIcon::Disabled);
-        icon.addPixmap(createMenuIconPixmap(s, highlightColor), QIcon::Active);
     }
-
     return icon;
 }
+#endif
 
 static void recolourPix(QImage &img, const QColor &col, double opacity=1.0)
 {
@@ -285,7 +251,7 @@ static QPixmap recolour(const QImage &img, const QColor &col, double opacity=1.0
     return QPixmap::fromImage(i);
 }
 
-static Icon createRecolourableIcon(const QString &name, const QColor &stdColor, const QColor &highlightColor)
+static Icon createRecolourableIcon(const QString &name, const QColor &stdColor)
 {
     if (QColor(Qt::black)==stdColor) {
         // Text colour is black, so is icon, therefore no need to recolour!!!
@@ -298,44 +264,38 @@ static Icon createRecolourableIcon(const QString &name, const QColor &stdColor,
         QImage img(QChar(':')+name+QString::number(s));
         if (!img.isNull()) {
             icon.addPixmap(recolour(img, stdColor));
-            icon.addPixmap(recolour(img, stdColor, constDisabledOpacity), QIcon::Disabled);
-            icon.addPixmap(recolour(img, highlightColor), QIcon::Active);
         }
     }
     return icon;
 }
 
-static QColor stdColor;
-static QColor highlightColor;
-
-static void updateSidebarIcon(Icon &i, const QString &name, const QColor &color, QIcon::Mode mode)
+static void updateMonoSvgIcon(Icon &i, const QString &type, const QString &name, const QColor &color, QIcon::Mode mode)
 {
-    QColor adjusted=color;
     int darkValue=constDarkValue;
     int lightValue=constLightValue;
 
-    if (isVeryDark(adjusted)) {
+    if (isVeryDark(color)) {
         QColor bgnd=QApplication::palette().color(QPalette::Active, QPalette::Background);
         if (bgnd.value()<224) {
             darkValue=48;
         }
-    } else if (isVeryLight(adjusted)) {
+    } else if (isVeryLight(color)) {
         QColor bgnd=QApplication::palette().color(QPalette::Active, QPalette::Background);
         if (bgnd.value()<224) {
             lightValue=232;
         }
     }
 
-    if (darkValue==constDarkValue && isVeryDark(adjusted)) {
-        i.addFile(":sidebar-"+name+"-dark", QSize(), mode);
-    } else if (lightValue==constLightValue && isVeryLight(adjusted)) {
-        i.addFile(":sidebar-"+name+"-light", QSize(), mode);
+    if (darkValue==constDarkValue && isVeryDark(color)) {
+        i.addFile(":"+type+"-"+name+"-dark", QSize(), mode);
+    } else if (lightValue==constLightValue && isVeryLight(color)) {
+        i.addFile(":"+type+"-"+name+"-light", QSize(), mode);
     } else { // Neither black nor white, so we need to recolour...
         Icon std;
-        std.addFile(":sidebar-"+name+"-dark", QSize(), mode);
+        std.addFile(":"+type+"-"+name+"-dark", QSize(), mode);
         // Now recolour the icon!
         QList<int> sizes=QList<int>() << 16 << 22 << 32 << 48 << 64;
-        QColor col=clampColor(adjusted, constDarkLimit, darkValue, constLightLimit, lightValue);
+        QColor col=clampColor(color, constDarkLimit, darkValue, constLightLimit, lightValue);
         foreach (int s, sizes) {
             QImage img=std.pixmap(s, s, mode).toImage().convertToFormat(QImage::Format_ARGB32);
             recolourPix(img, col);
@@ -344,160 +304,25 @@ static void updateSidebarIcon(Icon &i, const QString &name, const QColor &color,
     }
 }
 
-static Icon loadSidebarIcon(const QString &name, const QColor &normal, const QColor &selected)
+static Icon loadMonoSvgIcon(const QString &type, const QString &name, const QColor &normal, const QColor &selected)
 {
     Icon i;
-    updateSidebarIcon(i, name, normal, QIcon::Normal);
+    updateMonoSvgIcon(i, type, name, normal, QIcon::Normal);
     if (normal!=selected) {
-        updateSidebarIcon(i, name, selected, QIcon::Selected);
+        updateMonoSvgIcon(i, type, name, selected, QIcon::Selected);
     }
     return i;
 }
 
-Icons::Icons()
+static Icon loadSidebarIcon(const QString &name, const QColor &normal, const QColor &selected)
 {
-    calcIconColors(stdColor, highlightColor);
-    singleIcon=createSingleIcon(stdColor, highlightColor);
-    consumeIcon=createConsumeIcon(stdColor, highlightColor);
-    menuIcon=createMenuIcon(stdColor, highlightColor);
-    streamCategoryIcon=Icon(QLatin1String("oxygen")==Icon::currentTheme().toLower() ? "inode-directory" : "folder-music");
-    QString iconFile=QString(INSTALL_PREFIX"/share/")+QCoreApplication::applicationName()+"/icons/stream.png";
-    if (QFile::exists(iconFile)) {
-        streamIcon.addFile(iconFile);
-    }
-    if (streamIcon.isNull()) {
-        streamIcon=Icon("applications-internet");
-    }
-    albumIcon=Icon("media-optical");
-    audioFileIcon=Icon("audio-x-generic");
-    playlistIcon=Icon("view-media-playlist");
-    dynamicRuleIcon=Icon("media-playlist-shuffle");
-    configureIcon=Icon("configure");
-    connectIcon=Icon("dialog-ok");
-    disconnectIcon=Icon("media-eject");
-    speakerIcon=Icon("speaker");
-    editIcon=Icon("document-edit");
-    searchIcon=Icon("edit-find");
-    clearListIcon=Icon("edit-clear-list");
-    repeatIcon=createRecolourableIcon("repeat", stdColor, highlightColor);
-    shuffleIcon=createRecolourableIcon("shuffle", stdColor, highlightColor);
-    filesIcon=Icon("document-multiple");
-    cancelIcon=Icon("dialog-cancel");
-    importIcon=Icon("document-import");
-    if (editIcon.isNull()) {
-        editIcon=Icon("text-editor");
-    }
-    if (importIcon.isNull()) {
-        importIcon=Icon("down");
-    }
-    libraryIcon=Icon::create("lib", constStdSizes);
-    radioStreamIcon=Icon::create("radio", constStdSizes);
-    addRadioStreamIcon=Icon::create("addradio", constStdSizes);
-    variousArtistsIcon=Icon::create("va", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128);
-    artistIcon=Icon::create("artist", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128);
-    #ifndef ENABLE_KDE_SUPPORT
-    QList<int> appSizes=QList<int>() << 16 << 22 << 24 << 32 << 48 << 64;
-    foreach (int s, appSizes) {
-        appIcon.addFile(QString(":cantata%1.png").arg(s), QSize(s, s));
-    }
-    appIcon.addFile(":cantata.svg");
-
-    shortcutsIcon=Icon("preferences-desktop-keyboard");
-    #ifndef Q_OS_WIN
-    if (shortcutsIcon.isNull()) {
-        shortcutsIcon=Icon("keyboard");
-    }
-    if (albumIcon.isNull()) {
-        albumIcon=Icon("media-optical-audio");
-    }
-    if (configureIcon.isNull()) {
-        configureIcon=Icon("gtk-preferences");
-    }
-    if (connectIcon.isNull()) {
-        connectIcon=Icon("gtk-stock-ok");
-        if (connectIcon.isNull()) {
-            connectIcon=Icon("go-bottom");
-        }
-    }
-    if (speakerIcon.isNull()) {
-        speakerIcon=Icon("audio-speakers");
-        if (speakerIcon.isNull()) {
-            speakerIcon=Icon("gnome-volume-control");
-        }
-    }
-    if (dynamicRuleIcon.isNull()) {
-        dynamicRuleIcon=Icon("text-x-generic");
-    }
-    if (playlistIcon.isNull()) {
-        playlistIcon=Icon("audio-x-mp3-playlist");
-        if (playlistIcon.isNull()) {
-            playlistIcon=Icon("audio-x-generic");
-        }
-    }
-    if (editIcon.isNull()) {
-        editIcon=Icon("gtk-edit");
-    }
-    if (clearListIcon.isNull()) {
-        clearListIcon=Icon("edit-clear");
-    }
-    if (filesIcon.isNull()) {
-        filesIcon=Icon("empty");
-    }
-    if (cancelIcon.isNull()) {
-        cancelIcon=Icon("gtk-cancel");
-    }
-    #endif // Q_OS_WIN
-    #endif // ENABLE_KDE_SUPPORT
-
-    if (streamCategoryIcon.isNull()) {
-        streamCategoryIcon=libraryIcon;
-    }
+    return loadMonoSvgIcon(QLatin1String("sidebar"), name, normal, selected);
 }
 
-void Icons::initSidebarIcons()
-{
-    if (Settings::self()->monoSidebarIcons()) {
-        QColor textCol=QApplication::palette().color(QPalette::Active, QPalette::ButtonText);
-        QColor highlightedTexCol=QApplication::palette().color(QPalette::Active, QPalette::HighlightedText);
-        playqueueIcon=loadSidebarIcon("playqueue", textCol, highlightedTexCol);
-        artistsIcon=loadSidebarIcon("artists", textCol, highlightedTexCol);
-        albumsIcon=loadSidebarIcon("albums", textCol, highlightedTexCol);
-        foldersIcon=loadSidebarIcon("folders", textCol, highlightedTexCol);
-        playlistsIcon=loadSidebarIcon("playlists", textCol, highlightedTexCol);
-        dynamicIcon=loadSidebarIcon("dynamic", textCol, highlightedTexCol);
-        streamsIcon=loadSidebarIcon("streams", textCol, highlightedTexCol);
-        onlineIcon=loadSidebarIcon("online", textCol, highlightedTexCol);
-        infoSidebarIcon=loadSidebarIcon("info", textCol, highlightedTexCol);
-        #ifdef ENABLE_DEVICES_SUPPORT
-        devicesIcon=loadSidebarIcon("devices", textCol, highlightedTexCol);
-        #endif
-    } else {
-        playqueueIcon=Icon("media-playback-start");
-        artistsIcon=artistIcon;
-        albumsIcon=albumIcon;
-        foldersIcon=Icon("inode-directory");
-        playlistsIcon=playlistIcon;
-        dynamicIcon=dynamicRuleIcon;
-        streamsIcon=radioStreamIcon;
-        onlineIcon=Icon("applications-internet");
-        if (QLatin1String("gnome")==Icon::currentTheme().toLower()) {
-            QColor col=QApplication::palette().color(QPalette::Active, QPalette::ButtonText);
-            infoSidebarIcon=loadSidebarIcon("info", col, col);
-        } else {
-            infoSidebarIcon=Icon("dialog-information");
-        }
-        #ifdef ENABLE_DEVICES_SUPPORT
-        devicesIcon=Icon("multimedia-player");
-        #endif
-    }
-}
-
-#if !defined ENABLE_KDE_SUPPORT && !defined Q_OS_WIN
-// For some reason, the -symbolic icons on Ubuntu have a lighter colour when disabled!
-// This looks odd to me, so base the disabled icon on the enabled version but with opacity
-// set to default value...
+#if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
 static void setDisabledOpacity(Icon &icon)
 {
+    static const double constDisabledOpacity=0.5;
     Icon copy;
     for (int i=0; i<2; ++i) {
         QIcon::State state=(QIcon::State)i;
@@ -524,49 +349,238 @@ static void setDisabledOpacity(Icon &icon)
     }
     icon=copy;
 }
-#else
-#define setDisabledOpacity(A) ;
+
+static Icon loadMediaIcon(const QString &name, const QColor &normal, const QColor &selected)
+{
+    Icon icon=loadMonoSvgIcon(QLatin1String("media"), name, normal, selected);
+    setDisabledOpacity(icon);
+    return icon;
+}
 #endif
 
-void Icons::initToolbarIcons(const QColor &color, bool forceLight)
+Icons::Icons()
 {
-    bool light=forceLight || isLight(color);
+    QColor stdColor=calcIconColor();
+    singleIcon=createSingleIcon(stdColor);
+    consumeIcon=createConsumeIcon(stdColor);
+    #ifdef USE_SYSTEM_MENU_ICON
+    menuIcon=Icon("applications-system");
+    #else
+    menuIcon=createMenuIcon(stdColor);
+    #endif
 
-    if (light) {
-        QColor col(Qt::white);
-        QColor highlight(col.darker(constShadeFactor));
-        toolbarMenuIcon=createMenuIcon(col, highlight);
-    } else {
-        toolbarMenuIcon=menuIcon;
+    #ifdef ENABLE_STREAMS
+    streamCategoryIcon=Icon(QLatin1String("oxygen")==Icon::currentTheme().toLower() ? "inode-directory" : "folder-music");
+    #endif
+
+    QString iconFile=QString(CANTATA_SYS_ICONS_DIR+"stream.png");
+    if (QFile::exists(iconFile)) {
+        streamIcon.addFile(iconFile);
     }
+    if (streamIcon.isNull()) {
+        streamIcon=Icon("applications-internet");
+    }
+    albumIcon=Icon("media-optical");
+    podcastIcon=Icon("inode-directory");
+    downloadedPodcastEpisodeIcon=Icon("document-save-as");
+    audioFileIcon=Icon("audio-x-generic");
+    playlistIcon=Icon("view-media-playlist");
+    folderIcon=Icon("inode-directory");
+    dynamicRuleIcon=Icon("media-playlist-shuffle");
+    configureIcon=Icon("configure");
+    connectIcon=Icon("dialog-ok");
+    disconnectIcon=Icon("media-eject");
+    speakerIcon=Icon("speaker");
+    editIcon=Icon("document-edit");
+    searchIcon=Icon("edit-find");
+    clearListIcon=Icon("edit-clear-list");
+    repeatIcon=createRecolourableIcon("repeat", stdColor);
+    shuffleIcon=createRecolourableIcon("shuffle", stdColor);
+    filesIcon=Icon("document-multiple");
+    cancelIcon=Icon("dialog-cancel");
+    importIcon=Icon("document-import");
+    if (editIcon.isNull()) {
+        editIcon=Icon("text-editor");
+    }
+    if (importIcon.isNull()) {
+        importIcon=Icon("down");
+    }
+    radioStreamIcon=Icon::create("radio", constStdSizes);
+    addRadioStreamIcon=Icon::create("addradio", constStdSizes);
+    variousArtistsIcon=Icon::create("va", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128, true);
+    artistIcon=Icon::create("artist", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128, true);
+    lastFmIcon.addFile(":lastfm.svg");
+    #ifndef ENABLE_KDE_SUPPORT
+    #ifndef ENABLE_UBUNTU
+    appIcon=Icon("cantata");
+    #endif // ENABLE_UBUNTU
+    shortcutsIcon=Icon("preferences-desktop-keyboard");
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
+    if (shortcutsIcon.isNull()) {
+        shortcutsIcon=Icon("keyboard");
+    }
+    if (albumIcon.isNull()) {
+        albumIcon=Icon("media-optical-audio");
+    }
+    if (configureIcon.isNull()) {
+        configureIcon=Icon("gtk-preferences");
+    }
+    if (connectIcon.isNull()) {
+        connectIcon=Icon("gtk-stock-ok");
+        if (connectIcon.isNull()) {
+            connectIcon=Icon("go-bottom");
+        }
+    }
+    if (speakerIcon.isNull()) {
+        speakerIcon=Icon("audio-speakers");
+        if (speakerIcon.isNull()) {
+            speakerIcon=Icon("gnome-volume-control");
+        }
+    }
+    #ifdef ENABLE_DYNAMIC
+    if (dynamicRuleIcon.isNull()) {
+        dynamicRuleIcon=Icon("text-x-generic");
+    }
+    #endif
+    if (playlistIcon.isNull()) {
+        playlistIcon=Icon("audio-x-mp3-playlist");
+        if (playlistIcon.isNull()) {
+            playlistIcon=audioFileIcon;
+        }
+    }
+    if (editIcon.isNull()) {
+        editIcon=Icon("gtk-edit");
+    }
+    if (clearListIcon.isNull()) {
+        clearListIcon=Icon("gtk-delete");
+    }
+    if (filesIcon.isNull()) {
+        filesIcon=Icon("empty");
+    }
+    if (cancelIcon.isNull()) {
+        cancelIcon=Icon("gtk-cancel");
+    }
+    #endif // Q_OS_WIN && Q_OS_MAC
+    #endif // ENABLE_KDE_SUPPORT
 
-    #ifndef Q_OS_WIN
-    if (light && GtkStyle::useSymbolicIcons()) {
-        toolbarPrevIcon=Icon("media-skip-backward-symbolic");
-        toolbarPlayIcon=Icon("media-playback-start-symbolic");
-        toolbarPauseIcon=Icon("media-playback-pause-symbolic");
-        toolbarStopIcon=Icon("media-playback-stop-symbolic");
-        toolbarNextIcon=Icon("media-skip-forward-symbolic");
-        toolbarVolumeMutedIcon=Icon("audio-volume-muted-symbolic");
-        toolbarVolumeLowIcon=Icon("audio-volume-low-symbolic");
-        toolbarVolumeMediumIcon=Icon("audio-volume-medium-symbolic");
-        toolbarVolumeHighIcon=Icon("audio-volume-high-symbolic");
-        QColor col(196, 196, 196);
+    #ifdef ENABLE_STREAMS
+    if (streamCategoryIcon.isNull()) {
+        streamCategoryIcon=folderIcon;
+    }
+    #endif
+}
+
+static bool monoSb=true;
+bool Icons::monoSidebarIcons()
+{
+    return monoSb;
+}
+
+void Icons::initSidebarIcons()
+{
+    if (Settings::self()->monoSidebarIcons()) {
+        monoSb=true;
+        QColor textCol=QApplication::palette().color(QPalette::Active, QPalette::WindowText);
+        QColor highlightedTexCol=QApplication::palette().color(QPalette::Active, QPalette::HighlightedText);
+        playqueueIcon=loadSidebarIcon(QLatin1String("playqueue"), textCol, highlightedTexCol);
+        artistsIcon=loadSidebarIcon(QLatin1String("artists"), textCol, highlightedTexCol);
+        albumsIcon=loadSidebarIcon(QLatin1String("albums"), textCol, highlightedTexCol);
+        foldersIcon=loadSidebarIcon(QLatin1String("folders"), textCol, highlightedTexCol);
+        playlistsIcon=loadSidebarIcon(QLatin1String("playlists"), textCol, highlightedTexCol);
+        #ifdef ENABLE_DYNAMIC
+        dynamicIcon=loadSidebarIcon(QLatin1String("dynamic"), textCol, highlightedTexCol);
+        #endif
+        #ifdef ENABLE_STREAMS
+        streamsIcon=loadSidebarIcon(QLatin1String("streams"), textCol, highlightedTexCol);
+        #endif
+        #ifdef ENABLE_ONLINE_SERVICES
+        onlineIcon=loadSidebarIcon(QLatin1String("online"), textCol, highlightedTexCol);
+        #endif
+        infoSidebarIcon=loadSidebarIcon(QLatin1String("info"), textCol, highlightedTexCol);
+        #ifdef ENABLE_DEVICES_SUPPORT
+        devicesIcon=loadSidebarIcon(QLatin1String("devices"), textCol, highlightedTexCol);
+        #endif
+        searchTabIcon=loadSidebarIcon(QLatin1String("search"), textCol, highlightedTexCol);
+    } else {
+        monoSb=false;
+        playqueueIcon=Icon("media-playback-start");
+        artistsIcon=artistIcon;
+        albumsIcon=albumIcon;
+        foldersIcon=Icon("inode-directory");
+        playlistsIcon=playlistIcon;
+        #ifdef ENABLE_DYNAMIC
+        dynamicIcon=dynamicRuleIcon;
+        #endif
+        #ifdef ENABLE_STREAMS
+        streamsIcon=radioStreamIcon;
+        #endif
+        #ifdef ENABLE_ONLINE_SERVICES
+        onlineIcon=Icon("applications-internet");
+        #endif
+        if (QLatin1String("gnome")==Icon::currentTheme().toLower()) {
+            QColor col=QApplication::palette().color(QPalette::Active, QPalette::WindowText);
+            infoSidebarIcon=loadSidebarIcon("info", col, col);
+        } else {
+            infoSidebarIcon=Icon("dialog-information");
+        }
+        #ifdef ENABLE_DEVICES_SUPPORT
+        devicesIcon=Icon("multimedia-player");
+        #endif
+        searchTabIcon=searchIcon;
+    }
+}
+
+void Icons::initToolbarIcons(const QColor &toolbarText)
+{
+    #ifdef USE_SYSTEM_MENU_ICON
+    Q_UNUSED(toolbarText)
+    #endif
+    QColor stdColor=calcIconColor();
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
+    if (GtkStyle::useSymbolicIcons()) {
+        bool rtl=QApplication::isRightToLeft();
+        QColor col=GtkStyle::symbolicColor();
+        toolbarPrevIcon=loadMediaIcon(QLatin1String(rtl ? "prev-rtl" : "prev"), col, col);
+        toolbarPlayIcon=loadMediaIcon(QLatin1String(rtl ? "play-rtl" : "play"), col, col);
+        toolbarPauseIcon=loadMediaIcon(QLatin1String("pause"), col, col);
+        toolbarStopIcon=loadMediaIcon(QLatin1String("stop"), col, col);
+        toolbarNextIcon=loadMediaIcon(QLatin1String(rtl ? "next-rtl" : "next"), col, col);
         infoIcon=loadSidebarIcon("info", col, col);
+        #ifdef USE_SYSTEM_MENU_ICON
+        toolbarMenuIcon=loadMonoSvgIcon(QLatin1String("menu"), QLatin1String("icon"), col, col);
+        menuIcon=loadMonoSvgIcon(QLatin1String("menu"), QLatin1String("icon"), stdColor, stdColor);
+        #else
+        if (col==stdColor) {
+            toolbarMenuIcon=menuIcon;
+        } else {
+            toolbarMenuIcon=createMenuIcon(col);
+        }
+        #endif
     } else
     #endif
+    {
+        #ifdef USE_SYSTEM_MENU_ICON
+        toolbarMenuIcon=menuIcon;
+        #else
+        if (toolbarText==stdColor) {
+            toolbarMenuIcon=menuIcon;
+        } else {
+            toolbarMenuIcon=createMenuIcon(toolbarText);
+        }
+        #endif
         if (QLatin1String("gnome")==Icon::currentTheme().toLower()) {
-            QColor col=QApplication::palette().color(QPalette::Active, QPalette::ButtonText);
+            QColor col=QApplication::palette().color(QPalette::Active, QPalette::WindowText);
             infoIcon=loadSidebarIcon("info", col, col);
         }
+    }
 
     if (infoIcon.isNull()) {
         infoIcon=Icon("dialog-information");
     }
 
-    #if !defined ENABLE_KDE_SUPPORT && !defined Q_OS_WIN
+    #if !defined ENABLE_KDE_SUPPORT && !defined Q_OS_WIN && !defined Q_OS_MAC
     if (QLatin1String("gnome")==Icon::currentTheme().toLower()) {
-        QColor col=QApplication::palette().color(QPalette::Active, QPalette::ButtonText);
+        QColor col=QApplication::palette().color(QPalette::Active, QPalette::WindowText);
         contextIcon=loadSidebarIcon("info", col, col);
     } else
     #endif
@@ -574,39 +588,17 @@ void Icons::initToolbarIcons(const QColor &color, bool forceLight)
 
     if (toolbarPrevIcon.isNull()) {
         toolbarPrevIcon=Icon::getMediaIcon("media-skip-backward");
-    } else {
-        setDisabledOpacity(toolbarPrevIcon);
     }
     if (toolbarPlayIcon.isNull()) {
         toolbarPlayIcon=Icon::getMediaIcon("media-playback-start");
-    } else {
-        setDisabledOpacity(toolbarPlayIcon);
     }
     if (toolbarPauseIcon.isNull()) {
         toolbarPauseIcon=Icon::getMediaIcon("media-playback-pause");
-    } else {
-        setDisabledOpacity(toolbarPauseIcon);
     }
     if (toolbarStopIcon.isNull()) {
         toolbarStopIcon=Icon::getMediaIcon("media-playback-stop");
-    } else {
-        setDisabledOpacity(toolbarStopIcon);
     }
     if (toolbarNextIcon.isNull()) {
         toolbarNextIcon=Icon::getMediaIcon("media-skip-forward");
-    } else {
-        setDisabledOpacity(toolbarNextIcon);
-    }
-    if (toolbarVolumeMutedIcon.isNull()) {
-        toolbarVolumeMutedIcon=Icon("audio-volume-muted");
-    }
-    if (toolbarVolumeLowIcon.isNull()) {
-        toolbarVolumeLowIcon=Icon("audio-volume-low");
-    }
-    if (toolbarVolumeMediumIcon.isNull()) {
-        toolbarVolumeMediumIcon=Icon("audio-volume-medium");
-    }
-    if (toolbarVolumeHighIcon.isNull()) {
-        toolbarVolumeHighIcon=Icon("audio-volume-high");
     }
 }
