@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,40 +24,46 @@
 #ifndef ITEMVIEW_H
 #define ITEMVIEW_H
 
+#include "config.h"
 #include "ui_itemview.h"
+#include "treeview.h"
 #include <QMap>
+#include <QList>
+#include <QPair>
 
 class ProxyModel;
 class Spinner;
-class QAction;
 class QTimer;
 class GroupedView;
 class ActionItemDelegate;
 class MessageOverlay;
+class Icon;
+class TableView;
 
-class ViewEventHandler : public QObject
+class KeyEventHandler : public QObject
+{
+    Q_OBJECT
+public:
+    KeyEventHandler(QAbstractItemView *v, QAction *a=0);
+    void setDeleteAction(QAction *a) { deleteAct=a; }
+Q_SIGNALS:
+    void backspacePressed();
+protected:
+    bool eventFilter(QObject *obj, QEvent *event);
+protected:
+    QAbstractItemView *view;
+    QAction *deleteAct;
+    bool interceptBackspace;
+};
+
+class ViewEventHandler : public KeyEventHandler
 {
 public:
     ViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v);
-
 protected:
     bool eventFilter(QObject *obj, QEvent *event);
-
-protected:
-    ActionItemDelegate *delegate;
-    QAbstractItemView *view;
-};
-
-class ListViewEventHandler : public ViewEventHandler
-{
-public:
-    ListViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v, QAction *a);
-
-protected:
-    bool eventFilter(QObject *obj, QEvent *event);
-
 private:
-    QAction *act;
+    ActionItemDelegate *delegate;
 };
 
 class ItemView : public QWidget, public Ui::ItemView
@@ -67,40 +73,35 @@ public:
 
     enum Mode
     {
+        Mode_BasicTree,
         Mode_SimpleTree,
         Mode_DetailedTree,
+
+        Mode_GroupedTree,
+
+        Mode_Table,
+
         Mode_List,
         Mode_IconTop,
-        Mode_GroupedTree,
 
         Mode_Count
     };
 
-    enum Role
-    {
-        Role_ImageSize = Qt::UserRole+256,
-        Role_MainText,
-        Role_SubText,
-        Role_TitleText,
-        Role_Image,
-        Role_Capacity,
-        Role_CapacityText,
-        Role_Actions
-    };
-
     static Mode toMode(const QString &str);
     static QString modeStr(Mode m);
-
     static void setup();
+    static const QLatin1String constSearchActiveKey;
 
     ItemView(QWidget *p=0);
     virtual ~ItemView();
 
+    void load(const QString &group);
+    void save(const QString &group);
     void allowGroupedView();
+    void allowTableView(TableView *v);
     void addAction(QAction *act);
     void setMode(Mode m);
     Mode viewMode() const { return mode; }
-    void setLevel(int level, bool haveChildren=true);
     QAbstractItemView * view() const;
     void setModel(ProxyModel *m);
     void clearSelection() { view()->selectionModel()->clearSelection(); }
@@ -108,6 +109,7 @@ public:
     void select(const QModelIndex &idx);
     QModelIndexList selectedIndexes(bool sorted=true) const;
     QString searchText() const;
+    QString searchCategory() const;
     void clearSearchText();
     void setUniformRowHeights(bool v);
     void setAcceptDrops(bool v);
@@ -119,26 +121,41 @@ public:
     void setDeleteAction(QAction *act);
     void setRootIsDecorated(bool v) { treeView->setRootIsDecorated(v); }
     void showIndex(const QModelIndex &idx, bool scrollTo);
-    void focusSearch();
     void setSearchLabelText(const QString &text);
     void setSearchVisible(bool v);
     bool isSearchActive() const;
     void setStartClosed(bool sc);
     bool isStartClosed();
     void expandAll(const QModelIndex &index=QModelIndex());
-    void expand(const QModelIndex &index);
+    void expand(const QModelIndex &index, bool singleOnly=false);
     void showMessage(const QString &message, int timeout);
     void setBackgroundImage(const QIcon &icon);
+    bool isAnimated() const;
+    void setAnimated(bool a);
+    void setPermanentSearch();
+    void setSearchCategories(const QList<QPair<QString, QString> > &categories);
+    void setSearchCategory(const QString &id);
+    void setSearchResetLevel(int l) { searchResetLevel=l; }
+    void setSearchIndex(const QModelIndex &idx) { searchIndex=idx; }
+    void showEvent(QShowEvent *ev);
+    void goToTop();
+
+private:
+    void setLevel(int level, bool haveChildren=true);
+    bool usingTreeView() const { return mode<=Mode_DetailedTree; }
+    bool usingListView() const { return mode>=Mode_List; }
 
 public Q_SLOTS:
+    void focusSearch(const QString &text=QString());
+    void focusView();
     void showSpinner(bool v=true);
     void hideSpinner();
     void collectionRemoved(quint32 key);
     void updateRows();
     void updateRows(const QModelIndex &idx);
     void backActivated();
-    void homeActivated();
     void setExpanded(const QModelIndex &idx, bool exp=true);
+    void closeSearch();
 
 Q_SIGNALS:
     void searchItems();
@@ -151,26 +168,32 @@ private Q_SLOTS:
     void itemClicked(const QModelIndex &index);
     void itemActivated(const QModelIndex &index);
     void delaySearchItems();
+    void doSearch();
+    void searchActive(bool a);
     void activateItem(const QModelIndex &index, bool emitRootSet=true);
 
 private:
+    void collapseToLevel();
     QAction * getAction(const QModelIndex &index);
 
 private:
     QTimer *searchTimer;
     ProxyModel *itemModel;
-    QAction *backAction;
-    QAction *homeAction;
     int currentLevel;
     Mode mode;
+    QString currentText;
     QMap<int, QString> prevText;
     QModelIndex prevTopIndex;
     QSize iconGridSize;
     QSize listGridSize;
     GroupedView *groupedView;
+    TableView *tableView;
     Spinner *spinner;
     MessageOverlay *msgOverlay;
     QIcon bgndIcon;
+    bool performedSearch;
+    int searchResetLevel;
+    QModelIndex searchIndex;
 };
 
 #endif
