@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,6 +26,7 @@
 #include "treeview.h"
 #include "config.h"
 #include "icons.h"
+#include "support/utils.h"
 #include <QMimeData>
 #include <QDrag>
 #include <QMouseEvent>
@@ -34,8 +35,9 @@
 #include <QPaintEvent>
 
 ListView::ListView(QWidget *parent)
-        : QListView(parent)
-        , menu(0)
+    : QListView(parent)
+    , eventFilter(0)
+    , menu(0)
 {
     setDragEnabled(true);
     setContextMenuPolicy(Qt::NoContextMenu);
@@ -44,6 +46,7 @@ ListView::ListView(QWidget *parent)
     setAlternatingRowColors(false);
     setUniformItemSizes(true);
     setAttribute(Qt::WA_MouseTracking);
+    setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     connect(this, SIGNAL(customContextMenuRequested(const QPoint &)), SLOT(showCustomContextMenu(const QPoint &)));
 }
 
@@ -63,43 +66,13 @@ void ListView::selectionChanged(const QItemSelection &selected, const QItemSelec
 bool ListView::haveSelectedItems() const
 {
     // Dont need the sorted type of 'selectedIndexes' here...
-    return selectionModel()->selectedIndexes().count()>0;
+    return selectionModel() && selectionModel()->selectedIndexes().count()>0;
 }
 
 bool ListView::haveUnSelectedItems() const
 {
     // Dont need the sorted type of 'selectedIndexes' here...
-    return selectionModel()->selectedIndexes().count()!=model()->rowCount();
-}
-
-void ListView::startDrag(Qt::DropActions supportedActions)
-{
-    QModelIndexList indexes = selectedIndexes();
-    if (indexes.count() > 0) {
-        QMimeData *data = model()->mimeData(indexes);
-        if (!data) {
-            return;
-        }
-        QDrag *drag = new QDrag(this);
-        drag->setMimeData(data);
-        QPixmap pix;
-
-        if (1==indexes.count()) {
-            QVariant var=model()->data(indexes.first(), ItemView::Role_Image);
-            QImage img=var.value<QImage>();
-            if (img.isNull()) {
-                pix=var.value<QPixmap>();
-            } else {
-                pix=QPixmap::fromImage(img);
-            }
-        }
-        if (pix.isNull()) {
-            drag->setPixmap(Icons::self()->albumIcon.pixmap(64, 64));
-        } else {
-            drag->setPixmap(pix.width()<64 ? pix : pix.scaled(QSize(64, 64), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-        }
-        drag->start(supportedActions);
-    }
+    return selectionModel() && selectionModel()->selectedIndexes().count()!=model()->rowCount();
 }
 
 void ListView::mouseReleaseEvent(QMouseEvent *event)
@@ -111,7 +84,7 @@ void ListView::mouseReleaseEvent(QMouseEvent *event)
 
 QModelIndexList ListView::selectedIndexes(bool sorted) const
 {
-    QModelIndexList indexes=selectionModel()->selectedIndexes();
+    QModelIndexList indexes=selectionModel() ? selectionModel()->selectedIndexes() : QModelIndexList();
     if (sorted) {
         qSort(indexes);
     }
@@ -165,6 +138,10 @@ void ListView::paintEvent(QPaintEvent *e)
 // Workaround for https://bugreports.qt-project.org/browse/QTBUG-18009
 void ListView::correctSelection()
 {
+    if (!selectionModel()) {
+        return;
+    }
+
     QItemSelection s = selectionModel()->selection();
     setCurrentIndex(currentIndex());
     selectionModel()->select(s, QItemSelectionModel::SelectCurrent);
