@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,6 +26,7 @@
 
 #include <QListView>
 #include <QPixmap>
+#include "treeview.h"
 
 class QIcon;
 
@@ -41,7 +42,7 @@ public:
     void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
     bool haveSelectedItems() const;
     bool haveUnSelectedItems() const;
-    void startDrag(Qt::DropActions supportedActions);
+    void startDrag(Qt::DropActions supportedActions) { TreeView::drag(supportedActions, this, selectedIndexes()); }
     void mouseReleaseEvent(QMouseEvent *event);
     QModelIndexList selectedIndexes() const { return selectedIndexes(false); }
     QModelIndexList selectedIndexes(bool sorted) const;
@@ -49,6 +50,8 @@ public:
     void addDefaultAction(QAction *act);
     void setBackgroundImage(const QIcon &icon);
     void paintEvent(QPaintEvent *e);
+    void installFilter(QObject *f) { eventFilter=f; installEventFilter(f); }
+    QObject * filter() const { return eventFilter; }
 
 private Q_SLOTS:
     void correctSelection();
@@ -58,6 +61,7 @@ Q_SIGNALS:
     bool itemsSelected(bool);
 
 private:
+    QObject *eventFilter;
     QMenu *menu;
     QPixmap bgnd;
 };
