@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,34 +22,61 @@
  */
 
 #include "toolbutton.h"
-#include "icon.h"
-#include "gtkstyle.h"
+#include "support/icon.h"
+#include "support/gtkstyle.h"
+#include "support/touchproxystyle.h"
+#include "config.h"
+#include "support/utils.h"
 #include <QMenu>
 #include <QStylePainter>
 #include <QStyleOptionToolButton>
 #include <QApplication>
+#include <QPainter>
 
 ToolButton::ToolButton(QWidget *parent)
     : QToolButton(parent)
+    #ifdef USE_SYSTEM_MENU_ICON
+    , hideMenuIndicator(GtkStyle::isActive())
+    #else
+    , hideMenuIndicator(true)
+    #endif
 {
     Icon::init(this);
-    setAutoRaise(true);
+    #ifdef Q_OS_MAC
+    setStyleSheet("QToolButton {border: 0}");
+    #endif
+    setFocusPolicy(Qt::NoFocus);
 }
 
 void ToolButton::paintEvent(QPaintEvent *e)
 {
+    #ifdef Q_OS_MAC
+    if (isDown() || isChecked()) {
+        QPainter p(this);
+        QColor col(palette().color(QPalette::WindowText));
+        QRect r(rect());
+        QPainterPath path=Utils::buildPath(QRectF(r.x()+1.5, r.y()+0.5, r.width()-2, r.height()-1), 2.5);
+        p.setRenderHint(QPainter::Antialiasing, true);
+        p.setPen(col);
+        p.drawPath(path);
+        col.setAlphaF(0.1);
+        p.fillPath(path, col);
+    }
+    #endif
     #if QT_VERSION > 0x050000
     // Hack to work-around Qt5 sometimes leaving toolbutton in 'raised' state.
     QStylePainter p(this);
     QStyleOptionToolButton opt;
     initStyleOption(&opt);
-    opt.features=QStyleOptionToolButton::None;
+    if (hideMenuIndicator) {
+        opt.features=QStyleOptionToolButton::None;
+    }
     if (opt.state&QStyle::State_MouseOver && this!=QApplication::widgetAt(QCursor::pos())) {
         opt.state&=~QStyle::State_MouseOver;
     }
     p.drawComplexControl(QStyle::CC_ToolButton, opt);
     #else
-    if (menu()) {
+    if (menu() && hideMenuIndicator) {
         QStylePainter p(this);
         QStyleOptionToolButton opt;
         initStyleOption(&opt);
@@ -63,6 +90,11 @@ void ToolButton::paintEvent(QPaintEvent *e)
 
 QSize ToolButton::sizeHint() const
 {
+    if (!hideMenuIndicator) {
+        QSize sz=QToolButton::sizeHint();
+        return Utils::touchFriendly() ? QSize(sz.width()*TouchProxyStyle::constScaleFactor, sz.height()) : sz;
+    }
+
     if (sh.isValid()) {
         return sh;
     }
@@ -77,7 +109,7 @@ QSize ToolButton::sizeHint() const
     } else {
         sh=QToolButton::sizeHint();
     }
-    sh=QSize(qMax(sh.width(), sh.height()), qMax(sh.width(), sh.height()));
+    sh=QSize(qMax(sh.width(), sh.height())*(Utils::touchFriendly() ? TouchProxyStyle::constScaleFactor : 1.0), qMax(sh.width(), sh.height()));
     return sh;
 }
 
