@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,10 +22,11 @@
  */
 
 #include "treeview.h"
-#include "itemview.h"
+#include "models/roles.h"
 #include "icons.h"
 #include "config.h"
 #include "basicitemdelegate.h"
+#include "support/utils.h"
 #include <QMouseEvent>
 #include <QPaintEvent>
 #include <QPainter>
@@ -34,18 +35,16 @@
 #include <QMap>
 #include <QStyle>
 #include <QList>
+#include <QApplication>
+#include <QHeaderView>
 
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobalSettings>
-#endif
-
-#ifdef ENABLE_KDE_SUPPORT
 #define SINGLE_CLICK KGlobalSettings::singleClick()
 #else
 #define SINGLE_CLICK style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, this)
 #endif
 
-
 QImage TreeView::setOpacity(const QImage &orig, double opacity)
 {
     QImage img=QImage::Format_ARGB32==orig.format() ? orig : orig.convertToFormat(QImage::Format_ARGB32);
@@ -88,8 +87,10 @@ bool TreeView::getForceSingleClick()
 }
 
 TreeView::TreeView(QWidget *parent, bool menuAlwaysAllowed)
-        : QTreeView(parent)
-        , alwaysAllowMenu(menuAlwaysAllowed)
+    : QTreeView(parent)
+    , eventFilter(0)
+    , forceSingleColumn(false)
+    , alwaysAllowMenu(menuAlwaysAllowed)
 {
     setDragEnabled(true);
     setContextMenuPolicy(Qt::NoContextMenu);
@@ -98,6 +99,7 @@ TreeView::TreeView(QWidget *parent, bool menuAlwaysAllowed)
     setAlternatingRowColors(false);
     setSelectionMode(QAbstractItemView::ExtendedSelection);
     setSelectionBehavior(QAbstractItemView::SelectRows);
+    setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     // Treeview does not seem to need WA_MouseTracking set, even with QGtkStyle items still
     // highlight under mouse. And enabling WA_MouseTracking here seems to cause drag-n-drop
     // errors if an item is dragged onto playqueue whilst playqueue has a selected item!
@@ -120,6 +122,7 @@ void TreeView::setPageDefaults()
     setDragDropMode(QAbstractItemView::DragOnly);
     setSortingEnabled(true);
     setAnimated(true);
+    forceSingleColumn=true;
 }
 
 void TreeView::setExpandOnClick()
@@ -141,29 +144,28 @@ void TreeView::selectionChanged(const QItemSelection &selected, const QItemSelec
 bool TreeView::haveSelectedItems() const
 {
     // Dont need the sorted type of 'selectedIndexes' here...
-    return selectionModel()->selectedIndexes().count()>0;
+    return selectionModel() && selectionModel()->selectedIndexes().count()>0;
 }
 
 bool TreeView::haveUnSelectedItems() const
 {
     // Dont need the sorted type of 'selectedIndexes' here...
-    return selectionModel()->selectedIndexes().count()!=model()->rowCount();
+    return selectionModel() && selectionModel()->selectedIndexes().count()!=model()->rowCount();
 }
 
-void TreeView::startDrag(Qt::DropActions supportedActions)
+void TreeView::drag(Qt::DropActions supportedActions, QAbstractItemView *view, const QModelIndexList &items)
 {
-    QModelIndexList indexes = selectedIndexes();
-    if (indexes.count() > 0) {
-        QMimeData *data = model()->mimeData(indexes);
+    if (items.count() > 0) {
+        QMimeData *data = view->model()->mimeData(items);
         if (!data) {
             return;
         }
-        QDrag *drag = new QDrag(this);
+        QDrag *drag = new QDrag(view);
         drag->setMimeData(data);
         QPixmap pix;
 
-        if (1==indexes.count()) {
-            QVariant var=model()->data(indexes.first(), ItemView::Role_Image);
+        if (1==items.count()) {
+            QVariant var=view->model()->data(items.first(), Cantata::Role_Image);
             QImage img=var.value<QImage>();
             if (img.isNull()) {
                 pix=var.value<QPixmap>();
@@ -171,10 +173,11 @@ void TreeView::startDrag(Qt::DropActions supportedActions)
                 pix=QPixmap::fromImage(img);
             }
         }
+        int pixSize=Icon::stdSize(Utils::scaleForDpi(32));
         if (pix.isNull()) {
-            drag->setPixmap(Icons::self()->albumIcon.pixmap(32, 32));
+            drag->setPixmap(Icons::self()->audioFileIcon.pixmap(pixSize, pixSize));
         } else {
-            drag->setPixmap(pix.width()<32 ? pix : pix.scaled(QSize(32, 32), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+            drag->setPixmap(pix.width()<pixSize ? pix : pix.scaled(QSize(pixSize, pixSize), Qt::KeepAspectRatio, Qt::SmoothTransformation));
         }
         drag->start(supportedActions);
     }
@@ -189,9 +192,13 @@ void TreeView::mouseReleaseEvent(QMouseEvent *event)
 
 QModelIndexList TreeView::selectedIndexes(bool sorted) const
 {
+    if (!selectionModel()) {
+        return QModelIndexList();
+    }
+
     if (sorted) {
         return sortIndexes(selectionModel()->selectedIndexes());
-    } else if (model()->columnCount()>1) {
+    } else if (model() && model()->columnCount()>1) {
         QModelIndexList list=selectionModel()->selectedIndexes();
         QModelIndexList sel;
         foreach (const QModelIndex &idx, list) {
@@ -262,21 +269,50 @@ QModelIndexList TreeView::sortIndexes(const QModelIndexList &list)
     return sorted;
 }
 
-void TreeView::expandAll(const QModelIndex &idx)
+void TreeView::expandAll(const QModelIndex &idx, bool singleLevelOnly)
 {
     quint32 count=model()->rowCount(idx);
     for (quint32 i=0; i<count; ++i) {
-        expand(model()->index(i, 0, idx));
+        expand(model()->index(i, 0, idx), singleLevelOnly);
     }
 }
 
-void TreeView::expand(const QModelIndex &idx)
+void TreeView::collapseToLevel(int level, const QModelIndex &idx)
+{
+    quint32 count=model()->rowCount(idx);
+    if (level) {
+        for (quint32 i=0; i<count; ++i) {
+            collapseToLevel(level-1, model()->index(i, 0, idx));
+        }
+    } else {
+        for (quint32 i=0; i<count; ++i) {
+            collapse(model()->index(i, 0, idx));
+        }
+    }
+}
+
+void TreeView::expand(const QModelIndex &idx, bool singleOnly)
 {
     if (idx.isValid()) {
         setExpanded(idx, true);
-        quint32 count=model()->rowCount(idx);
-        for (quint32 i=0; i<count; ++i) {
-            expand(idx.child(i, 0));
+        if (!singleOnly) {
+            quint32 count=model()->rowCount(idx);
+            for (quint32 i=0; i<count; ++i) {
+                expand(idx.child(i, 0));
+            }
+        }
+    }
+}
+
+void TreeView::collapse(const QModelIndex &idx, bool singleOnly)
+{
+    if (idx.isValid()) {
+        setExpanded(idx, false);
+        if (!singleOnly) {
+            quint32 count=model()->rowCount(idx);
+            for (quint32 i=0; i<count; ++i) {
+                collapse(idx.child(i, 0));
+            }
         }
     }
 }
@@ -322,6 +358,16 @@ void TreeView::setModel(QAbstractItemModel *m)
     QAbstractItemModel *old=model();
     QTreeView::setModel(m);
 
+    if (forceSingleColumn && m) {
+        int columnCount=m->columnCount();
+        if (columnCount>1) {
+            QHeaderView *hdr=header();
+            for (int i=1; i<columnCount; ++i) {
+                hdr->setSectionHidden(i, true);
+            }
+        }
+    }
+
     if (old) {
         disconnect(old, SIGNAL(layoutChanged()), this, SLOT(correctSelection()));
     }
@@ -334,17 +380,21 @@ void TreeView::setModel(QAbstractItemModel *m)
 // Workaround for https://bugreports.qt-project.org/browse/QTBUG-18009
 void TreeView::correctSelection()
 {
+    if (!selectionModel()) {
+        return;
+    }
+
     QItemSelection s = selectionModel()->selection();
     setCurrentIndex(currentIndex());
     selectionModel()->select(s, QItemSelectionModel::SelectCurrent);
 }
 
-void TreeView::itemWasActivated(const QModelIndex &index)
-{
-    if (!forceSingleClick) {
-        setExpanded(index, !isExpanded(index));
-    }
-}
+//void TreeView::itemWasActivated(const QModelIndex &index)
+//{
+//    if (!forceSingleClick) {
+//        setExpanded(index, !isExpanded(index));
+//    }
+//}
 
 void TreeView::itemWasClicked(const QModelIndex &index)
 {
